// TODO: rename the whole "projects" section to something like "editor" to reflect the logical separation between explorer/admin areas
angular.module('meshAdminUi.projects', [
    'meshAdminUi.projects.formBuilder',
    'meshAdminUi.projects.imageEditor',
    'meshAdminUi.common',
    'ui.router'
]);

var meshAdminUi;
(function (meshAdminUi) {
    /**
     *
     */
    var ProjectController = (function () {
        function ProjectController($scope, explorerContentsListService, editorService, deleteNodeDialog, dataService, contextService, wipService, dispatcher, nodeSelector, notifyService) {
            var _this = this;
            this.explorerContentsListService = explorerContentsListService;
            this.editorService = editorService;
            this.deleteNodeDialog = deleteNodeDialog;
            this.dataService = dataService;
            this.contextService = contextService;
            this.wipService = wipService;
            this.dispatcher = dispatcher;
            this.nodeSelector = nodeSelector;
            this.notifyService = notifyService;
            this.mobileEditView = false;
            this.contents = [];
            this.wipType = 'contents';
            this.projectName = contextService.getProject().name;
            this.createPermission = contextService.getCurrentNode().permissions.create;
            this.populateSchemas();
            this.populateTags();
            var nodeOpenedHandler = function () {
                _this.mobileEditView = true;
            };
            var nodeClosedHandler = function () {
                if (wipService.getOpenItems(_this.wipType).length === 0) {
                    _this.mobileEditView = false;
                }
            };
            dispatcher.subscribe(dispatcher.events.editorServiceNodeOpened, nodeOpenedHandler);
            dispatcher.subscribe(dispatcher.events.editorServiceNodeClosed, nodeClosedHandler);
            $scope.$on('$destroy', function () {
                dispatcher.unsubscribeAll(nodeClosedHandler, nodeOpenedHandler);
            });
        }
        ProjectController.$inject = ["$scope", "explorerContentsListService", "editorService", "deleteNodeDialog", "dataService", "contextService", "wipService", "dispatcher", "nodeSelector", "notifyService"];
        /**
         * Fill the vm with all available schemas.
         */
        ProjectController.prototype.populateSchemas = function () {
            var _this = this;
            this.dataService.getProjectSchemas(this.projectName)
                .then(function (result) { return _this.schemas = result.data; });
        };
        ProjectController.prototype.populateTags = function () {
            var _this = this;
            this.dataService.getProjectTags(this.projectName)
                .then(function (tags) { return _this.tags = tags; });
        };
        ProjectController.prototype.createNewNode = function (schemaUuid) {
            this.editorService.create(schemaUuid, this.contextService.getCurrentNode().uuid);
        };
        /**
         * Open the selected content items for editing (i.e. add them to the wipService open items)
         */
        ProjectController.prototype.openSelected = function () {
            var _this = this;
            this.explorerContentsListService.getSelection().forEach(function (uuid) { return _this.editorService.open(uuid); });
            this.explorerContentsListService.clearSelection();
        };
        /**
         * Show the node selector dialog and move selected nodes to that folder.
         */
        ProjectController.prototype.moveSelected = function () {
            var _this = this;
            var uuids = this.explorerContentsListService.getSelection();
            this.nodeSelector.open({
                containersOnly: true,
                includeRootNode: true,
                title: 'SELECT_DESTINATION',
                startingNodeUuid: this.contextService.getCurrentNode().parentNode.uuid
            })
                .then(function (selection) { return _this.dataService.moveNodes(_this.projectName, uuids, selection[0].uuid); })
                .then(function (movedUuids) { return _this.notifyService.toast('MOVED_NODES', { count: movedUuids.length }); })
                .then(function () { return _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged); })
                .catch(function (response) {
                if (response) {
                    _this.notifyService.toast(response.data.message);
                }
            });
        };
        /**
         * Delete the selected content items
         */
        ProjectController.prototype.deleteSelected = function () {
            var _this = this;
            var uuids = this.explorerContentsListService.getSelection();
            this.deleteNodeDialog.showMulti()
                .then(function (result) {
                var deleteAllLanguages = !!result.deleteAllLangs;
                return _this.dataService.deleteNodes(_this.projectName, uuids, deleteAllLanguages);
            })
                .then(function (deletedUuids) {
                deletedUuids.forEach(function (uuid) {
                    if (_this.wipService.isOpen(_this.wipType, uuid)) {
                        _this.wipService.closeItem(_this.wipType, { uuid: uuid });
                    }
                    if (_this.explorerContentsListService.isSelected(uuid)) {
                        _this.explorerContentsListService.toggleSelect(uuid);
                    }
                });
                _this.notifyService.toast('DELETED_NODES', { count: deletedUuids.length });
            })
                .then(function () { return _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged); });
        };
        ProjectController.prototype.nodesSelected = function () {
            return 0 < this.explorerContentsListService.getSelection().length;
        };
        ProjectController.prototype.getOpenWips = function () {
            return this.wipService.getOpenItems(this.wipType);
        };
        return ProjectController;
    }());
    angular.module('meshAdminUi.projects')
        .controller('projectController', ProjectController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    angular.module('meshAdminUi.projects')
        .config(routesConfig);
    function routesConfig($stateProvider) {
        $stateProvider
            .state('projects', {
            abstract: true,
            url: '',
            views: {
                'main': {
                    templateUrl: 'projects/projects.html',
                    controller: 'projectController',
                    controllerAs: 'vm'
                }
            },
            onEnter: function () {
                document.body.classList.add('project-module');
            },
            onExit: function () {
                document.body.classList.remove('project-module');
            }
        })
            .state('projects.node', {
            url: '/projects/:projectName/:nodeId',
            views: {
                'node-contents': {
                    templateUrl: "projects/projectExplorer/projectExplorer.html",
                    controller: 'ProjectExplorerController',
                    controllerAs: 'vm'
                }
            },
            resolve: {
                currentNode: updateContext
            }
        })
            .state('projectsList', {
            url: '/projects',
            views: {
                'main': {
                    templateUrl: 'projects/projectsList/projectsList.html',
                    controller: 'ProjectsListController',
                    controllerAs: 'vm'
                }
            },
            resolve: {
                parentNode: updateContext
            }
        });
    }
    routesConfig.$inject = ["$stateProvider"];
    /**
     * Update the context service with the current project. Used in the "resolve" property of the
     * state defs so that it will get invoked by any child states of a project on page load.
     */
    function updateContext($q, $stateParams, dataService, contextService) {
        var projectName = $stateParams.projectName || '', nodeId = $stateParams.nodeId, result;
        if (projectName !== '') {
            var qProject = dataService.getProjectByName(projectName), qNode = dataService.getNode(projectName, nodeId);
            result = $q.all([qProject, qNode])
                .then(function (result) {
                var project = result[0], node = result[1];
                contextService.setProject(project);
                contextService.setCurrentNode(node);
                return node;
            });
        }
        else {
            contextService.setProject();
            contextService.setCurrentNode();
        }
        return result;
    }
    updateContext.$inject = ["$q", "$stateParams", "dataService", "contextService"];
})(meshAdminUi || (meshAdminUi = {}));

angular.module('meshAdminUi.login', []);

var meshAdminUi;
(function (meshAdminUi) {
    var LoginController = (function () {
        function LoginController($timeout, $state, $mdDialog, authService) {
            this.$timeout = $timeout;
            this.$state = $state;
            this.$mdDialog = $mdDialog;
            this.authService = authService;
            this.success = false;
            this.loggingIn = false;
        }
        LoginController.$inject = ["$timeout", "$state", "$mdDialog", "authService"];
        LoginController.prototype.submitForm = function (event, userName, password) {
            var _this = this;
            this.loggingIn = true;
            // The user may already be "logged in" as a public user from viewing the front-end
            // (if that is on the same domain as the admin app), so we must explicitly log out
            // to ensure the user is not then logged in as the public uer.
            this.authService.logOut()
                .catch(function (err) { })
                .then(function () { return _this.authService.logIn(userName, password); })
                .then(function () {
                _this.success = true;
                _this.$timeout(function () { return _this.$state.go('projectsList'); }, 1500);
            })
                .catch(function () {
                _this.showErrorDialog();
                _this.loggingIn = false;
            });
        };
        /**
         * Displays an error message on login failure.
         */
        LoginController.prototype.showErrorDialog = function () {
            return this.$mdDialog.show({
                templateUrl: 'login/loginErrorDialog.html',
                controller: 'LoginDialogController',
                controllerAs: 'vm'
            });
        };
        return LoginController;
    }());
    var LoginDialogController = (function () {
        function LoginDialogController($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        LoginDialogController.$inject = ["$mdDialog"];
        LoginDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return LoginDialogController;
    }());
    angular.module('meshAdminUi.login')
        .controller('LoginDialogController', LoginDialogController)
        .controller('LoginController', LoginController);
})(meshAdminUi || (meshAdminUi = {}));

angular.module('meshAdminUi.common', [
    'meshAdminUi.common.i18n',
    'meshAdminUi.common.wipService',
    'ngCookies',
    'ngMaterial'
]);

angular.module('meshAdminUi.admin', [
    'meshAdminUi.common',
    'ui.router',
    'ui.ace'
]);

var meshAdminUi;
(function (meshAdminUi) {
    angular.module('meshAdminUi.admin')
        .config(routesConfig);
    function routesConfig($stateProvider) {
        $stateProvider
            .state('admin', {
            url: '/admin',
            abstract: true,
            data: {
                isAdminState: true
            }
        })
            .state('admin.users', {
            abstract: true
        })
            .state('admin.users.list', {
            url: '/users',
            views: {
                'main@': {
                    templateUrl: 'admin/users/list/userList.html',
                    controller: 'UserListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.users.detail', {
            url: '/users/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/users/detail/userDetail.html',
                    controller: 'UserDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.users.create', {
            url: '/users/new',
            views: {
                'main@': {
                    templateUrl: 'admin/users/detail/userDetail.html',
                    controller: 'UserDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.groups', {
            abstract: true
        })
            .state('admin.groups.list', {
            url: '/groups',
            views: {
                'main@': {
                    templateUrl: 'admin/groups/list/groupList.html',
                    controller: 'GroupListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.groups.detail', {
            url: '/groups/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/groups/detail/groupDetail.html',
                    controller: 'GroupDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.groups.create', {
            url: '/groups/new',
            views: {
                'main@': {
                    templateUrl: 'admin/groups/detail/groupDetail.html',
                    controller: 'GroupDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.roles', {
            abstract: true
        })
            .state('admin.roles.list', {
            url: '/roles',
            views: {
                'main@': {
                    templateUrl: 'admin/roles/list/roleList.html',
                    controller: 'RoleListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.roles.detail', {
            url: '/roles/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/roles/detail/roleDetail.html',
                    controller: 'RoleDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.roles.create', {
            url: '/roles/new',
            views: {
                'main@': {
                    templateUrl: 'admin/roles/detail/roleDetail.html',
                    controller: 'RoleDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.projects', {
            abstract: true
        })
            .state('admin.projects.list', {
            url: '/projects',
            views: {
                'main@': {
                    templateUrl: 'admin/projects/list/projectList.html',
                    controller: 'ProjectListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.projects.detail', {
            url: '/projects/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/projects/detail/projectDetail.html',
                    controller: 'ProjectDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.projects.create', {
            url: '/projects/new',
            views: {
                'main@': {
                    templateUrl: 'admin/projects/detail/projectDetail.html',
                    controller: 'ProjectDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.schemas', {
            abstract: true
        })
            .state('admin.schemas.list', {
            url: '/schemas',
            views: {
                'main@': {
                    templateUrl: 'admin/schemas/list/schemaList.html',
                    controller: 'SchemaListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.schemas.detail', {
            url: '/schemas/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/schemas/detail/schemaDetail.html',
                    controller: 'SchemaDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.schemas.create', {
            url: '/schemas/new',
            views: {
                'main@': {
                    templateUrl: 'admin/schemas/detail/schemaDetail.html',
                    controller: 'SchemaDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.microschemas', {
            abstract: true
        })
            .state('admin.microschemas.list', {
            url: '/microschemas',
            views: {
                'main@': {
                    templateUrl: 'admin/microschemas/list/microschemaList.html',
                    controller: 'MicroschemaListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.microschemas.detail', {
            url: '/microschemas/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/microschemas/detail/microschemaDetail.html',
                    controller: 'MicroschemaDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.microschemas.create', {
            url: '/microschemas/new',
            views: {
                'main@': {
                    templateUrl: 'admin/microschemas/detail/microschemaDetail.html',
                    controller: 'MicroschemaDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.tags', {
            abstract: true
        })
            .state('admin.tags.list', {
            url: '/tags',
            views: {
                'main@': {
                    templateUrl: 'admin/tags/list/tagList.html',
                    controller: 'TagListController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.tags.detail', {
            url: '/tags/:uuid',
            views: {
                'main@': {
                    templateUrl: 'admin/tags/detail/tagDetail.html',
                    controller: 'TagDetailController',
                    controllerAs: 'vm'
                }
            }
        })
            .state('admin.tags.create', {
            url: '/tags/new',
            views: {
                'main@': {
                    templateUrl: 'admin/tags/detail/tagDetail.html',
                    controller: 'TagDetailController',
                    controllerAs: 'vm'
                }
            }
        });
    }
    routesConfig.$inject = ["$stateProvider"];
})(meshAdminUi || (meshAdminUi = {}));

/**
 * Root module for the app.
 */
angular.module('meshAdminUi', [
    'meshAdminUi.templates',
    // app sub-modules
    'meshAdminUi.common',
    'meshAdminUi.login',
    'meshAdminUi.projects',
    'meshAdminUi.admin',
    // third-party modules
    'ui.router',
    'ngAnimate',
    'ngMaterial',
    'angularUtils.directives.dirPagination',
    'angular-loading-bar',
    'ngFileUpload',
    'dndLists'
]);

var meshAdminUi;
(function (meshAdminUi) {
    angular.module('meshAdminUi')
        .config(appConfig)
        .run(appRunBlock);
    /**
     * App-wide config settings.
     */
    function appConfig($stateProvider, $locationProvider, $urlRouterProvider, $mdThemingProvider, $animateProvider, dataServiceProvider, paginationTemplateProvider, cfpLoadingBarProvider) {
        configRoutes($stateProvider);
        $urlRouterProvider.otherwise('/projects');
        $locationProvider.hashPrefix('!');
        $animateProvider.classNameFilter(/animate/);
        $mdThemingProvider.theme('default')
            .primaryPalette('light-blue')
            .accentPalette('blue');
        dataServiceProvider.setApiUrl(meshUiConfig.apiUrl);
        paginationTemplateProvider.setPath('common/components/pagination/pagination.html');
        cfpLoadingBarProvider.latencyThreshold = 300;
        cfpLoadingBarProvider.includeSpinner = false;
    }
    appConfig.$inject = ["$stateProvider", "$locationProvider", "$urlRouterProvider", "$mdThemingProvider", "$animateProvider", "dataServiceProvider", "paginationTemplateProvider", "cfpLoadingBarProvider"];
    /**
     * Configuration of top-level routes
     * @param $stateProvider
     */
    function configRoutes($stateProvider) {
        $stateProvider
            .state('login', {
            url: '/login',
            views: {
                'main': {
                    templateUrl: 'login/login.html',
                    controller: 'LoginController',
                    controllerAs: 'vm'
                }
            }
        });
    }
    /**
     * Tasks to be run when the app is bootstrapped.
     */
    function appRunBlock($rootScope, $state, authService, dispatcher) {
        /**
         * Route unauthenticated users to the login page.
         */
        $rootScope.$on('$stateChangeStart', function (event, toState) {
            if (toState.name !== 'login' && !authService.isLoggedIn()) {
                event.preventDefault();
                $state.go('login');
            }
            else if (toState.name === 'login' && authService.isLoggedIn()) {
                event.preventDefault();
                $state.go('projectsList');
            }
        });
        // Polyfill for HTMLElement#remove()
        // https://developer.mozilla.org/en-US/docs/Web/API/ChildNode/remove#Polyfill
        if (!('remove' in Element.prototype)) {
            Element.prototype.remove = function () {
                if (this.parentNode) {
                    this.parentNode.removeChild(this);
                }
            };
        }
        /**
         * Register a callback to redirect to the login screen whenever the user gets
         * logged out.
         */
        dispatcher.subscribe(dispatcher.events.logoutSuccess, function () { return $state.go('login'); });
        /**
         * Dynamically insert the version here via the gulp-replace plugin during build.
         * The build task uses a regex to find and replace the comment below, so don't change it.
         */
        window.meshUiVersion = "0.6.4";
    }
    appRunBlock.$inject = ["$rootScope", "$state", "authService", "dispatcher"];
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var controls = meshUiConfig.microschemaControls;
    var controlsLocation = meshUiConfig.microschemaControlsLocation;
    /**
     * This function loads the .js and .css files associated with any custom microschema controls
     * defined in the meshUiConfig.microschemaControls array. The .js files should be AngularJS
     * directives which are then lazily registered with the app under the naming convention:
     *
     * "foo" -> "customControlFoo"
     */
    function customControlsResourceLoader($compileProvider) {
        var capitalizeFirst = function (str) { return str.charAt(0).toUpperCase() + str.slice(1); };
        if (controls) {
            var resources = controls.reduce(function (prev, curr) {
                var files = ['js', 'css'].map(function (ext) { return (controlsLocation + "/" + curr + "." + ext); });
                return prev.concat(files);
            }, []);
            head.load(resources, function () {
                // the following expression is overly-complex because the phantomjs karma test runner
                // does not like me trying to access the value of an undefined variable (meshMicroschemaControls)
                // and causes an error which stops the unit tests. Doing `typeof` on it works ok though.
                var customControls = typeof meshMicroschemaControls === 'undefined' ? {} : meshMicroschemaControls;
                for (var controlName in customControls) {
                    if (customControls.hasOwnProperty(controlName)) {
                        var name_1 = 'customControl' + capitalizeFirst(controlName);
                        var fn = customControls[controlName];
                        $compileProvider.directive(name_1, fn);
                    }
                }
            });
        }
    }
    customControlsResourceLoader.$inject = ["$compileProvider"];
    /**
     * Loads the template .html files for each registered microschema custom control via XHR and puts it
     * into the AngularJS template cache.
     */
    function customControlsTemplateLoader($templateRequest, $sce) {
        if (controls) {
            controls.map(function (name) { return (controlsLocation + "/" + name + ".html"); })
                .forEach(function (url) { return $templateRequest($sce.trustAsResourceUrl(url)); });
        }
    }
    customControlsTemplateLoader.$inject = ["$templateRequest", "$sce"];
    angular.module('meshAdminUi.projects.formBuilder', [])
        .config(customControlsResourceLoader)
        .run(customControlsTemplateLoader);
})(meshAdminUi || (meshAdminUi = {}));

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var meshAdminUi;
(function (meshAdminUi) {
    var DefaultControlController = (function () {
        function DefaultControlController($scope) {
            var _this = this;
            $scope.$watch(function () { return _this.fieldModel.value; }, function (val) {
                _this.value = angular.copy(val);
            });
        }
        DefaultControlController.$inject = ["$scope"];
        return DefaultControlController;
    }());
    function makeStandardDDO(type, controller) {
        var controllerName = controller || 'DefaultControlController';
        return {
            restrict: 'E',
            templateUrl: "projects/components/formBuilder/standardControls/" + type + "Control.html",
            controller: controllerName,
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                fieldModel: '='
            }
        };
    }
    /**
     * Input for string field types
     */
    function stringDirective() {
        return makeStandardDDO('string');
    }
    /**
     * Input for number field types
     */
    function numberDirective() {
        return makeStandardDDO('number');
    }
    /**
     * Since the input[type="date"] directive requires a Date object, we need to convert the
     * timestamp into a Date object and bind to that.
     */
    var DateController = (function () {
        function DateController($scope, $element) {
            var _this = this;
            this.$scope = $scope;
            this.$element = $element;
            this.focussed = '';
            this.time = { h: null, m: null, s: null };
            // this flag prevents the update callback from firing as soon as the
            // control loads and performs a mutation of the value to a Date object.
            var initialDateConversionDone = false;
            if (0 < this.fieldModel.value) {
                this.valueMoment = moment.unix(this.fieldModel.value);
            }
            else {
                // if the value is not yet set, default it to the current timestamp
                this.valueMoment = moment();
                this.fieldModel.update(this.valueMoment.unix());
            }
            this.value = this.valueMoment.toDate();
            this.setTimeFields(this.valueMoment);
            $scope.$watch(function () { return _this.value.valueOf(); }, function (newVal) {
                if (newVal) {
                    if (initialDateConversionDone) {
                        _this.valueMoment = moment(newVal);
                        _this.fieldModel.update(_this.valueMoment.unix());
                        _this.setTimeFields(_this.valueMoment);
                    }
                    else {
                        initialDateConversionDone = true;
                    }
                }
            });
        }
        DateController.$inject = ["$scope", "$element"];
        DateController.prototype.setTimeFields = function (currentMoment) {
            this.time = {
                h: currentMoment.format('HH'),
                m: currentMoment.format('mm'),
                s: currentMoment.format('ss')
            };
        };
        DateController.prototype.keydown = function (event) {
            // Number pad keyCodes do not work with String.fromCharCode(). See http://stackoverflow.com/a/4793913/772859
            var isNumPadNumber = function (keyCode) { return (96 <= event.which && event.which <= 105); };
            var input = event.target;
            var key = {
                leftArrow: 37,
                rightArrow: 39,
                upArrow: 38,
                downArrow: 40
            };
            var char = String.fromCharCode(event.which);
            var isNumeric = /[0-9]/.test(char) || isNumPadNumber(event.which);
            var isAlpha = /[a-zA-Z]/.test(char) && !isNumPadNumber(event.which);
            if (isAlpha && !isNumeric) {
                event.preventDefault();
                return;
            }
            if (event.which === key.upArrow) {
                this.increment(input);
                event.preventDefault();
            }
            else if (event.which === key.downArrow) {
                this.decrement(input);
                event.preventDefault();
            }
            if (input.selectionStart === input.value.length && event.which === key.rightArrow) {
                this.focusNext(input);
            }
            if (input.selectionEnd === 0 && event.which === key.leftArrow) {
                this.focusPrev(input);
            }
            if (input.selectionStart === 1) {
                if (input.value.length === 1 && isNumeric) {
                    this.focusNext(input);
                }
            }
        };
        DateController.prototype.updateValue = function (input) {
            var method;
            if (input.classList.contains('hour')) {
                method = "hour";
            }
            else if (input.classList.contains('minute')) {
                method = "minute";
            }
            else {
                method = "second";
            }
            this.valueMoment[method](input.value);
            this.value = this.valueMoment.toDate();
        };
        DateController.prototype.increment = function (input) {
            this.valueMoment.add(this.getIncrementAmount(input), 's');
            this.value = this.valueMoment.toDate();
        };
        DateController.prototype.decrement = function (input) {
            this.valueMoment.subtract(this.getIncrementAmount(input), 's');
            this.value = this.valueMoment.toDate();
        };
        DateController.prototype.getIncrementAmount = function (input) {
            var factors = {
                hour: 3600,
                minute: 60,
                second: 1
            };
            return factors[input.classList[0]];
        };
        DateController.prototype.focusFirst = function (event) {
            if (event.target.classList.contains('time-picker')) {
                var input = this.$element[0].querySelectorAll('.time-picker input')[0];
                input.focus();
            }
        };
        DateController.prototype.focus = function (event, focusTarget) {
            this.$element[0].querySelector('.time-picker').classList.add('focus');
            this.focussed = focusTarget;
            event.preventDefault();
        };
        DateController.prototype.blur = function (event) {
            this.$element[0].querySelector('.time-picker').classList.remove('focus');
            this.updateValue(event.target);
        };
        DateController.prototype.focusNext = function (input) {
            var nextElement = angular.element(input).next()[0];
            if (nextElement && nextElement.tagName === 'INPUT') {
                setTimeout(function () {
                    nextElement.focus();
                    nextElement.select();
                }, 0);
            }
        };
        DateController.prototype.focusPrev = function (input) {
            function prev(element) {
                var elm = element.previousSibling;
                while (elm != null && elm.nodeType !== 1) {
                    elm = elm.previousSibling;
                }
                return elm && elm.tagName === 'INPUT' ? elm : undefined;
            }
            var previousElement = prev(input);
            if (previousElement) {
                setTimeout(function () {
                    previousElement.focus();
                    previousElement.select();
                }, 0);
            }
        };
        return DateController;
    }());
    /**
     * Input for number field types
     */
    function dateDirective() {
        return makeStandardDDO('date', 'DateController');
    }
    /**
     * Input for boolean field types
     */
    function booleanDirective() {
        return makeStandardDDO('boolean');
    }
    /**
     * Input for select field types
     */
    function selectDirective() {
        return makeStandardDDO('select');
    }
    var NodeController = (function () {
        function NodeController(nodeSelector, i18nService, i18n, mu) {
            this.nodeSelector = nodeSelector;
            this.i18nService = i18nService;
            this.i18n = i18n;
            this.mu = mu;
        }
        NodeController.$inject = ["nodeSelector", "i18nService", "i18n", "mu"];
        NodeController.prototype.showDialog = function (event) {
            var _this = this;
            var displayMode = 'list';
            var options = {
                displayMode: displayMode,
                allow: this.fieldModel.allow || []
            };
            event.preventDefault();
            this.nodeSelector.open(options)
                .then(function (nodes) {
                var newVal = nodes[0];
                _this.fieldModel.value = newVal;
                _this.fieldModel.update(newVal);
            });
        };
        /**
         * Remove a node by setting it to null.
         * TODO: currently not working in Mesh, see https://jira.gentics.com/browse/CL-434 (comments)
         */
        NodeController.prototype.removeNode = function () {
            this.fieldModel.value = null;
            this.fieldModel.update(null);
        };
        /**
         * Returns a string representation of the selected node.
         */
        NodeController.prototype.getNodeName = function () {
            var node = this.fieldModel.value;
            var displayName = node && node.fields && node.fields[node.displayField];
            var uuid = node && node.uuid;
            var noneSelected = this.i18n('NO_NODE_SELECTED');
            return displayName || uuid || noneSelected;
        };
        NodeController.prototype.getBinaryUrl = function () {
            var linkedNode = this.fieldModel.value;
            var nodeBinaryField = this.mu.getFirstBinaryField(linkedNode);
            return this.mu.getBinaryFileUrl(this.fieldModel.projectName, linkedNode.uuid, this.i18nService.getCurrentLang().code, nodeBinaryField.key, nodeBinaryField.value.sha512sum);
        };
        NodeController.prototype.hasBinaryField = function () {
            var firstBinaryField = this.mu.getFirstBinaryField(this.fieldModel.value);
            return !!(firstBinaryField && firstBinaryField.value);
        };
        /**
         * Returns true if the value is valid according to the "required" property.
         */
        NodeController.prototype.valueIsValid = function () {
            if (this.fieldModel.required) {
                if (!this.fieldModel.value) {
                    return false;
                }
            }
            return true;
        };
        return NodeController;
    }());
    /**
     * Input for node field types
     */
    function nodeDirective() {
        return makeStandardDDO('node', 'NodeController');
    }
    var ListController = (function (_super) {
        __extends(ListController, _super);
        function ListController($scope, dataService, mu) {
            var _this = this;
            _super.call(this, $scope);
            this.$scope = $scope;
            this.dataService = dataService;
            this.mu = mu;
            this.listFieldModels = [];
            this.availableMicroschemas = [];
            /**
             * This watcher uses the expensive "deep object comparison" mode (third arg "true") in order to
             * ensure it fires the handler whenever a micronode field value changes. If a shallow comparison
             * (or $watchCollection) is used, changes to a micronode field value will not trigger an update
             * of the local fieldModel, leading to data being lost when re-ordering items.
             */
            $scope.$watch(function () { return _this.fieldModel.value; }, function (list) {
                if (list) {
                    _this.updateListFieldModels(list);
                }
            }, true);
            if (this.fieldModel.listType === 'micronode') {
                this.dataService.getProjectMicroschemas(this.fieldModel.projectName)
                    .then(function (result) {
                    var projectMicroschemas = result.data.map(function (microschema) { return microschema.name; });
                    _this.availableMicroschemas = _this.fieldModel.allow
                        .filter(function (name) { return -1 < projectMicroschemas.indexOf(name); });
                });
            }
        }
        ListController.$inject = ["$scope", "dataService", "mu"];
        ListController.prototype.updateListFieldModels = function (list) {
            var _this = this;
            if (typeof list !== 'undefined') {
                var type_1 = this.fieldModel.listType;
                this.listFieldModels.length = list.length;
                list.forEach(function (value, i) {
                    var target = _this.listFieldModels[i];
                    var exists = !!target;
                    var isPrimitiveValue = angular.isString(value) || angular.isNumber(value);
                    var micronodeIdsMatch = exists && _this.idsMatch(target.value, value);
                    if (exists && (micronodeIdsMatch || isPrimitiveValue)) {
                        // if this is the same micronode, then we just update the value. Likewise if it is a
                        // primitive value (number or string). This allows us to maintain the same object
                        // reference which means the ng-repeat can operate more efficiently and also
                        // mitigates issues to do with inputs losing focus.
                        _this.listFieldModels[i].value = value;
                    }
                    else {
                        _this.listFieldModels[i] = _this.createListItemFieldModel(type_1, value, i);
                    }
                });
            }
        };
        /**
         * Returns true if the uuid or tempId of the two micronodes match.
         */
        ListController.prototype.idsMatch = function (existingNode, newNode) {
            if (existingNode.uuid && newNode.uuid && existingNode.uuid === newNode.uuid) {
                return true;
            }
            if (existingNode.tempId && newNode.tempId && existingNode.tempId === newNode.tempId) {
                return true;
            }
            return false;
        };
        /**
         * Each item in the list needs its own NodeFieldModel object which can then be passed into the
         * Proxy and generate the sub-s.
         */
        ListController.prototype.createListItemFieldModel = function (type, value, index) {
            var path = angular.copy(this.fieldModel.path);
            path.push(index);
            var schemaDef = {
                type: type
            };
            if (value.microschema) {
                schemaDef.allow = [value.microschema.name];
            }
            var model = this.fieldModel.createChild(value, schemaDef, path);
            return model;
        };
        /**
         * Tracking function to be used in the ng-repeat of the list items. Primitive types should
         * be tracked by $index, whereas microschemas need to be tracked by reference to the
         * object itself in order for sorting to work correctly.
         */
        ListController.prototype.tracker = function (item, $id, $index) {
            return item.id;
        };
        /**
         * Add a new, empty microschema control to the list.
         */
        ListController.prototype.addControl = function (microschemaName) {
            var _this = this;
            this.dataService.getMicroschemaByName(microschemaName)
                .then(function (microschema) { return _this.createEmptyMicroschemaObject(microschema); })
                .then(function (newMicroschemaObject) {
                if (!_this.fieldModel.value) {
                    _this.fieldModel.value = [];
                }
                _this.fieldModel.value.push(newMicroschemaObject);
                _this.fieldModel.update(_this.fieldModel.value);
            });
        };
        /**
         * Add a new primitive-type item to the list
         */
        ListController.prototype.addItem = function () {
            var defaultValue = this.getDefaultValue(this.fieldModel.listType, this.fieldModel);
            if (!this.fieldModel.value) {
                this.fieldModel.value = [];
            }
            this.fieldModel.value.push(defaultValue);
            this.fieldModel.update(this.fieldModel.value);
        };
        /**
         * Remove the item at index from the list
         */
        ListController.prototype.removeItem = function (index) {
            this.fieldModel.value.splice(index, 1);
            this.fieldModel.update(this.fieldModel.value);
        };
        /**
         * Record the index of the item that is being dragged.
         */
        ListController.prototype.startDrag = function (index) {
            this.dragStartIndex = index;
        };
        /**
         * Remove the original dragged item from the list.
         */
        ListController.prototype.endDrag = function (item, dragEndIndex, list) {
            var indexToSplice;
            if (dragEndIndex < this.dragStartIndex) {
                indexToSplice = this.dragStartIndex + 1;
            }
            else {
                indexToSplice = this.dragStartIndex;
            }
            list.splice(dragEndIndex, 0, item.value); // add the new position
            list.splice(indexToSplice, 1); // remove the old position
            this.fieldModel.update(this.fieldModel.value);
        };
        /**
         * Create an empty microschema instance, populated with default values.
         */
        ListController.prototype.createEmptyMicroschemaObject = function (microschema) {
            var _this = this;
            var newMicroschemaObject = {
                "microschema": {
                    "name": microschema.name,
                    "uuid": microschema.uuid
                },
                "fields": {},
                "tempId": this.mu.generateGuid()
            };
            microschema.fields.forEach(function (fieldModel) {
                newMicroschemaObject.fields[fieldModel.name] = _this.getDefaultValue(fieldModel.type, fieldModel);
            });
            return newMicroschemaObject;
        };
        /**
         * Returns the correct default value for a field definition object.
         */
        ListController.prototype.getDefaultValue = function (type, fieldModel) {
            var defaultValue = null;
            if (typeof fieldModel.defaultValue !== 'undefined') {
                defaultValue = fieldModel.defaultValue;
            }
            else if (type === 'number') {
                if (typeof fieldModel.min !== 'undefined') {
                    if (typeof fieldModel.max !== 'undefined') {
                        defaultValue = Math.round((fieldModel.min + fieldModel.max) / 2);
                    }
                    else {
                        defaultValue = fieldModel.min;
                    }
                }
                else {
                    defaultValue = 0;
                }
            }
            else if (type === 'string' || type === 'html') {
                defaultValue = '';
            }
            else if (type === 'boolean') {
                defaultValue = false;
            }
            else if (type === 'date') {
                defaultValue = Math.floor(Date.now() / 1000);
            }
            else if (type === 'select') {
                defaultValue = fieldModel.options[0] || "";
            }
            else if (type === 'list') {
                defaultValue = [];
            }
            else if (type === 'node') {
                defaultValue = {};
            }
            return defaultValue;
        };
        return ListController;
    }(DefaultControlController));
    /**
     * Input for list field types
     */
    function listDirective() {
        return makeStandardDDO('list', 'ListController');
    }
    var BinaryController = (function () {
        function BinaryController($scope, i18nService, mu) {
            var _this = this;
            this.i18nService = i18nService;
            this.mu = mu;
            this.transform = {};
            $scope.$watch(function () { return _this.fieldModel.value; }, function (val) {
                _this.value = angular.copy(val);
            });
        }
        BinaryController.$inject = ["$scope", "i18nService", "mu"];
        BinaryController.prototype.getBinaryFileUrl = function () {
            if (this.fileToUpload && this.fileToUpload.hasOwnProperty('$ngfBlobUrl')) {
                return this.fileToUpload.$ngfBlobUrl;
            }
            else if (this.fieldModel.value && typeof this.fieldModel.value.sha512sum !== 'undefined') {
                return this.mu.getBinaryFileUrl(this.fieldModel.projectName, this.fieldModel.node.uuid, this.i18nService.getCurrentLang().code, this.fieldModel.name, this.fieldModel.value.sha512sum);
            }
            else {
                return '';
            }
        };
        /**
         * Update the transform parameters after the image has been edited in the edit dialog.
         */
        BinaryController.prototype.updateTransform = function (transform) {
            this.transform = transform;
            this.updateValue();
        };
        /**
         * A new binary file has been selected, so reset the transform params.
         */
        BinaryController.prototype.updateImageSource = function () {
            this.transform = {};
            this.updateValue();
        };
        BinaryController.prototype.updateValue = function () {
            var newValue = this.fileToUpload || this.fieldModel.value || {};
            newValue.transform = angular.copy(this.transform);
            this.fieldModel.update(newValue);
        };
        /**
         * Returns true if the value is valid according to the "required" property.
         */
        BinaryController.prototype.valueIsValid = function () {
            if (this.fieldModel.required) {
                return this.getBinaryFileUrl() !== '';
            }
            return true;
        };
        return BinaryController;
    }());
    function binaryDirective() {
        return makeStandardDDO('binary', 'BinaryController');
    }
    angular.module('meshAdminUi.projects.formBuilder')
        .controller('DefaultControlController', DefaultControlController)
        .controller('ListController', ListController)
        .controller('DateController', DateController)
        .controller('NodeController', NodeController)
        .controller('BinaryController', BinaryController)
        .directive('mhStringControl', stringDirective)
        .directive('mhNumberControl', numberDirective)
        .directive('mhBooleanControl', booleanDirective)
        .directive('mhDateControl', dateDirective)
        .directive('mhSelectControl', selectDirective)
        .directive('mhNodeControl', nodeDirective)
        .directive('mhListControl', listDirective)
        .directive('mhBinaryControl', binaryDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * This is the Aloha Editor implementation.
     */
    function htmlControlDirective(i18nService, nodeSelector, editorService, contextService, dataService) {
        // an array of functions that will be called when Aloha has loaded.
        var callbacks = [];
        var alohaPlugins = [];
        var defaultAlohaPlugins = [
            'common/autoparagraph',
            'common/contenthandler',
            'common/format',
            'common/highlighteditables',
            'common/list',
            'common/paste',
            'common/table',
            'common/ui'
        ];
        // we store a reference to the toolbar element as it is re-used by all
        // editable instances in the app, so we only need to get it once.
        var alohaToolbar;
        if (meshUiConfig.alohaPlugins && meshUiConfig.alohaPlugins instanceof Array && 0 < meshUiConfig.alohaPlugins.length) {
            alohaPlugins = meshUiConfig.alohaPlugins;
        }
        else {
            alohaPlugins = defaultAlohaPlugins;
        }
        alohaPlugins.push('mesh/mesh-link');
        var HtmlControlController = (function () {
            function HtmlControlController($scope, $element) {
                var _this = this;
                this.$scope = $scope;
                this.$element = $element;
                /**
                 * We need a way to let the control know when the inner htmlField (content editable div)
                 * is focused. We have no direct access to that information, so we need to set up
                 * event listeners on the native "focus" and "blur" events and use them to update
                 * the scope.
                 */
                this.isFocused = false;
                this.htmlField = $element[0].querySelector('.htmlField');
                this.toolbarContainer = this.$element[0].querySelector('.toolbar-container');
                this.lastContent = this.fieldModel.value;
                if (Aloha.ready) {
                    Aloha.ready(function () { return _this.registerBindings(); });
                }
                else {
                    callbacks.push(function () { return _this.registerBindings(); });
                }
                // initialize the content.
                this.syncView();
            }
            /**
             * Convert the path array into a string for use as a unique ID.
             */
            HtmlControlController.prototype.getPathName = function () {
                return this.fieldModel.path.join('_');
            };
            /**
             * Returns true if the value is valid according to the "required" property.
             */
            HtmlControlController.prototype.valueIsValid = function () {
                if (this.fieldModel.required) {
                    if (!this.lastContent || this.lastContent.trim() === '') {
                        return false;
                    }
                }
                return true;
            };
            /**
             * Pull out the Aloha toolbar from the top level of the DOM, and put it above the html input area so
             * we can style it to be the correct width.
             */
            HtmlControlController.prototype.getAlohaToolbarElement = function () {
                if (!alohaToolbar) {
                    alohaToolbar = document.querySelector('.aloha-ui-toolbar');
                    alohaToolbar.remove();
                }
                this.toolbarContainer.appendChild(alohaToolbar);
            };
            HtmlControlController.prototype.registerBindings = function () {
                var _this = this;
                var $ = Aloha.jQuery;
                var $editorPane = $('.editor-pane-container');
                var $toolbarContainer = $(this.toolbarContainer);
                /**
                 * This function controls how the Aloha toolbar gets positioned. It can be either stuck to the top
                 * edge of the editable area, or if the editable area is long and "goes off the top" of the editorPane,
                 * the Aloha toolbar then floats at the top. If the editable region is totally off the screen, we fade
                 * out the toolbar.
                 */
                var positionToolbarContainer = function () {
                    var $editable = $(_this.htmlField);
                    $toolbarContainer.css({ width: $editable.width() });
                    var toolbarHeight = Math.floor(alohaToolbar.getBoundingClientRect().height);
                    var containerHeight = 95 <= toolbarHeight ? toolbarHeight : 95;
                    var wipTabHeight = 33;
                    var labelHeight = _this.htmlField.previousElementSibling.offsetHeight + 1;
                    var controlOffsetTop = $editable.closest('.html-container')[0].getBoundingClientRect().top;
                    var paneTop = $editorPane[0].getBoundingClientRect().top;
                    var editableBottom = $editable[0].getBoundingClientRect().bottom;
                    var editableIsObscured = $editable.offset().top - containerHeight < paneTop;
                    var editableIsOffScreen = editableBottom < paneTop + containerHeight;
                    if (editableIsOffScreen) {
                        // toolbar is stuck to the bottom of the text area
                        $toolbarContainer.addClass('hidden');
                    }
                    else if (editableIsObscured) {
                        // toolbar is "floating"
                        $toolbarContainer.css({ top: wipTabHeight });
                        $toolbarContainer.removeClass('hidden');
                    }
                    else {
                        // toolbar is stuck to the top of the text area
                        $toolbarContainer.css({ top: controlOffsetTop - paneTop - containerHeight + wipTabHeight + labelHeight });
                        $toolbarContainer.removeClass('hidden');
                    }
                };
                var alohaActivated = function (jQueryEvent, alohaEditable) {
                    if (_this.eventTargetIsThisElement(alohaEditable)) {
                        _this.getAlohaToolbarElement();
                        alohaToolbar.classList.remove('hidden');
                        positionToolbarContainer();
                        $editorPane.on('scroll', positionToolbarContainer);
                        $(window).on('resize', positionToolbarContainer);
                        _this.$scope.$applyAsync(function () {
                            _this.isFocused = true;
                            if (_this.toolbarContainer) {
                                _this.toolbarContainer.classList.add('open');
                            }
                        });
                    }
                };
                var alohaDeactivated = function (jQueryEvent, alohaEditable) {
                    if (alohaEditable !== undefined && _this.eventTargetIsThisElement(alohaEditable)) {
                        $editorPane.off('scroll', positionToolbarContainer);
                        $(window).off('resize', positionToolbarContainer);
                        alohaToolbar.classList.add('hidden');
                        alohaToolbar.remove();
                        _this.$scope.$apply(function () {
                            // handle the highlighting of the element
                            _this.isFocused = false;
                            if (_this.toolbarContainer) {
                                _this.toolbarContainer.classList.remove('open');
                            }
                            /**
                             * View -> Model data binding
                             */
                            var currentContent = alohaEditable.editable.getContents();
                            if (currentContent !== _this.lastContent) {
                                _this.fieldModel.update(currentContent);
                                _this.lastContent = currentContent;
                            }
                        });
                    }
                };
                // in a setTimeout so that the editable "id" attribute has a chance to get
                // interpolated by Angular into the correct string value.
                setTimeout(function () { return $(_this.htmlField).aloha(); });
                $(Aloha).bind('aloha-editable-activated', alohaActivated);
                $(Aloha).bind('aloha-editable-deactivated', alohaDeactivated);
                /**
                 * Model -> data binding
                 */
                this.$scope.$watch(function () { return _this.fieldModel.value; }, function (val) {
                    if (typeof val !== 'undefined') {
                        _this.syncView();
                    }
                });
                this.$scope.$on('$destroy', function () {
                    $(Aloha).unbind('aloha-editable-activated', alohaActivated);
                    $(Aloha).unbind('aloha-editable-deactivated', alohaDeactivated);
                    $(_this.htmlField).mahalo();
                });
            };
            HtmlControlController.prototype.eventTargetIsThisElement = function (alohaEditable) {
                return this.htmlField.getAttribute('id') === alohaEditable.editable.getId();
            };
            HtmlControlController.prototype.syncView = function () {
                if (this.fieldModel.value) {
                    this.htmlField.innerHTML = this.fieldModel.value;
                }
            };
            HtmlControlController.$inject = ['$scope', '$element'];
            return HtmlControlController;
        }());
        function htmlControlCompileFn() {
            if (!window['Aloha']) {
                var activeSettings = void 0;
                var defaultAlohaSettings = {
                    "plugins": {
                        "formatlesspaste": {
                            "config": {
                                "strippedElements": ["a", "em", "strong", "small", "s", "cite", "q", "dfn", "abbr", "time", "code", "var", "samp", "kbd", "sub", "sup", "i", "b", "u", "mark", "ruby", "rt", "rp", "bdi", "bdo", "ins", "del"],
                                "button": "False",
                                "formatlessPasteOption": "1"
                            }
                        },
                        "block": {
                            "dragdrop": "1",
                            "dropzones": [".dropzone > div"],
                            "config": {
                                "toggleDragdropGlobal": "true"
                            }
                        },
                        "list": {
                            "templates": {
                                "ul": {
                                    "classes": []
                                },
                                "ol": {
                                    "classes": []
                                }
                            }
                        }
                    },
                    "logLevels": {
                        "error": true
                    },
                    "readonly": false,
                    "bundles": {
                        "custom": "/custom"
                    },
                    "contentHandler": {
                        "insertHtml": ["word", "generic", "oembed", "sanitize"],
                        "getContents": ["blockelement", "basic"],
                        "initEditable": ["blockelement"]
                    },
                    "sanitizeCharacters": {
                        "ï": "i",
                        "î": "i",
                        " ": "_",
                        "ë": "e",
                        "ê": "e",
                        "é": "e",
                        "è": "e",
                        "Ä": "Ae",
                        "ä": "ae",
                        "â": "a",
                        "à": "a",
                        "Ü": "Ue",
                        "ü": "ue",
                        "ß": "ss",
                        "û": "u",
                        "ù": "u",
                        "ö": "oe",
                        "Ö": "Oe",
                        "ô": "o"
                    }
                };
                var requiredAlohaSettings = {
                    "sidebar": {
                        "disabled": true
                    },
                    "floatingmenu": {
                        draggable: false
                    },
                    "plugins": {
                        "mesh-link": {
                            "selectNodeClick": function () {
                                return nodeSelector.open();
                            },
                            "resolveNodeName": function (uuid) {
                                return dataService.getNode(contextService.getProject().name, uuid);
                            },
                            "previewNodeClick": function (uuid) {
                                editorService.open(uuid);
                            },
                            "getLanguageCode": function () { return i18nService.getCurrentLang().code; }
                        }
                    },
                    "toolbar": {
                        "tabs": [{
                                "label": "tab.format.label"
                            }, {
                                "label": "tab.insert.label",
                                "components": [["gcnArena"]]
                            }, {
                                "label": "tab.link.label",
                                "components": ["selectNode", "selectedNode", "editLink", "removeLink"]
                            }]
                    },
                    "i18n": {
                        "current": 'en'
                    },
                    "locale": 'en'
                };
                if (meshUiConfig.alohaSettings && 0 < Object.keys(meshUiConfig.alohaSettings).length) {
                    activeSettings = angular.merge({}, meshUiConfig.alohaSettings, requiredAlohaSettings);
                }
                else {
                    activeSettings = angular.merge({}, defaultAlohaSettings, requiredAlohaSettings);
                }
                // configure Aloha editor
                window['Aloha'] = {};
                window['Aloha'].settings = activeSettings;
                loadScript('assets/vendor/aloha-editor/lib/aloha-full.js', initAloha);
            }
            // satisfy the TypeScript compiler.
            return {};
        }
        /**
         * Create the Aloha editable areas and run any callbacks that have
         * been registered.
         */
        function initAloha() {
            callbacks.forEach(function (fn) {
                fn();
            });
        }
        /**
         * this function will work cross-browser for loading scripts asynchronously
         * Based on http://stackoverflow.com/a/7719185/772859
         */
        function loadScript(src, callback) {
            var s, r, t;
            r = false;
            s = document.createElement('script');
            s.type = 'text/javascript';
            s.src = src;
            s.setAttribute('data-aloha-plugins', alohaPlugins.join(','));
            s.onload = s.onreadystatechange = function () {
                //console.log( this.readyState ); //uncomment this line to see which ready states are called.
                if (!r && (!this.readyState || this.readyState == 'complete')) {
                    r = true;
                    callback();
                }
            };
            t = document.getElementsByTagName('script')[0];
            t.parentNode.insertBefore(s, t);
        }
        return {
            restrict: 'E',
            compile: htmlControlCompileFn,
            templateUrl: 'projects/components/formBuilder/standardControls/htmlControl.html',
            controller: HtmlControlController,
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                fieldModel: '='
            }
        };
    }
    htmlControlDirective.$inject = ["i18nService", "nodeSelector", "editorService", "contextService", "dataService"];
    angular.module('meshAdminUi.projects.formBuilder')
        .directive('mhHtmlControl', htmlControlDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var NewTagDialogController = (function () {
        function NewTagDialogController(tagFamilies, tagName, $mdDialog) {
            this.tagFamilies = tagFamilies;
            this.tagName = tagName;
            this.$mdDialog = $mdDialog;
        }
        NewTagDialogController.$inject = ["tagFamilies", "tagName", "$mdDialog"];
        NewTagDialogController.prototype.querySearch = function (query) {
            if (query === '') {
                return this.tagFamilies;
            }
            else {
                return this.tagFamilies &&
                    this.tagFamilies.filter(function (tagFamilies) { return -1 < tagFamilies.name.indexOf(query); });
            }
        };
        NewTagDialogController.prototype.create = function () {
            this.$mdDialog.hide({
                tagName: this.tagName,
                tagFamilyRef: this.selectedTagFamily,
                newTagFamilyName: this.tagFamilySearch
            });
        };
        NewTagDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return NewTagDialogController;
    }());
    var TagsInputController = (function () {
        function TagsInputController($q, $mdDialog, $mdUtil, notifyService, dataService, contextService) {
            this.$q = $q;
            this.$mdDialog = $mdDialog;
            this.$mdUtil = $mdUtil;
            this.notifyService = notifyService;
            this.dataService = dataService;
            this.contextService = contextService;
            this.projectName = contextService.getProject().name;
            this.populateTags();
        }
        TagsInputController.$inject = ["$q", "$mdDialog", "$mdUtil", "notifyService", "dataService", "contextService"];
        /**
         * If the tags or tagFamilies have not been passed in, fetch them from the dataService.
         */
        TagsInputController.prototype.populateTags = function () {
            var _this = this;
            if (!this.tags) {
                this.dataService.getProjectTags(this.projectName)
                    .then(function (tags) { return _this.tags = tags; });
            }
            if (!this.tagFamilies) {
                this.dataService.getTagFamilies(this.projectName)
                    .then(function (result) { return _this.tagFamilies = result.data; });
            }
        };
        TagsInputController.prototype.querySearch = function (query) {
            if (query === '') {
                return this.tags;
            }
            else {
                var filterName = function (tag) { return -1 < tag.name.toLowerCase().indexOf(query.toLowerCase()); };
                return this.tags.filter(filterName);
            }
        };
        TagsInputController.prototype.newTagDialog = function (name) {
            var _this = this;
            this.$mdDialog.show({
                controller: 'NewTagDialogController',
                controllerAs: 'vm',
                bindToController: true,
                templateUrl: 'projects/components/tagsInput/newTagDialog.html',
                locals: {
                    tagFamilies: this.tagFamilies,
                    tagName: name
                }
            })
                .then(function (response) {
                _this.responseToTagFamilyPromise(response)
                    .then(function (tagFamily) {
                    var tag = _this.createEmptyTag(response.tagName, tagFamily);
                    return _this.dataService.persistTag(_this.projectName, tag);
                })
                    .then(function (tag) {
                    _this.tags.push(tag);
                    _this.addTag(tag);
                    _this.tagSearch = '';
                })
                    .catch(function (error) {
                    if (error && error.data && error.data.message) {
                        _this.notifyService.toast(error.data.message);
                    }
                });
            });
        };
        /**
         * Given a response object from the new tag dialog, this method figures out whether a new tag
         * family needs to be created, or whether an existing one should be used.
         *
         * In either case a promise is returned which resolves to a tag family node.
         */
        TagsInputController.prototype.responseToTagFamilyPromise = function (response) {
            var _this = this;
            var getTagFamily;
            if (!response.tagFamilyRef && response.newTagFamilyName) {
                var existingTagFamily = this.checkExistingTagFamily(response.newTagFamilyName);
                if (existingTagFamily) {
                    getTagFamily = this.$q.when(existingTagFamily);
                }
                else {
                    var tagFamily = this.createEmptyTagFamily(response.newTagFamilyName);
                    getTagFamily = this.dataService.persistTagFamily(this.projectName, tagFamily)
                        .then(function (tagFamily) {
                        _this.tagFamilies.push(tagFamily);
                        return tagFamily;
                    });
                }
            }
            else {
                getTagFamily = this.$q.when(response.tagFamilyRef);
            }
            return getTagFamily;
        };
        /**
         * Given a tag family name, checks the existing tag families to see if it matches an
         * existing tag family, and returns that tag family if so.
         */
        TagsInputController.prototype.checkExistingTagFamily = function (name) {
            var matches = this.tagFamilies.filter(function (tf) {
                return tf.name.toLowerCase() === name.toLowerCase();
            });
            if (0 < matches.length) {
                return matches[0];
            }
        };
        TagsInputController.prototype.addTag = function (tag) {
            var _this = this;
            this.onAdd({ tag: tag });
            this.tagSearch = '';
            // HACK ALERT: strange issue in Firefox where the div.md-scroll-mask element
            // created by the angular-material autocomplete component remains after the
            // tag has been selected. We will brute-force remove any lingering scroll
            // masks from the DOM.
            // See https://github.com/angular/material/issues/3287#issuecomment-192345556
            setTimeout(function () {
                _this.$mdUtil.enableScrolling();
            }, 50);
        };
        TagsInputController.prototype.createEmptyTagFamily = function (name) {
            return {
                name: name
            };
        };
        TagsInputController.prototype.createEmptyTag = function (name, tagFamily) {
            return {
                tagFamily: {
                    name: tagFamily.name,
                    uuid: tagFamily.uuid
                },
                name: name
            };
        };
        return TagsInputController;
    }());
    /**
     * The search/nav bar component which allows and contextual search of projects.
     */
    function tagsInputDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/tagsInput/tagsInput.html',
            controller: 'TagsInputController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                onAdd: '&',
                tags: '=',
                tagFamilies: '=?'
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('tagsInput', tagsInputDirective)
        .controller('NewTagDialogController', NewTagDialogController)
        .controller('TagsInputController', TagsInputController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * This class defines the behaviour of the pane resize control.
     */
    var ResizerController = (function () {
        function ResizerController($scope, $element) {
            var _this = this;
            this.$scope = $scope;
            this.$element = $element;
            this.resizerStyle = {
                'max-width': '',
                'min-width': ''
            };
            this.barWidth = 10;
            this.resizerMouseIsDown = false;
            this.handleStyle = {};
            var mouseUpHandler = function (event) {
                _this.resizerMouseup(event);
            };
            var mouseMoveHandler = function (event) {
                _this.resizerMousemove(event);
            };
            $element.on('mouseup', mouseUpHandler);
            $element.on('mousemove', mouseMoveHandler);
            $scope.$on('$destroy', function () {
                $element.off('mouseup', mouseUpHandler);
                $element.off('mousemove', mouseMoveHandler);
            });
        }
        ResizerController.$inject = ["$scope", "$element"];
        ResizerController.prototype.resizerMousedown = function (event, barWidth) {
            this.barWidth = barWidth;
            this.resizerMouseIsDown = true;
            event.preventDefault();
        };
        ResizerController.prototype.resizerMouseup = function (event) {
            var _this = this;
            this.resizerMouseIsDown = false;
            //this.$element.css({ position: 'relative' });
            this.$scope.$apply(function () {
                _this.resizerStyle = {
                    'max-width': _this.percentage + '%',
                    'min-width': _this.percentage + '%'
                };
            });
        };
        ResizerController.prototype.resizerMousemove = function (event) {
            var _this = this;
            if (this.resizerMouseIsDown) {
                var minWidth = 20, maxWidth = 80, container = event.currentTarget, mouseLeftOffset_1 = event.clientX - this.barWidth, containerWidth = container.offsetWidth, containerOffset = container.getBoundingClientRect().left, percentage = (mouseLeftOffset_1 - containerOffset) / containerWidth * 100;
                this.percentage = Math.max(Math.min(percentage, maxWidth), minWidth);
                // if the mouse has moved outside of the container, trigger the mouseup event.
                if (event.clientY < event.currentTarget.getBoundingClientRect().top) {
                    this.resizerMouseup(event);
                }
                if (minWidth < percentage && percentage < maxWidth) {
                    this.$scope.$apply(function () { return _this.handleStyle.left = _this.adjustOffsetLeft(mouseLeftOffset_1) + 'px'; });
                }
                event.preventDefault();
                return mouseLeftOffset_1;
            }
        };
        /**
         * Since the .projects-sliding-frame class includes the `transform` CSS property, it causes a new position
         * context to be created. This will throw off the calculation of the `left` position of the resize bar,
         * so we need to compensate for this.
         */
        ResizerController.prototype.adjustOffsetLeft = function (leftVal) {
            var positionContextElement = document.querySelector('.projects-sliding-frame');
            return leftVal - positionContextElement.offsetLeft;
        };
        return ResizerController;
    }());
    function resizerDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/resizer/resizer.html',
            require: '^resizerFrame',
            replace: true,
            scope: {},
            link: function (scope, element, attrs, ctrl) {
                scope.ctrl = ctrl;
                scope.mouseDown = function (event) {
                    var resizeBar = document.querySelector('.resize-bar'), barWidth = resizeBar.getBoundingClientRect().width;
                    ctrl.handleStyle.height = resizeBar.getBoundingClientRect().height + 'px';
                    ctrl.handleStyle.left = (resizeBar.offsetLeft - barWidth / 2) + 'px';
                    ctrl.resizerMousedown(event, barWidth);
                };
            }
        };
    }
    function resizerFrameDirective() {
        return {
            restrict: 'A',
            controller: 'resizerController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                resizerStyle: '='
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .controller('resizerController', ResizerController)
        .directive('resizer', resizerDirective)
        .directive('resizerFrame', resizerFrameDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ProjectSearchBarController = (function () {
        function ProjectSearchBarController($scope, i18n, mu, contextService, searchService, dispatcher) {
            var _this = this;
            this.i18n = i18n;
            this.mu = mu;
            this.contextService = contextService;
            this.searchService = searchService;
            this.dispatcher = dispatcher;
            this.selectedTags = [];
            this.searchTerm = '';
            this.searchAll = false;
            this.availableTags = this.availableTags || [];
            this.updateCurrentContext(contextService.getProject(), contextService.getCurrentNode());
            this.debouncedPublish = mu.debounce(function () { return _this.publishSearchParameters(); }, 250);
            var changeHandler = function (event, project, node) {
                _this.searchAll = false;
                _this.searchTerm = '';
                _this.publishSearchParameters();
                _this.updateCurrentContext(project, node);
            };
            dispatcher.subscribe(dispatcher.events.contextChanged, changeHandler);
            $scope.$on('$destroy', function () { return dispatcher.unsubscribeAll(changeHandler); });
            $scope.$watch(function () { return _this.searchAll; }, function (oldVal, newVal) {
                if (oldVal !== newVal) {
                    _this.publishSearchParameters();
                }
            });
        }
        ProjectSearchBarController.$inject = ["$scope", "i18n", "mu", "contextService", "searchService", "dispatcher"];
        ProjectSearchBarController.prototype.getPlaceholderText = function () {
            var text = this.i18n('SEARCH_IN') + ' ';
            if (this.searchAll) {
                text += this.i18n('WHOLE_PROJECT');
            }
            else {
                text += this.currentNode.fields[this.currentNode.displayField] || this.currentProject.name;
            }
            return text + ', # to filter by tag';
        };
        /**
         * Update the view model with the current context
         */
        ProjectSearchBarController.prototype.updateCurrentContext = function (currentProject, currentNode) {
            this.currentProject = currentProject;
            this.currentNode = currentNode;
        };
        ProjectSearchBarController.prototype.searchTextChanged = function (term) {
            if (!/^#/.test(term)) {
                this.debouncedPublish();
            }
        };
        ProjectSearchBarController.prototype.addTagFilter = function (tag) {
            if (!tag || !tag.uuid) {
                return;
            }
            if (this.selectedTags.indexOf(tag) === -1) {
                this.selectedTags.push(tag);
                this.searchTerm = '';
                this.publishSearchParameters();
            }
        };
        ProjectSearchBarController.prototype.removeTagFilter = function (tag) {
            var index = this.selectedTags.indexOf(tag);
            this.selectedTags.splice(index, 1);
            this.publishSearchParameters();
        };
        ProjectSearchBarController.prototype.publishSearchParameters = function () {
            var params = {
                searchTerm: this.searchTerm,
                tagFilters: this.selectedTags,
                searchAll: this.searchAll
            };
            this.searchService.setParams(params);
            this.dispatcher.publish(this.dispatcher.events.explorerSearchParamsChanged, params);
        };
        ProjectSearchBarController.prototype.getTagMatches = function (query) {
            if (/^#/.test(query)) {
                var tagQuery_1 = query.match(/^#(\S*)/)[1];
                if (tagQuery_1 === '') {
                    return this.availableTags;
                }
                else {
                    var filterName = function (tag) { return -1 < tag.name.toLowerCase().indexOf(tagQuery_1); };
                    return this.availableTags.filter(filterName);
                }
            }
        };
        return ProjectSearchBarController;
    }());
    /**
     * The search/nav bar component which allows and contextual search of projects.
     */
    function projectSearchBarDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/projectSearchBar/projectSearchBar.html',
            controller: 'projectSearchBarController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                availableTags: '='
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('projectSearchBar', projectSearchBarDirective)
        .controller('projectSearchBarController', ProjectSearchBarController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var PreviewSelectorController = (function () {
        function PreviewSelectorController($q, dataService, contextService) {
            this.$q = $q;
            this.dataService = dataService;
            this.contextService = contextService;
            this.previewUrls = [];
            this.linkRenderingMode = 'full';
            this.linkRenderingModes = ['off', 'full', 'medium', 'short'];
            if (this.canPreview()) {
                var projectName = contextService.getProject().name;
                for (var name_1 in meshUiConfig.previewUrls) {
                    if (name_1 === 'default' || name_1 === projectName) {
                        var urls = meshUiConfig.previewUrls[name_1].map(function (obj) {
                            var label = Object.keys(obj)[0];
                            var url = obj[label];
                            return { label: label, url: url };
                        });
                        this.previewUrls = this.previewUrls.concat(urls);
                    }
                }
                this.previewUrl = this.previewUrls[0].url;
            }
        }
        PreviewSelectorController.$inject = ["$q", "dataService", "contextService"];
        /**
         * Returns true if a previewUrl has been provided in the mesh-ui-config.js file.
         */
        PreviewSelectorController.prototype.canPreview = function () {
            var url = meshUiConfig.previewUrls;
            return !!(url && url !== '');
        };
        PreviewSelectorController.prototype.getPreviewUrls = function () {
            return this.previewUrls;
        };
        /**
         * If a previewUrl has been set in the mesh-ui-config.js file, this method can be used to open that URL
         * and POST the current node data to it, for the purpose of rendering a preview.
         */
        PreviewSelectorController.prototype.preview = function (previewUrl, linkRenderMode, node) {
            var _this = this;
            var stringFields = JSON.stringify(node.fields);
            var projectName = this.contextService.getProject().name;
            // We need to get the node with the `resolveLinks` param in order to
            // get the `languagePaths` property, which may be needed for the preview.
            var nodeResponse = this.dataService.getNode(projectName, node.uuid, { resolveLinks: linkRenderMode });
            var fieldsResponse = this.dataService.renderLinksInText(stringFields, linkRenderMode);
            this.$q.all([nodeResponse, fieldsResponse])
                .then(function (result) {
                var nodeClone = result[0];
                nodeClone.fields = result[1];
                _this.postNodeToUrl(nodeClone, previewUrl);
            });
        };
        /**
         * POST a serialized version of the node to the given URL, causing it to open in a new window.
         */
        PreviewSelectorController.prototype.postNodeToUrl = function (node, url) {
            var form = document.createElement("form");
            form.action = url;
            form.method = 'POST';
            form.target = '_blank';
            var input = document.createElement("textarea");
            input.name = 'node';
            input.value = JSON.stringify(node);
            form.appendChild(input);
            form.style.display = 'none';
            document.body.appendChild(form);
            form.submit();
            form.remove();
        };
        return PreviewSelectorController;
    }());
    function previewSelectorDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/previewSelector/previewSelector.html',
            controller: 'PreviewSelectorController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                node: '='
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('previewSelector', previewSelectorDirective)
        .controller('PreviewSelectorController', PreviewSelectorController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Displays a thumbnail representation of a node. If the node has any binary fields, the first one is used
     * as the thumbnail. If it is a container, it shows a folder icon.
     */
    var NodeThumbnailController = (function () {
        function NodeThumbnailController($scope, mu, contextService) {
            var _this = this;
            this.mu = mu;
            this.contextService = contextService;
            this.binaryProperties = {};
            var unwatch = $scope.$watch(function () { return _this.node; }, function (val) {
                if (val !== undefined) {
                    _this.checkForBinaryFields(val);
                    unwatch();
                }
            });
        }
        NodeThumbnailController.$inject = ["$scope", "mu", "contextService"];
        /**
         * Check if this node has any binary fields, get the first binary field if one exists
         * and populate the various component vars.
         */
        NodeThumbnailController.prototype.checkForBinaryFields = function (node) {
            var firstBinaryField = this.mu.getFirstBinaryField(node);
            if (firstBinaryField.key !== undefined) {
                this.hasBinaryField = true;
                this.binaryProperties.isImage = this.mu.isImageField(firstBinaryField.value);
                this.binaryProperties.fieldName = firstBinaryField.key;
                this.binaryProperties.binaryField = firstBinaryField.value;
            }
        };
        NodeThumbnailController.prototype.getFileUrl = function () {
            var projectName = this.contextService.getProject().name;
            var imageOptions = { width: 128 };
            return this.mu.getBinaryFileUrl(projectName, this.node.uuid, this.node.language, this.binaryProperties.fieldName, this.binaryProperties.binaryField.sha512sum, imageOptions);
        };
        NodeThumbnailController.prototype.getFileName = function () {
            if (this.hasBinaryField) {
                return this.binaryProperties.binaryField.fileName;
            }
            return '';
        };
        NodeThumbnailController.prototype.getFileExt = function () {
            var filename = this.getFileName();
            if (filename) {
                var parts = filename.split('.');
                return parts[parts.length - 1];
            }
            return '';
        };
        return NodeThumbnailController;
    }());
    function nodeThumbnailDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/nodeThumbnail/nodeThumbnail.html',
            controller: 'NodeThumbnailController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                node: '=',
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('nodeThumbnail', nodeThumbnailDirective)
        .controller('NodeThumbnailController', NodeThumbnailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * The nodeSelector service is used to configure and display a node selector modal dialog.
     */
    var NodeSelector = (function () {
        function NodeSelector($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        NodeSelector.$inject = ["$mdDialog"];
        /**
         * Opens the node selector dialog. Accepts an `options` object where the following
         * options may be configured:
         *
         * - allow: array, allowed schema types to filter by.
         * - title: string, title of the dialog box.
         * - multiSelect: bool, whether more than one node may be selected.
         * - displayMode: string, either 'list' or 'grid'
         */
        NodeSelector.prototype.open = function (options) {
            options = options || {};
            return this.$mdDialog.show({
                templateUrl: 'projects/components/nodeSelector/nodeSelectorDialog.html',
                controller: 'NodeSelectDialogController',
                controllerAs: 'dialog',
                locals: {
                    allow: options.allow || [],
                    containersOnly: options.containersOnly || false,
                    title: options.title || 'SELECT_NODE',
                    multiSelect: options.multiSelect || false,
                    displayMode: options.displayMode || 'list',
                    includeRootNode: !!options.includeRootNode,
                    startingNodeUuid: options.startingNodeUuid
                },
                bindToController: true
            });
        };
        return NodeSelector;
    }());
    meshAdminUi.NodeSelector = NodeSelector;
    var NodeSelectDialogController = (function () {
        function NodeSelectDialogController($mdDialog) {
            this.$mdDialog = $mdDialog;
            this.selectedNodes = [];
        }
        NodeSelectDialogController.$inject = ["$mdDialog"];
        NodeSelectDialogController.prototype.select = function () {
            this.$mdDialog.hide(this.selectedNodes);
        };
        NodeSelectDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return NodeSelectDialogController;
    }());
    var NodeSelectorController = (function () {
        function NodeSelectorController(dataService, i18nService, mu, contextService) {
            var _this = this;
            this.dataService = dataService;
            this.i18nService = i18nService;
            this.contextService = contextService;
            this.itemsPerPage = 20;
            this.currentPage = 1;
            this.selectedNodesHash = {};
            this.currentProject = contextService.getProject();
            this.currentNode = contextService.getCurrentNode();
            this.selectedNodes = this.selectedNodes instanceof Array ? this.selectedNodes : [];
            var initialUuid = this.startingNodeUuid || this.currentProject.rootNode.uuid;
            dataService.getNode(this.currentProject.name, initialUuid)
                .then(function (node) {
                if (_this.startingNodeUuid && node.parentNode) {
                    // if the startingNodeUuid was specified, we want to go up a level
                    // in the tree so that it can be selected.
                    return dataService.getNode(_this.currentProject.name, node.parentNode.uuid);
                }
                else {
                    return node;
                }
            })
                .then(function (node) { return _this.currentNode = node; })
                .then(function () { return _this.populateContents(); });
            this.filterNodes = function (node) {
                return mu.nodeFilterFn(node, _this.q);
            };
        }
        NodeSelectorController.$inject = ["dataService", "i18nService", "mu", "contextService"];
        NodeSelectorController.prototype.toggleSelect = function (node, event) {
            if (this.isAllowedSchema(node)) {
                if (this.isSelected(node)) {
                    this.deselectNode(node);
                }
                else {
                    this.selectNode(node);
                }
            }
            else {
                this.openNode(node, event);
            }
        };
        NodeSelectorController.prototype.selectNode = function (node) {
            if (!this.multiSelect) {
                this.selectedNodesHash = {};
                this.selectedNodes = [];
            }
            this.selectedNodesHash[node.uuid] = node;
            this.selectedNodes.push(node);
        };
        NodeSelectorController.prototype.deselectNode = function (node) {
            delete this.selectedNodesHash[node.uuid];
            this.selectedNodes = this.selectedNodes.filter(function (n) {
                return n.uuid !== node.uuid;
            });
        };
        NodeSelectorController.prototype.isSelected = function (node) {
            if (node && node.uuid) {
                return !!this.selectedNodesHash[node.uuid];
            }
            else {
                return false;
            }
        };
        NodeSelectorController.prototype.openNode = function (node, event) {
            var _this = this;
            if (node.container !== false) {
                event.stopPropagation();
                this.dataService.getNode(this.currentProject.name, node.uuid)
                    .then(function (node) {
                    _this.currentNode = node;
                    _this.populateContents();
                });
            }
        };
        NodeSelectorController.prototype.populateContents = function () {
            var _this = this;
            this.dataService.getChildNodes(this.currentProject.name, this.currentNode.uuid, { perPage: 9999999 })
                .then(function (response) {
                _this.nodes = response.data.filter(function (node) { return _this.isAllowedSchemaOrFolder(node); });
                if (_this.includeRootNode) {
                    _this.addRootNode();
                }
                return _this.dataService.getBreadcrumb(_this.currentProject, _this.currentNode);
            })
                .then(function (breadcrumbs) {
                var breadcrumbLabels = breadcrumbs.map(function (node) {
                    return {
                        name: node.fields[node.displayField],
                        uuid: node.uuid
                    };
                });
                breadcrumbLabels.push({
                    name: _this.currentProject.name,
                    uuid: _this.currentProject.rootNode.uuid
                });
                _this.breadcrumbs = breadcrumbLabels.reverse();
            });
        };
        NodeSelectorController.prototype.pageChanged = function (pageNumber) {
            this.currentPage = pageNumber;
        };
        /**
         * If we are in the rootNode, add it to the start of the nodes array.
         */
        NodeSelectorController.prototype.addRootNode = function () {
            if (this.currentProject.rootNode.uuid === this.currentNode.uuid) {
                var rootNode = angular.copy(this.currentNode);
                rootNode.fields['name'] = "Root Node";
                rootNode.displayField = 'name';
                rootNode.schema.name = "rootNode";
                this.nodes.unshift(rootNode);
            }
        };
        /**
         * Returns true if the node is not available in the current language
         */
        NodeSelectorController.prototype.isUntranslated = function (node) {
            return node.language !== this.i18nService.getCurrentLang().code;
        };
        NodeSelectorController.prototype.getDisplayName = function (node) {
            var displayName = node.fields[node.displayField];
            var langCode = node.language && node.language.toUpperCase();
            var langString = this.isUntranslated(node) && langCode ? " (" + langCode + ")" : '';
            return displayName + langString;
        };
        NodeSelectorController.prototype.isAllowedSchema = function (node) {
            if (0 < this.allowedSchemas.length) {
                return -1 < this.allowedSchemas.indexOf(node.schema.name);
            }
            else {
                return true;
            }
        };
        NodeSelectorController.prototype.isAllowedSchemaOrFolder = function (node) {
            if (!this.isAllowedSchema(node) && !node.container) {
                return false;
            }
            if (this.containersOnly) {
                return !!node.container;
            }
            return true;
        };
        return NodeSelectorController;
    }());
    function nodeSelectorDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/nodeSelector/nodeSelector.html',
            controller: 'nodeSelectorController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                selectedNodes: '=',
                allowedSchemas: '=',
                multiSelect: '=',
                displayMode: '=',
                containersOnly: '=',
                includeRootNode: '=',
                startingNodeUuid: '=',
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .service('nodeSelector', NodeSelector)
        .controller('NodeSelectDialogController', NodeSelectDialogController)
        .controller('nodeSelectorController', NodeSelectorController)
        .directive('nodeSelector', nodeSelectorDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var NodeLanguageMenuController = (function () {
        function NodeLanguageMenuController($scope, i18nService) {
            var _this = this;
            this.$scope = $scope;
            this.i18nService = i18nService;
            this.omitCurrentLanguage = false;
            this.showTranslateButton = true;
            this.unusedLangs = [];
            $scope.$watch(function () { return _this.node && _this.node.uuid; }, function (val) {
                if (val) {
                    _this.unusedLangs = _this.getUnusedLangs();
                }
            });
        }
        NodeLanguageMenuController.$inject = ["$scope", "i18nService"];
        NodeLanguageMenuController.prototype.getUnusedLangs = function () {
            var _this = this;
            var available = this.i18nService.getAvailableLanguages();
            if (!this.node.availableLanguages) {
                return [];
            }
            return available.filter(function (lang) {
                return _this.node.availableLanguages.indexOf(lang.code) == -1;
            });
        };
        NodeLanguageMenuController.prototype.getAvailableLanguages = function (node) {
            var _this = this;
            var langCodes;
            if (this.omitCurrentLanguage !== false && this.omitCurrentLanguage !== 'false') {
                langCodes = node.availableLanguages.filter(function (lang) { return lang !== node.language; });
            }
            else {
                langCodes = node.availableLanguages;
            }
            return langCodes.map(function (code) { return _this.i18nService.getLanguageInfo(code); });
        };
        NodeLanguageMenuController.prototype.translateClick = function (code) {
            this.onTranslateClick({ code: code });
        };
        NodeLanguageMenuController.prototype.goToTranslation = function (code) {
            this.onChangeLanguage({ code: code });
        };
        return NodeLanguageMenuController;
    }());
    function nodeLanguageMenuDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/nodeLanguageMenu/nodeLanguageMenu.html',
            controller: 'NodeLanguageMenuController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                node: '=',
                omitCurrentLanguage: '@',
                showTranslateButton: '@',
                onTranslateClick: '&',
                onChangeLanguage: '&'
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('nodeLanguageMenu', nodeLanguageMenuDirective)
        .controller('NodeLanguageMenuController', NodeLanguageMenuController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ImageEditorController = (function () {
        function ImageEditorController($mdDialog, $scope, $timeout, src, initialTransform) {
            var _this = this;
            this.$mdDialog = $mdDialog;
            this.$scope = $scope;
            this.src = src;
            this.initialTransform = initialTransform;
            this.transformParams = {};
            // used to hide the contents which the cropper is initializing to
            // prevent some layout thrashing.
            this.loaded = false;
            this.selectedTab = 'crop';
            var defaultParams = {
                src: this.src,
                width: 0,
                height: 0,
                cropx: 0,
                cropy: 0,
                cropw: 0,
                croph: 0,
                scale: 1
            };
            // if any transform params have been passed in via initialTransform, use them instead
            // of the default value.
            for (var key in defaultParams) {
                this.transformParams[key] = this.initialTransform[key] || defaultParams[key];
            }
            // ensure we always use an up-to-date source string.
            this.transformParams.src = src;
            $timeout(function () { return _this.loaded = true; }, 500);
        }
        ImageEditorController.$inject = ["$mdDialog", "$scope", "$timeout", "src", "initialTransform"];
        /**
         * Update the transform parameters. Should be invoked from one of the imageEditor "plugins" as a
         * callback when that plugin makes some alteration to the params. The newParams argument need not be a
         * complete IImageTransformParams object - it can specify only a new scale, for example, and only
         * that property will get updated.
         */
        ImageEditorController.prototype.updateTransformParams = function (newParams) {
            for (var param in newParams) {
                if (newParams.hasOwnProperty(param) && this.transformParams.hasOwnProperty(param)) {
                    this.transformParams[param] = newParams[param];
                }
            }
        };
        /**
         * The "reCalcViewDimensions" event is required by the rzSlider directive to
         * ensure the resize slider has the correct width. Since the slider is being created inside some dynamic DOM,
         * it is initially sized incorrectly. Therefore we need to manually trigger this event in order to force a
         * recalculation of its dimensions.
         */
        ImageEditorController.prototype.resizeTabSelected = function () {
            this.$scope.$broadcast('reCalcViewDimensions');
            this.selectedTab = 'resize';
        };
        /**
         * Close the modal and return the transform params.
         */
        ImageEditorController.prototype.save = function () {
            this.$mdDialog.hide(this.transformParams);
        };
        /**
         * Cancel the modal and return nothing.
         */
        ImageEditorController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return ImageEditorController;
    }());
    /**
     * This controller is responsible for creating and opening the modal window which
     * contains the image editor component.
     */
    var EditableImageController = (function () {
        function EditableImageController($timeout, $scope, $element, $mdDialog) {
            var _this = this;
            this.$mdDialog = $mdDialog;
            this.offsetRight = -500;
            // since the edit image button has varying width depending on the 
            // length of the word "edit" in different languages, we need to set 
            // the right offset dynamically.
            var unwatch = $scope.$watch(function () { return _this.src; }, function (val) {
                if (val) {
                    $timeout(function () {
                        var el = $element[0].querySelector('.edit-image-button');
                        var iconWidth = 35;
                        _this.offsetRight = iconWidth - el.offsetWidth;
                    });
                    unwatch();
                }
            });
        }
        EditableImageController.$inject = ["$timeout", "$scope", "$element", "$mdDialog"];
        EditableImageController.prototype.editImage = function () {
            var _this = this;
            return this.$mdDialog.show({
                templateUrl: 'projects/components/imageEditor/imageEditor.html',
                controller: 'ImageEditorController',
                controllerAs: 'vm',
                locals: {
                    src: this.src,
                    initialTransform: this.initialTransform,
                },
                bindToController: true
            })
                .then(function (result) {
                _this.onEdit({ transform: result });
            });
        };
        return EditableImageController;
    }());
    /**
     * Creates an overlay button which adds a link to open the image editor modal.
     *
     * Accepts the following attributes:
     * src: The source URL of the image to be edited.
     * initialTransform: The initial transform to apply to the image.
     * onEdit: A function that will be invoked with a `transform` argument when an image is edited.
     */
    function editableImageDirective() {
        return {
            restrict: 'E',
            template: "<div class=\"editable-image-container\">\n                            <div class=\"edit-image-button\" \n                                ng-style=\"{ right: vm.offsetRight + 'px' }\"\n                                ng-click=\"vm.editImage()\" \n                                ng-if=\"vm.src !== ''\">\n                                <i class=\"icon-crop\"></i> <span translate>EDIT</span>\n                            </div>\n                            <ng-transclude></ng-transclude>\n                       </div>",
            controller: 'EditableImageController',
            controllerAs: 'vm',
            bindToController: true,
            transclude: true,
            scope: {
                src: '=',
                initialTransform: '=',
                onEdit: '&'
            }
        };
    }
    angular.module('meshAdminUi.projects.imageEditor', ['rzModule'])
        .directive('editableImage', editableImageDirective)
        .controller('EditableImageController', EditableImageController)
        .controller('ImageEditorController', ImageEditorController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ImageEditorResizerController = (function () {
        function ImageEditorResizerController($scope) {
            var _this = this;
            this.$scope = $scope;
            this.scale = 1;
            this.resizeImageStyle = {};
            this.resizeContainerStyle = {};
            this.resizePos = { x: 0, y: 0 };
            /**
             * Event handler to be invoked when user starts dragging the preview container.
             */
            this.startMove = function (event) {
                event.preventDefault();
                _this.ox = _this.getClientX(event);
                _this.oy = _this.getClientY(event);
                _this.$previewContainer.on('mousemove touchmove', _this.move);
                _this.$previewContainer.on('mouseup touchend mouseleave', _this.endMove);
            };
            /**
             * Event handler to be invoked on dragging the preview container.
             */
            this.move = function (event) {
                event.preventDefault();
                _this.dx = _this.getClientX(event) - _this.ox;
                _this.dy = _this.getClientY(event) - _this.oy;
                _this.$scope.$apply(function () {
                    _this.resizeContainerStyle.transform = _this.makeTransformString(_this.dx, _this.dy);
                });
            };
            /**
             * Event handler to be invoked once dragging has ended.
             */
            this.endMove = function (event) {
                event.preventDefault();
                _this.resizePos.x += isNaN(_this.dx) ? 0 : _this.dx;
                _this.resizePos.y += isNaN(_this.dy) ? 0 : _this.dy;
                _this.$previewContainer.off('mousemove touchmove', _this.move);
                _this.$previewContainer.off('mouseup touchend mouseleave', _this.endMove);
            };
            this.initResizePreview();
            this.sliderOptions = {
                floor: 0.1,
                ceil: 2,
                step: 0.01,
                precision: 2,
                onChange: function () {
                    _this.onResize({ params: { scale: _this.scale } });
                }
            };
            this.scale = this.params.scale;
            $scope.$watchCollection(function () { return _this.params; }, function () { return _this.updateScaleDimensions(); });
            // remove the drag event handler when this scope is destroyed.
            $scope.$on('$destroy', function () { return _this.$previewContainer.off('mousedown touchstart', _this.startMove); });
        }
        ImageEditorResizerController.$inject = ["$scope"];
        /**
         * Position the resize preview container in the center, and register the event handlers for moving
         * it around. The setTimeout is needed to ensure that the DOM has been created before attempting
         * to select elements in it, which otherwise would not yet have been created.
         */
        ImageEditorResizerController.prototype.initResizePreview = function () {
            var _this = this;
            setTimeout(function () {
                _this.$previewContainer = angular.element(document.querySelector('.resize-area .resize-container'));
                _this.resizePos.x = window.innerWidth / 2;
                _this.resizePos.y = document.querySelector('.resize-area').offsetHeight / 2;
                _this.resizeContainerStyle.transform = _this.makeTransformString(0, 0);
                _this.$previewContainer.on('mousedown touchstart', _this.startMove);
            });
        };
        /**
         * Create a string representing the CSS "transform" property for positioning the
         * scale preview container.
         * @param dx - an x delta to offset the position by
         * @param dy - a y delta to offset the position by
         */
        ImageEditorResizerController.prototype.makeTransformString = function (dx, dy) {
            if (dx === void 0) { dx = 0; }
            if (dy === void 0) { dy = 0; }
            var x = dx + this.resizePos.x - this.params.cropw * this.scale / 2;
            var y = dy + this.resizePos.y - this.params.croph * this.scale / 2;
            return "translate3d(" + x + "px, " + y + "px, 0)";
        };
        /**
         * Set the scale back to 1.
         */
        ImageEditorResizerController.prototype.resetScale = function () {
            this.scale = 1;
            this.updateScaleDimensions();
        };
        /**
         * Set the scale based on a change to the width value
         */
        ImageEditorResizerController.prototype.changeScaleWidth = function () {
            this.scale = this.scaleWidth / this.params.cropw;
            this.updateScaleDimensions();
        };
        /**
         * Set the scale based on a change to the height value
         */
        ImageEditorResizerController.prototype.changeScaleHeight = function () {
            this.scale = this.scaleHeight / this.params.croph;
            this.updateScaleDimensions();
        };
        /**
         * Calculate and update the scaled dimensions of the image.
         */
        ImageEditorResizerController.prototype.updateScaleDimensions = function () {
            this.onResize({ params: { scale: this.scale } });
            this.scaleWidth = Math.round(this.params.cropw * this.scale);
            this.scaleHeight = Math.round(this.params.croph * this.scale);
            this.setImageResizeStyles();
        };
        /**
         * Calculate and set values for the objects which are bound to the resize preview DOM CSS.
         */
        ImageEditorResizerController.prototype.setImageResizeStyles = function () {
            var _this = this;
            var scale = function (val) { return val * _this.scale; };
            this.resizeImageStyle = {
                width: scale(this.params.width) + 'px',
                height: scale(this.params.height) + 'px',
                'margin-left': -scale(this.params.cropx) + 'px',
                'margin-top': -scale(this.params.cropy) + 'px'
            };
            this.resizeContainerStyle = {
                width: scale(this.params.cropw) + 'px',
                height: scale(this.params.croph) + 'px',
                transform: this.makeTransformString()
            };
        };
        /**
         * Get the "clientX" property from a Mouse/Touch event.
         */
        ImageEditorResizerController.prototype.getClientX = function (event) {
            return this.getEventProp(event, 'clientX');
        };
        /**
         * Get the "clientY" property from a Mouse.Touch event.
         */
        ImageEditorResizerController.prototype.getClientY = function (event) {
            return this.getEventProp(event, 'clientY');
        };
        /**
         * Get the named property from an event, accounting for differences between a MouseEvent and a
         * TouchEvent in different browsers. E.g. in Chrome, a TouchEvent exposes a "touches" array containing
         * the properties of each touch.
         */
        ImageEditorResizerController.prototype.getEventProp = function (event, prop) {
            var isTouchEvent = typeof TouchEvent !== 'undefined' && event instanceof TouchEvent;
            return isTouchEvent ? event.touches[0][prop] : event[prop];
        };
        return ImageEditorResizerController;
    }());
    function imageEditorResizerDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/imageEditor/imageEditorResizer.html',
            controller: 'ImageEditorResizerController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                params: '=',
                onResize: '&'
            }
        };
    }
    angular.module('meshAdminUi.projects.imageEditor')
        .directive('imageEditorResizer', imageEditorResizerDirective)
        .controller('ImageEditorResizerController', ImageEditorResizerController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ImageEditorCropperController = (function () {
        function ImageEditorCropperController($scope) {
            var _this = this;
            this.cropperOptions = {
                aspectRatio: 'free'
            };
            this.initCropper();
            $scope.$on('$destroy', function () { return _this.cropper.destroy(); });
        }
        ImageEditorCropperController.$inject = ["$scope"];
        /**
         * Initialize the Cropper plugin. A setTimeout is required to allow Angular to
         * create and append the DOM nodes needed to init the cropper.
         */
        ImageEditorCropperController.prototype.initCropper = function () {
            var _this = this;
            setTimeout(function () {
                var image = document.querySelector('#mesh-image-editor-subject');
                _this.cropper = new Cropper(image, {
                    dragMode: 'move',
                    strict: true,
                    toggleDragModeOnDblclick: false,
                    rotatable: false,
                    autoCropArea: 1,
                    built: function () {
                        // set the image natural dimensions
                        var imageData = _this.cropper.getImageData();
                        var params = {
                            width: imageData.naturalWidth,
                            height: imageData.naturalHeight,
                        };
                        _this.onCrop({ params: params });
                        if (_this.params.cropw && _this.params.croph) {
                            // The transform params were pre-specified via the intialTransform attribute on the
                            // image editor component.
                            _this.cropper.setData({
                                y: _this.params.cropy,
                                x: _this.params.cropx,
                                width: _this.params.cropw,
                                height: _this.params.croph
                            });
                        }
                        else {
                            _this.setAspectRatioAndFitToImage(null);
                            _this.updateParams();
                        }
                    },
                    cropend: function () { return _this.updateParams(); }
                });
            });
        };
        /**
         * Set the Cropper aspect ratio and ensure that the crop area is not taking
         * up the entire container.
         */
        ImageEditorCropperController.prototype.setAspectRatioAndFitToImage = function (ratio) {
            // setting aspect ratio to null forces the crop area to match image dimensions
            this.cropper.setAspectRatio(ratio);
            var cropBox = this.cropper.getCropBoxData();
            var PAD = 5;
            if (cropBox.left === 0 && cropBox.top === 0) {
                // when the cropBox completely fills the container, it is hard to see
                // and interact with. In this case, we make it a bit smaller.
                var container = this.cropper.getContainerData();
                this.cropper.setCropBoxData({
                    top: PAD,
                    left: PAD,
                    width: container.width - 2 * PAD,
                    height: container.height - 2 * PAD
                });
            }
        };
        /**
         * Invoke the onCrop callback, passing the current crop parameters.
         */
        ImageEditorCropperController.prototype.updateParams = function () {
            var data = this.cropper.getData();
            var params = {
                cropx: data.x,
                cropy: data.y,
                cropw: data.width,
                croph: data.height
            };
            this.onCrop({ params: params });
        };
        /**
         * Set the aspect ratio of the crop box.
         * @param mode - either 'original', 'square', or 'free'
         */
        ImageEditorCropperController.prototype.setAspectRatio = function (mode) {
            var ratio = null;
            if (mode === 'original') {
                ratio = this.cropper.getImageData().aspectRatio;
            }
            else if (mode === 'square') {
                ratio = 1;
            }
            this.setAspectRatioAndFitToImage(ratio);
            this.updateParams();
        };
        /**
         * Fit the crop area to the image.
         */
        ImageEditorCropperController.prototype.fitToImage = function () {
            this.setAspectRatio(this.cropperOptions.aspectRatio);
        };
        return ImageEditorCropperController;
    }());
    function imageEditorCropperDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/imageEditor/imageEditorCropper.html',
            controller: 'ImageEditorCropperController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                params: '=',
                onCrop: '&'
            }
        };
    }
    angular.module('meshAdminUi.projects.imageEditor')
        .directive('imageEditorCropper', imageEditorCropperDirective)
        .controller('ImageEditorCropperController', ImageEditorCropperController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Directive that takes a microschema { name, uuid } and the fields object for it,
     * and either dispatches to a custom control (if one exists) or builds the form
     * for it from the standard set of controls.
     */
    function microschemaFormBuilderDirective($injector, $compile, dataService) {
        var defaultTemplate = '<div>' +
            '<label class="micronode-label">{{:: fieldModel.label || fieldModel.name  }}</label>' +
            '<div class="micronode-container editor-control">' +
            '<div ng-repeat="field in micronodeFieldModels">' +
            '<control-proxy field-model="field"></control-proxy>' +
            '</div>' +
            '</div></div>';
        function microschemaProxyLinkFn(scope, element) {
            var fieldModel = scope.fieldModel;
            var activeMicroschemaName = getActiveMicroschema(fieldModel);
            var renderMicroschema = function (microschema) {
                var template = defaultTemplate;
                if (customControlExistsFor(microschema.name)) {
                    template = getCustomControlTemplate(microschema.name);
                }
                else {
                    template = defaultTemplate;
                }
                var nodeFields = fieldModel.value || createEmptyMicronodeField(microschema.name);
                scope.fields = {};
                scope.micronodeFieldModels = microschema.fields.map(function (field) {
                    var fm = fieldModel.createChild(nodeFields, field, fieldModel.path.concat('fields'));
                    scope.fields[field.name] = fm;
                    return fm;
                });
                var compiledDom = $compile(template)(scope);
                element.append(compiledDom);
            };
            scope.microschemaName = activeMicroschemaName;
            dataService.getMicroschemaByName(activeMicroschemaName).then(renderMicroschema);
        }
        /**
         * Returns the currently-used microschema when a value is set. If no value is set but only one
         * type is allowed, return that.
         */
        function getActiveMicroschema(fieldModel) {
            if (fieldModel.value) {
                return fieldModel.value.microschema.name;
            }
            return fieldModel.allow[0];
        }
        /**
         * Creates an empty field value for a micronode.
         */
        function createEmptyMicronodeField(microschemaName) {
            return {
                microschema: {
                    name: microschemaName
                },
                fields: {}
            };
        }
        /**
         * Creates a wrapper template for the custom control for the microschema.
         */
        function getCustomControlTemplate(microschemaName) {
            var normalizedName = 'custom-control-' + microschemaName.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase(), template = "<" + normalizedName + "></" + normalizedName + ">";
            return ("<div class=\"microschema-container editor-control " + normalizedName + "\">") +
                '<label class="microschema-label">{{:: field.label || field.name  }}</label>' +
                template +
                '</div>';
        }
        /**
         * Returns true if the AngularJS $injector has a custom directive registered for the given
         * microschema name.
         */
        function customControlExistsFor(controlName) {
            var capitalizeFirst = function (str) {
                return str.charAt(0).toUpperCase() + str.slice(1);
            };
            var directiveName = 'customControl' + capitalizeFirst(controlName) + 'Directive';
            return $injector.has(directiveName);
        }
        return {
            restrict: 'E',
            link: microschemaProxyLinkFn,
            scope: {
                fieldModel: '='
            }
        };
    }
    microschemaFormBuilderDirective.$inject = ["$injector", "$compile", "dataService"];
    angular.module('meshAdminUi.projects.formBuilder')
        .directive('microschemaFormBuilder', microschemaFormBuilderDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * This service is responsible for creating the INodeFieldModel objects which are used by the
     * formBuilder and controls to generate the form for editing a node.
     */
    var FormBuilderService = (function () {
        function FormBuilderService(mu) {
            this.mu = mu;
        }
        FormBuilderService.$inject = ["mu"];
        /**
         * A NodeFieldModel object contains all the necessary data to render a single field in
         * the form, including the means to update itself.
         */
        FormBuilderService.prototype.createNodeFieldModels = function (config) {
            var _this = this;
            config.displayField = config.displayField || '';
            config.pathPrefix = config.pathPrefix || [];
            return config.schemaFields.map(function (schemaField) {
                var nodeConfig = config;
                nodeConfig.schemaField = schemaField;
                return _this.createNodeFieldModel(nodeConfig);
            });
        };
        /**
         * Creates and returns a NodeFieldModel object, which is used to generate
         * a single field in the node editor form.
         */
        FormBuilderService.prototype.createNodeFieldModel = function (config) {
            config.displayField = config.displayField || '';
            config.pathPrefix = config.pathPrefix || [];
            var model = angular.copy(config.schemaField);
            var path = config.pathPrefix.slice(0);
            if (config.schemaField.name) {
                path.push(config.schemaField.name);
            }
            config.nodeFields = this.ensureNodeFieldsExist(config.nodeFields, config.pathPrefix, config.schemaField);
            model.id = this.mu.generateGuid();
            model.value = angular.copy(this.getValueAtPath(config.nodeFields, path));
            model.path = path;
            model.canUpdate = config.canUpdate;
            model.isDisplayField = config.schemaField.name === config.displayField;
            model.projectName = config.projectName;
            model.node = config.node;
            model.onChange = config.onChange;
            model.update = this.makeUpdateFunction(model.path, config.nodeFields, config.onChange);
            model.createChild = this.makeCreateChildFunction(config);
            return model;
        };
        /**
         * Check that each of the fields specified in the schemaFields array has a counterpart defined
         * in the nodeFields object. If not, create a null placeholder property.
         */
        FormBuilderService.prototype.ensureNodeFieldsExist = function (nodeFields, path, schemaField) {
            if (!schemaField.name) {
                return nodeFields;
            }
            var pointer = this.getPointerByPath(nodeFields, path);
            if (0 < path.length) {
                if (pointer[path[path.length - 1]][schemaField.name] === undefined) {
                    pointer[path[path.length - 1]][schemaField.name] = null;
                }
            }
            else {
                if (!pointer[schemaField.name]) {
                    pointer[schemaField.name] = null;
                }
            }
            return nodeFields;
        };
        /**
         * Given an object and a path e.g. ['foo', 'bar'], return the value of
         * object.foo.bar.
         */
        FormBuilderService.prototype.getValueAtPath = function (object, path) {
            var pointer = this.getPointerByPath(object, path);
            return pointer[path[path.length - 1]];
        };
        /**
         * Returns a pre-configured function that will update the node field specified by a path array.
         * Note: currently the actual node fields get updated simply because we are keeping a reference to them
         * via the `nodeFields` object, which we mutate here in this function.
         *
         * TODO: consider a more explicit data-flow where we pass the new, updated fields object back and do not
         * rely on mutation. Note that the current design may be this way specifically to prevent thrashing
         * and focus issues when dealing with list types - new objects may cause the input to lost focus on
         * each change.
         */
        FormBuilderService.prototype.makeUpdateFunction = function (path, nodeFields, onChange) {
            var _this = this;
            return function (value) {
                onChange();
                _this.updateAtPath(nodeFields, path, value);
            };
        };
        /**
         * Factory function which returns a function that is used to create a child nodeFieldModel, which inherits
         * from the current one.
         */
        FormBuilderService.prototype.makeCreateChildFunction = function (config) {
            var _this = this;
            return function (childFields, childSchemaField, path) {
                var fields = config.nodeFields;
                if (!fields[path[0]]) {
                    fields[path[0]] = childFields;
                }
                var childConfig = angular.copy(config);
                childConfig.nodeFields = fields;
                childConfig.schemaField = childSchemaField;
                childConfig.pathPrefix = path;
                return _this.createNodeFieldModel(childConfig);
            };
        };
        /**
         * Given an object, update the value specified by the `path` array with the given value.
         */
        FormBuilderService.prototype.updateAtPath = function (object, path, value) {
            var pointer = this.getPointerByPath(object, path);
            return pointer[path[path.length - 1]] = value;
        };
        /**
         * Given an object and a path e.g. ['foo', 'bar'], return the a pointer to
         * the object.foo.bar property.
         */
        FormBuilderService.prototype.getPointerByPath = function (object, path) {
            var pointer = object;
            for (var i = 0; i < path.length - 1; i++) {
                var key = path[i];
                if (!pointer[key]) {
                    pointer[key] = {};
                }
                pointer = pointer[key];
            }
            return pointer;
        };
        return FormBuilderService;
    }());
    meshAdminUi.FormBuilderService = FormBuilderService;
    angular.module('meshAdminUi.projects.formBuilder')
        .service('formBuilderService', FormBuilderService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Component for auto-generating a form from schema data.
     *
     * API:
     * ====
     * fields = the object properties from the node fields
     * schema = the schema fields definitions
     *
     */
    function formBuilderDirective() {
        return {
            restrict: 'E',
            replace: true,
            templateUrl: 'projects/components/formBuilder/formBuilder.html',
            controller: 'formBuilderController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                fields: '=',
                schema: '=',
                onChange: '&',
                perms: '=',
                displayField: '=',
                projectName: '=',
                node: '='
            }
        };
    }
    var FormBuilderController = (function () {
        function FormBuilderController($scope, formBuilderService) {
            var _this = this;
            this.formBuilderService = formBuilderService;
            if (!this.perms) {
                this.perms = {};
            }
            var createNodeFields = function () {
                if (_this.schema && _this.fields) {
                    var canUpdate = _this.perms && _this.perms.update;
                    var config = {
                        nodeFields: _this.fields,
                        schemaFields: _this.schema,
                        canUpdate: canUpdate,
                        onChange: _this.onChange,
                        displayField: _this.displayField,
                        projectName: _this.projectName,
                        node: _this.node
                    };
                    _this.nodeFieldModels = formBuilderService.createNodeFieldModels(config);
                }
            };
            $scope.$watch(function () { return _this.schema; }, createNodeFields);
            $scope.$watch(function () { return _this.node; }, createNodeFields);
        }
        FormBuilderController.$inject = ["$scope", "formBuilderService"];
        return FormBuilderController;
    }());
    angular.module('meshAdminUi.projects.formBuilder')
        .directive('formBuilder', formBuilderDirective)
        .controller('formBuilderController', FormBuilderController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * This directive is used to dynamically populate the content editing form with the correct
     * type of input to match the schema fields. E.g. a field of type "string" should render as
     * a text input, whereas a field of type "date" should be a date picker, etc.
     *
     * Each of the standard components for the various field types can be found in the "standardControls" folder.
     * This proxy will delegate to the correct standard control based on the "field" attribute that is passed in.
     */
    function controlProxyDirective() {
        function controlProxyLinkFn(scope, element) {
            var isTopElement = element.parent()[0].tagName === 'FORM';
            if (isTopElement) {
                scope.flexAttrs = getFlexAttributes(scope.fieldModel.type);
            }
        }
        /**
         * Different field types have different flex attributes to set how wide they will appear in the form.
         * @param {string} type
         * @returns {*}
         */
        function getFlexAttributes(type) {
            var flexAttrs = {
                sm: 100,
                gtSm: 100,
                gtLg: 100
            };
            switch (type) {
                case 'html':
                case 'list':
                case 'microschema':
                case 'date':
                    //flexAttrs = 'flex="100"';
                    break;
                case 'number':
                case 'boolean':
                    //flexAttrs = 'flex-sm="50" flex-gt-sm="33" flex-gt-lg="20"';
                    flexAttrs.sm = 50;
                    flexAttrs.gtSm = 33;
                    flexAttrs.gtLg = 25;
                    break;
                case 'string':
                    //flexAttrs = 'flex="100"';
                    break;
                default:
            }
            return flexAttrs;
        }
        return {
            restrict: 'EA',
            link: controlProxyLinkFn,
            replace: true,
            templateUrl: 'projects/components/formBuilder/controlProxy.html',
            scope: {
                fieldModel: '='
            }
        };
    }
    angular.module('meshAdminUi.projects.formBuilder')
        .directive('controlProxy', controlProxyDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * This service allows communication between the various parts of the explorer contents list. Specifically it
     * stores which list items are currently selected.
     */
    var ExplorerContentsListService = (function () {
        function ExplorerContentsListService() {
            this.totalItems = 0;
            this.itemsPerPage = 0;
            this.selectedItems = [];
            this._areAllSelected = false;
        }
        /**
         * Toggle whether the items at index is selected.
         */
        ExplorerContentsListService.prototype.toggleSelect = function (uuid) {
            if (this.isSelected(uuid)) {
                var idx = this.selectedItems.indexOf(uuid);
                this.selectedItems.splice(idx, 1);
            }
            else {
                this.selectedItems.push(uuid);
            }
            this._areAllSelected = this.areAllSelected();
        };
        /**
         * Is the item at index currently selected?
         */
        ExplorerContentsListService.prototype.isSelected = function (uuid) {
            return -1 < this.selectedItems.indexOf(uuid);
        };
        ExplorerContentsListService.prototype.getSelection = function () {
            return this.selectedItems;
        };
        ExplorerContentsListService.prototype.clearSelection = function () {
            return this.selectedItems = [];
        };
        /**
         * Are all items selected?
         */
        ExplorerContentsListService.prototype.areAllSelected = function () {
            return this.selectedItems.length === this.itemsPerPage;
        };
        return ExplorerContentsListService;
    }());
    meshAdminUi.ExplorerContentsListService = ExplorerContentsListService;
    angular.module('meshAdminUi.projects')
        .service('explorerContentsListService', ExplorerContentsListService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * The table used to display the contents of a tag.
     *
     * @returns {ng.IDirective} Directive definition object
     */
    function explorerContentsListDirective() {
        return {
            restrict: 'E',
            replace: true,
            templateUrl: 'projects/components/explorerContentsList/explorerContentsList.html',
            controller: 'explorerContentsListController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                contents: '=',
                tagsArray: '=',
                itemsPerPage: '=',
                onPageChange: '&'
            }
        };
    }
    var ExplorerContentsListController = (function () {
        function ExplorerContentsListController($scope, $state, $q, dispatcher, mu, searchService, explorerContentsListService, contextService, editorService, i18nService, i18n, $timeout, nodeSelector, dataService, notifyService, deleteNodeDialog, wipService, confirmActionDialog) {
            var _this = this;
            this.$state = $state;
            this.$q = $q;
            this.dispatcher = dispatcher;
            this.mu = mu;
            this.searchService = searchService;
            this.explorerContentsListService = explorerContentsListService;
            this.contextService = contextService;
            this.editorService = editorService;
            this.i18nService = i18nService;
            this.i18n = i18n;
            this.$timeout = $timeout;
            this.nodeSelector = nodeSelector;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.deleteNodeDialog = deleteNodeDialog;
            this.wipService = wipService;
            this.confirmActionDialog = confirmActionDialog;
            this.searchQuery = '';
            this.tagsArray = {};
            this.wipType = 'contents';
            this.filterNodes = function (node) {
                return _this.mu.nodeFilterFn(node, _this.searchQuery);
            };
            explorerContentsListService.clearSelection();
            var searchTermHandler = function (event, params) {
                _this.searchQuery = params.searchTerm;
            };
            dispatcher.subscribe(dispatcher.events.explorerSearchParamsChanged, searchTermHandler);
            $scope.$on('$destroy', function () { return dispatcher.unsubscribeAll(searchTermHandler); });
            if (editorService.getOpenNodeId()) {
                editorService.open(editorService.getOpenNodeId());
            }
        }
        ExplorerContentsListController.$inject = ["$scope", "$state", "$q", "dispatcher", "mu", "searchService", "explorerContentsListService", "contextService", "editorService", "i18nService", "i18n", "$timeout", "nodeSelector", "dataService", "notifyService", "deleteNodeDialog", "wipService", "confirmActionDialog"];
        /**
         * Toggle whether the items at index is selected.
         */
        ExplorerContentsListController.prototype.toggleSelect = function (node, event) {
            if (event.target.classList.contains('node-item-menu-icon')) {
                return false;
            }
            if (this.isAvailableInCurrentLang(node)) {
                this.explorerContentsListService.toggleSelect(node.uuid);
            }
        };
        /**
         * Is the item at index currently selected?
         */
        ExplorerContentsListController.prototype.isSelected = function (uuid) {
            return this.explorerContentsListService.isSelected(uuid);
        };
        /**
         * Returns true if the node is available in the current language.
         */
        ExplorerContentsListController.prototype.isAvailableInCurrentLang = function (node) {
            var _this = this;
            return 0 < node.availableLanguages.filter(function (lang) { return _this.i18nService.getCurrentLang().code === lang; }).length;
        };
        /**
         * Transition to the contentEditor view for the given uuid
         */
        ExplorerContentsListController.prototype.openNode = function (node, event) {
            event.preventDefault();
            event.stopPropagation();
            if (node.container || node.hasOwnProperty('displayName')) {
                this.$state.go('projects.node', {
                    projectName: this.contextService.getProject().name, nodeId: node.uuid
                });
            }
            else if (this.isAvailableInCurrentLang(node)) {
                this.editNode(node, event);
            }
        };
        ExplorerContentsListController.prototype.openNodeInLanguage = function (code, node) {
            var _this = this;
            var languageName = this.i18nService.getLanguageInfo(code).name;
            this.confirmActionDialog.show({
                title: this.i18n('SWITCH_LANGUAGE_AND_OPEN_TITLE', { lang: languageName + " (" + code + ")" }),
                message: this.i18n('SWITCH_LANGUAGE_AND_OPEN_MESSAGE', { lang: languageName }),
                confirmLabel: 'OKAY',
                cancelLabel: 'CANCEL',
                confirmButtonClass: 'btn-primary'
            }).then(function () {
                _this.i18nService.setCurrentLang(code);
                _this.$state.reload();
                _this.$timeout(function () {
                    _this.editorService.open(node.uuid);
                }, 500);
            });
        };
        /**
         * Open the node up in the editor pane
         */
        ExplorerContentsListController.prototype.editNode = function (node, event) {
            event.preventDefault();
            event.stopPropagation();
            this.editorService.open(node.uuid);
        };
        ExplorerContentsListController.prototype.getBinaryRepresentation = function (item) {
            return item.path;
        };
        ExplorerContentsListController.prototype.isGlobal = function () {
            return this.searchService.getParams().searchAll;
        };
        ExplorerContentsListController.prototype.pageChanged = function (newPageNumber, schemaUuid) {
            this.onPageChange({ newPageNumber: newPageNumber, schemaUuid: schemaUuid });
        };
        ExplorerContentsListController.prototype.isPublished = function (node) {
            return node.version && node.version.number.substr(-2) === '.0';
        };
        /**
         * Returns true if the node has at least one binary field.
         */
        ExplorerContentsListController.prototype.hasBinaryField = function (node) {
            var firstBinaryField = this.mu.getFirstBinaryField(node);
            return !!(firstBinaryField && firstBinaryField.value);
        };
        /**
         * Returns true is the node has a binary field and that field is an image type.
         */
        ExplorerContentsListController.prototype.isImageNode = function (node) {
            var binaryField = this.mu.getFirstBinaryField(node);
            if (binaryField.value) {
                return this.mu.isImageField(binaryField.value);
            }
            return false;
        };
        /**
         * Create a copy of the selected node to some location selected from the dialog.
         */
        ExplorerContentsListController.prototype.copyNode = function (node, schema, event) {
            var _this = this;
            event.preventDefault();
            var nodeClone = this.mu.safeCloneNode(node, schema, 'copy');
            var projectName = this.contextService.getProject().name;
            var destinationNode;
            delete nodeClone.created;
            delete nodeClone.uuid;
            this.nodeSelector.open({
                containersOnly: true,
                includeRootNode: true,
                title: 'SELECT_DESTINATION',
                startingNodeUuid: node.parentNode.uuid
            })
                .then(function (selection) {
                destinationNode = selection[0];
                nodeClone.parentNode.uuid = destinationNode.uuid;
                return _this.dataService.persistNode(projectName, nodeClone);
            })
                .then(function () {
                _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged);
                _this.$state.go('projects.node', {
                    projectName: projectName, nodeId: destinationNode.uuid
                });
                _this.notifyService.toast('COPIED_NODE');
            })
                .catch(function (response) {
                if (response) {
                    _this.notifyService.toast(response.data.message);
                }
            });
        };
        /**
         * Show the node selector dialog and move selected nodes to that folder.
         */
        ExplorerContentsListController.prototype.moveNode = function (node, event) {
            var _this = this;
            event.preventDefault();
            var projectName = this.contextService.getProject().name;
            var destinationNode;
            this.nodeSelector.open({
                containersOnly: true,
                includeRootNode: true,
                title: 'SELECT_DESTINATION',
                startingNodeUuid: node.parentNode.uuid
            })
                .then(function (selection) {
                destinationNode = selection[0];
                return _this.dataService.moveNode(projectName, node.uuid, destinationNode.uuid);
            })
                .then(function () {
                _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged);
                _this.$state.go('projects.node', {
                    projectName: projectName, nodeId: destinationNode.uuid
                });
                _this.notifyService.toast('MOVED_NODES', { count: 1 });
            })
                .catch(function (response) {
                if (response) {
                    _this.notifyService.toast(response.data.message);
                }
            });
        };
        /**
         * Show the delete node dialog and delete the node.
         */
        ExplorerContentsListController.prototype.deleteNode = function (node, event) {
            var _this = this;
            event.preventDefault();
            var projectName = this.contextService.getProject().name;
            this.deleteNodeDialog.show(node)
                .then(function (langs) {
                if (_this.checkDeleteAll(langs, node)) {
                    return _this.dataService.deleteNode(projectName, node);
                }
                else {
                    var promises = langs.map(function (code) {
                        return _this.dataService.deleteNodeLanguage(projectName, node, code);
                    });
                    return _this.$q.all(promises);
                }
            })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.editorService.closeAll();
                _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged);
                if (_this.wipService.isOpen(_this.wipType, node.uuid)) {
                    _this.wipService.closeItem(_this.wipType, node);
                }
                _this.editorService.close();
            });
        };
        /**
         * Did the user select to delete all available languages?
         */
        ExplorerContentsListController.prototype.checkDeleteAll = function (langs, node) {
            if (!node.availableLanguages || node.availableLanguages.length < 2) {
                return true;
            }
            return node.availableLanguages.length === langs.length;
        };
        return ExplorerContentsListController;
    }());
    angular.module('meshAdminUi.projects')
        .directive('explorerContentsList', explorerContentsListDirective)
        .controller('explorerContentsListController', ExplorerContentsListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var tagDisplayLimit = meshUiConfig.tagDisplayLimit || 3;
    var ExpandableTagsListController = (function () {
        function ExpandableTagsListController() {
            this.tagsLimit = tagDisplayLimit;
            this.isExpanded = false;
        }
        ExpandableTagsListController.prototype.toggle = function (event) {
            event.stopPropagation();
            event.preventDefault();
            this.isExpanded = !this.isExpanded;
            this.tagsLimit = this.isExpanded ? 99999999 : tagDisplayLimit;
        };
        return ExpandableTagsListController;
    }());
    function expandableTagsListDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/expandableTagsList/expandableTagsList.html',
            controller: 'ExpandableTagsListController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                tags: '='
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('expandableTagsList', expandableTagsListDirective)
        .controller('ExpandableTagsListController', ExpandableTagsListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var EditorService = (function () {
        function EditorService($rootScope, dispatcher, $location) {
            var _this = this;
            this.$rootScope = $rootScope;
            this.dispatcher = dispatcher;
            this.$location = $location;
            /**
             * We need to watch the "edit" query string in order to react to
             * external URL changes, e.g. caused by use of browser back &
             * forward buttons.
             */
            $rootScope.$watch(function () { return $location.search().edit; }, function (newVal) {
                if (newVal && newVal !== _this.openNodeId) {
                    _this.open(newVal);
                }
            });
        }
        EditorService.$inject = ["$rootScope", "dispatcher", "$location"];
        EditorService.prototype.open = function (uuid) {
            var eventName = this.dispatcher.events.editorServiceNodeOpened;
            this.openNodeId = uuid;
            this.dispatcher.publish(eventName, uuid);
        };
        EditorService.prototype.create = function (schemaId, parentNodeUuid) {
            var eventName = this.dispatcher.events.editorServiceNodeOpened;
            this.openNodeId = undefined;
            this.dispatcher.publish(eventName, undefined, schemaId, parentNodeUuid);
        };
        EditorService.prototype.close = function () {
            var eventName = this.dispatcher.events.editorServiceNodeClosed;
            this.dispatcher.publish(eventName);
        };
        EditorService.prototype.closeAll = function () {
            var eventName = this.dispatcher.events.editorServiceNodeClosed;
            this.$location.search('edit', null);
            this.openNodeId = undefined;
            this.dispatcher.publish(eventName);
        };
        EditorService.prototype.getOpenNodeId = function () {
            return this.openNodeId;
        };
        return EditorService;
    }());
    meshAdminUi.EditorService = EditorService;
    angular.module('meshAdminUi.projects')
        .service('editorService', EditorService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    function editorPaneDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/editorPane/editorPane.html',
            controller: 'EditorPaneController',
            controllerAs: 'vm',
            scope: {}
        };
    }
    /**
     * Controller for the node edit/create form.
     */
    var EditorPaneController = (function () {
        function EditorPaneController($scope, $q, $state, $timeout, $location, deleteNodeDialog, editorService, dispatcher, mu, contextService, i18nService, dataService, wipService, notifyService) {
            var _this = this;
            this.$scope = $scope;
            this.$q = $q;
            this.$state = $state;
            this.$timeout = $timeout;
            this.$location = $location;
            this.deleteNodeDialog = deleteNodeDialog;
            this.editorService = editorService;
            this.dispatcher = dispatcher;
            this.mu = mu;
            this.contextService = contextService;
            this.i18nService = i18nService;
            this.dataService = dataService;
            this.wipService = wipService;
            this.notifyService = notifyService;
            this.wipType = 'contents';
            var ignoreNext = false;
            // Save a copy of the current language name to prevent a race condition when switching languages, where
            // this constructor is invoked before the $destroy() handler has a chance to
            // clean up the dispatcher subscriptions, causing init() to be called twice on the
            // newly-opened node.
            var currentLangName = i18nService.getCurrentLang().name;
            var init = function (event, nodeUuid, schemaUuid, parentNodeUuid) {
                /**
                 * When a new node is created, this init function will first be called when the editorService.create()
                 * is invoked and publishes the editorServiceNodeOpened event. In this case, nodeUuid is undefined.
                 *
                 * Next, the url will change to the new  temp id of the new node. This will trigger a watcher in the
                 * editorService which then opens that temp id, triggering a new editorServiceNodeOpened event.
                 * In order to prevent this init function being called twice, we set the `ignoreNext` value to
                 * true (at the bottom of this function), and then the next call to init() will set it
                 * back to false and return.
                 */
                if (ignoreNext) {
                    ignoreNext = false;
                    return;
                }
                // see note above at declaration of currentLangName
                if (currentLangName !== i18nService.getCurrentLang().name) {
                    return;
                }
                _this.projectName = contextService.getProject().name;
                _this.currentNodeId = nodeUuid;
                _this.contentModified = false;
                _this.tagsModified = false;
                _this.node = undefined;
                _this.selectedLangs = {};
                _this.selectedLangs[i18nService.getCurrentLang().code] = true; // set the default language
                _this.isLoaded = false;
                _this.binaryFile = undefined;
                _this.getNodeData(schemaUuid, parentNodeUuid)
                    .then(function () { return _this.isLoaded = true; })
                    .then(function () { return $location.search('edit', _this.node.uuid); });
                ignoreNext = (typeof nodeUuid === 'undefined');
            };
            var empty = function () {
                _this.isLoaded = false;
            };
            var emptyIfOpenNodeDeleted = function () {
                if (!wipService.isOpen(_this.wipType, _this.node.uuid)) {
                    empty();
                }
            };
            dispatcher.subscribe(dispatcher.events.editorServiceNodeOpened, init);
            dispatcher.subscribe(dispatcher.events.editorServiceNodeClosed, empty);
            dispatcher.subscribe(dispatcher.events.explorerContentsChanged, emptyIfOpenNodeDeleted);
            $scope.$on('$destroy', function () {
                if (_this.node !== undefined) {
                    _this.saveWipMetadata();
                }
                dispatcher.unsubscribeAll(init, empty, emptyIfOpenNodeDeleted);
            });
        }
        EditorPaneController.$inject = ["$scope", "$q", "$state", "$timeout", "$location", "deleteNodeDialog", "editorService", "dispatcher", "mu", "contextService", "i18nService", "dataService", "wipService", "notifyService"];
        EditorPaneController.prototype.isPublished = function () {
            return this.node && this.node.version && this.node.version.number.substr(-2) === '.0';
        };
        EditorPaneController.prototype.readyToPublish = function () {
            var isPublished = this.isPublished();
            return !isPublished || (isPublished && this.contentModified || this.tagsModified);
        };
        /**
         * Save any changes and then publish the node
         */
        EditorPaneController.prototype.publish = function () {
            var _this = this;
            var savePromise;
            var savingNode = false;
            if (this.contentModified || this.tagsModified) {
                savePromise = this.persist(this.node);
                savingNode = true;
            }
            else {
                savePromise = this.$q.when();
            }
            savePromise.then(function () {
                _this.dataService.publishNode(_this.projectName, _this.node)
                    .then(function (data) {
                    var newVersionInfo = data.availableLanguages[_this.node.language];
                    if (newVersionInfo) {
                        _this.node.version = newVersionInfo.version;
                        _this.wipService.updateItem(_this.wipType, _this.node);
                        _this.notifyService.toast('PUBLISHED');
                        if (!savingNode) {
                            return _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged);
                        }
                    }
                })
                    .catch(function (error) {
                    _this.notifyService.toast(error.data.message || error.data);
                });
            });
        };
        /**
         * Save the changes back to the server.
         */
        EditorPaneController.prototype.persist = function (originalNode, forceApiCall) {
            var _this = this;
            if (forceApiCall === void 0) { forceApiCall = false; }
            var promises = [];
            if (this.contentModified || forceApiCall) {
                promises.push(this.dataService.persistNode(this.projectName, originalNode, { expandAll: true }));
            }
            else {
                promises.push(this.$q.when(originalNode));
            }
            if (this.tagsModified) {
                promises.push(this.dataService.updateNodeTags(this.projectName, this.node, this.node.tags)
                    .then(function () { return originalNode; }));
            }
            return this.$q.all(promises)
                .then(function (result) {
                var node = result[0];
                _this.node = node;
                if (_this.isNew(originalNode)) {
                    _this.notifyService.toast('NEW_CONTENT_CREATED');
                    _this.wipService.closeItem(_this.wipType, originalNode);
                    _this.processNewNode(node);
                }
                else {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.wipService.setAsUnmodified(_this.wipType, _this.node);
                }
                if (_this.tagsModified) {
                    _this.dispatcher.publish(_this.dispatcher.events.explorerNodeTagsChanged, _this.node.uuid);
                }
                _this.contentModified = false;
                _this.tagsModified = false;
                _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged);
            })
                .catch(function (error) {
                _this.notifyService.toast(error.data.message || error.data);
            });
        };
        // TODO: This logic will largely be replaced by a dedicated translation endpoint.
        // See https://jira.gentics.com/browse/CL-396
        EditorPaneController.prototype.createTranslation = function (langCode, node) {
            var _this = this;
            var nodeClone = this.mu.safeCloneNode(node, this.schema, langCode.toUpperCase());
            nodeClone.language = langCode;
            // Display a warning if there are any binary fields - these cannot be handled properly
            // until the dedicated translation endpoint is implemented in Mesh.
            var firstBinaryField = this.mu.getFirstBinaryField(node);
            if (firstBinaryField.key !== undefined) {
                console.warn("Note: binary fields cannot yet be copied to translated version.");
            }
            this.persist(nodeClone, true)
                .then(function () {
                _this.updateCurrentNodeAvailableLangs(node, langCode);
                _this.i18nService.setCurrentLang(langCode);
                _this.$state.reload();
                _this.$timeout(function () {
                    _this.editorService.open(node.uuid);
                }, 500);
            });
        };
        EditorPaneController.prototype.updateCurrentNodeAvailableLangs = function (node, langCode) {
            if (node.availableLanguages instanceof Array) {
                node.availableLanguages.push(langCode);
            }
            else {
                node.availableLanguages = [node.language, langCode];
            }
        };
        EditorPaneController.prototype.openInLanguage = function (langCode, node) {
            var _this = this;
            this.i18nService.setCurrentLang(langCode);
            this.$state.reload();
            this.$timeout(function () {
                _this.editorService.open(node.uuid);
            }, 500);
        };
        EditorPaneController.prototype.processNewNode = function (node) {
            this.$location.search('edit', node.uuid);
            this.openInWipService(node);
        };
        EditorPaneController.prototype.addTag = function (tag) {
            if (!tag) {
                return;
            }
            var tagIsDuplicate = -1 < this.node.tags.map(function (tag) { return tag.uuid; }).indexOf(tag.uuid);
            if (!tagIsDuplicate) {
                this.tagsModified = true;
                this.wipService.setAsModified(this.wipType, this.node);
                this.node.tags.push({
                    name: tag.name,
                    uuid: tag.uuid,
                    tagFamily: tag.tagFamily.name
                });
            }
        };
        EditorPaneController.prototype.removeTag = function (tag) {
            if (!tag) {
                return;
            }
            this.tagsModified = true;
            this.wipService.setAsModified(this.wipType, this.node);
            var index = this.node.tags.map(function (tag) { return tag.uuid; }).indexOf(tag.uuid);
            this.node.tags.splice(index, 1);
        };
        /**
         * Delete the open node, displaying a confirmation dialog first before making the API call.
         */
        EditorPaneController.prototype.remove = function (node) {
            var _this = this;
            this.deleteNodeDialog.show(node)
                .then(function (langs) {
                if (!_this.isNew(node)) {
                    if (_this.checkDeleteAll(langs, node)) {
                        return _this.dataService.deleteNode(_this.projectName, node);
                    }
                    else {
                        var promises = langs.map(function (code) {
                            return _this.dataService.deleteNodeLanguage(_this.projectName, node, code);
                        });
                        return _this.$q.all(promises);
                    }
                }
            })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.closeWipAndClearPane(node);
                _this.editorService.closeAll();
                _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged);
            });
        };
        /**
         * Did the user select to delete all available languages?
         */
        EditorPaneController.prototype.checkDeleteAll = function (langs, node) {
            if (!node.availableLanguages || node.availableLanguages.length < 2) {
                return true;
            }
            return node.availableLanguages.length === langs.length;
        };
        EditorPaneController.prototype.closeWipAndClearPane = function (node) {
            this.wipService.closeItem(this.wipType, node);
            this.editorService.close();
            this.$location.search('edit', null);
        };
        /**
         * Set the wip as modified.
         */
        EditorPaneController.prototype.setAsModified = function () {
            this.contentModified = true;
            this.wipService.setAsModified(this.wipType, this.node);
        };
        EditorPaneController.prototype.saveWipMetadata = function () {
            this.wipService.setMetadata(this.wipType, this.node.uuid, 'selectedLangs', this.selectedLangs);
        };
        EditorPaneController.prototype.canDelete = function (node) {
            if (node) {
                return node.permissions.delete;
            }
        };
        /**
         * Returns true if the form has changed (contentModified) and all
         * required fields have a non-empty value.
         */
        EditorPaneController.prototype.formIsValid = function () {
            var _this = this;
            if (!this.contentModified && !this.tagsModified) {
                return false;
            }
            var fieldIsValidReducer = function (valid, field) {
                if (field.required) {
                    var nodeField = _this.node.fields[field.name];
                    if (_this.fieldIsEmpty(nodeField, field)) {
                        return false;
                    }
                    if (_this.numberFieldIsInvalid(nodeField, field)) {
                        return false;
                    }
                }
                return valid;
            };
            if (this.schema && this.schema.fields instanceof Array) {
                return this.schema.fields.reduce(fieldIsValidReducer, true);
            }
        };
        /**
         * Returns true if the field has an "empty" value - definition of which varies according to the
         * field type.
         */
        EditorPaneController.prototype.fieldIsEmpty = function (field, fieldDef) {
            if (fieldDef.type === 'binary') {
                // binary fields have a special check for "emptiness"
                if (field && !(field instanceof File) && !field.hasOwnProperty('sha512sum')) {
                    return true;
                }
            }
            if (!field || field === '') {
                return true;
            }
            return false;
        };
        /**
         * Returns true if the value of a number field is within any specified `min` and `max` limits.
         */
        EditorPaneController.prototype.numberFieldIsInvalid = function (field, fieldDef) {
            if (fieldDef.type === 'number') {
                if (fieldDef.min && field < fieldDef.min ||
                    fieldDef.max && fieldDef.max < field) {
                    return false;
                }
            }
            return false;
        };
        /**
         * Get the node object either from the server if this is being newly opened, or from the
         * wipService if it exists there.
         */
        EditorPaneController.prototype.getNodeData = function (schemaUuid, parentNodeUuid) {
            var _this = this;
            if (this.currentNodeId) {
                // loading existing node
                var wipContent_1 = this.wipService.getItem(this.wipType, this.currentNodeId);
                var wipIsModified = wipContent_1 && this.wipService.isModified(this.wipType, wipContent_1);
                if (wipContent_1 && wipIsModified) {
                    return this.populateFromWip(wipContent_1);
                }
                else {
                    return this.dataService.getNode(this.projectName, this.currentNodeId, { expandAll: true })
                        .then(function (node) {
                        _this.node = node;
                        if (!wipContent_1) {
                            _this.openInWipService(_this.node);
                        }
                        else {
                            // the node is already open as a wip, but is not modified. Therefore we use the
                            // newly-fetched data to update that wip.
                            _this.wipService.updateItem(_this.wipType, node);
                        }
                        return _this.dataService.getSchema(node.schema.uuid);
                    })
                        .then(function (schema) { return _this.schema = schema; });
                }
            }
            else if (schemaUuid) {
                // creating new node
                return this.dataService.getSchema(schemaUuid)
                    .then(function (schema) {
                    _this.node = _this.createEmptyContent(schema, parentNodeUuid);
                    _this.openInWipService(_this.node);
                    _this.schema = schema;
                });
            }
        };
        EditorPaneController.prototype.openInWipService = function (node) {
            this.wipService.openItem(this.wipType, node, {
                projectName: this.projectName,
                selectedLangs: this.selectedLangs
            });
        };
        /**
         * Populate the vm based on the node item retrieved from the wipService.
         */
        EditorPaneController.prototype.populateFromWip = function (wipContent) {
            var _this = this;
            var wipMetadata = this.wipService.getMetadata(this.wipType, wipContent.uuid);
            this.node = wipContent;
            this.contentModified = this.wipService.isModified(this.wipType, this.node);
            this.selectedLangs = wipMetadata.selectedLangs;
            return this.dataService.getSchema(this.node.schema.uuid)
                .then(function (schema) { return _this.schema = schema; });
        };
        /**
         * Create an empty node object which is pre-configured according to the arguments passed.
         */
        EditorPaneController.prototype.createEmptyContent = function (schema, parentNodeUuid) {
            return {
                permissions: {
                    read: true,
                    create: true,
                    update: true,
                    delete: true,
                    readPublished: true,
                    publish: true
                },
                uuid: this.wipService.generateTempId(),
                parentNode: {
                    uuid: parentNodeUuid
                },
                displayField: schema.displayField,
                language: this.i18nService.getCurrentLang().code,
                schema: {
                    uuid: schema.uuid,
                    name: schema.name
                },
                fields: {}
            };
        };
        /**
         * Is the node new, i.e. is has not come from Mesh, but is has been created on the client.
         */
        EditorPaneController.prototype.isNew = function (node) {
            return !node || !node.hasOwnProperty('created');
        };
        return EditorPaneController;
    }());
    angular.module('meshAdminUi.projects')
        .directive('editorPane', editorPaneDirective)
        .controller('EditorPaneController', EditorPaneController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var DeleteNodeDialog = (function () {
        function DeleteNodeDialog($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        DeleteNodeDialog.$inject = ["$mdDialog"];
        DeleteNodeDialog.prototype.show = function (node) {
            return this.$mdDialog.show({
                controller: 'ConfirmDeleteDialogController',
                controllerAs: 'vm',
                bindToController: true,
                templateUrl: 'projects/components/deleteNodeDialog/confirmDeleteDialog.html',
                locals: {
                    node: node
                }
            });
        };
        DeleteNodeDialog.prototype.showMulti = function () {
            return this.$mdDialog.show({
                controller: 'ConfirmDeleteDialogController',
                controllerAs: 'vm',
                bindToController: true,
                templateUrl: 'projects/components/deleteNodeDialog/confirmDeleteDialogMulti.html',
                locals: {
                    node: false
                }
            });
        };
        return DeleteNodeDialog;
    }());
    meshAdminUi.DeleteNodeDialog = DeleteNodeDialog;
    angular.module('meshAdminUi.projects')
        .service('deleteNodeDialog', DeleteNodeDialog);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ConfirmDeleteDialogController = (function () {
        function ConfirmDeleteDialogController($mdDialog, i18nService, node) {
            this.$mdDialog = $mdDialog;
            this.i18nService = i18nService;
            this.node = node;
            this.langsToDelete = {};
            this.deleteAllLangs = false;
            if (node) {
                this.langsToDelete[node.language] = true;
            }
            this.currentLangInfo = i18nService.getCurrentLang();
        }
        ConfirmDeleteDialogController.$inject = ["$mdDialog", "i18nService", "node"];
        ConfirmDeleteDialogController.prototype.getLangInfo = function (code) {
            return this.i18nService.getLanguageInfo(code);
        };
        ConfirmDeleteDialogController.prototype.deleteNode = function () {
            var _this = this;
            var selectedLangs = Object.keys(this.langsToDelete).filter(function (code) {
                return _this.langsToDelete[code] === true;
            });
            this.$mdDialog.hide(selectedLangs);
        };
        ConfirmDeleteDialogController.prototype.deleteNodeMulti = function () {
            this.$mdDialog.hide({
                deleteAllLangs: this.deleteAllLangs
            });
        };
        ConfirmDeleteDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return ConfirmDeleteDialogController;
    }());
    angular.module('meshAdminUi.projects')
        .controller('ConfirmDeleteDialogController', ConfirmDeleteDialogController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var BinaryThumbnailController = (function () {
        function BinaryThumbnailController(mu, $element, $scope) {
            var _this = this;
            this.mu = mu;
            this.$element = $element;
            this.$scope = $scope;
            // used to style image transforms (crop, scale etc)
            this.styles = { content: {}, container: {} };
            if (!(this.srcUrl && this.srcField) && !this.srcFile) {
                throw new Error('BinaryThumbnailController: Please specify either src-url & src-field, or src-file.');
            }
            this.$scope.$watchCollection(function () { return _this.transform; }, function () {
                _this.calculateStyles();
            });
        }
        BinaryThumbnailController.$inject = ["mu", "$element", "$scope"];
        BinaryThumbnailController.prototype.isImage = function () {
            if (this.srcField) {
                return this.mu.isImageField(this.srcField);
            }
            else {
                return this.mu.isImageMimeType(this.srcFile.type);
            }
        };
        BinaryThumbnailController.prototype.getFileUrl = function () {
            if (this.srcUrl) {
                return this.srcUrl;
            }
            else {
                return this.srcFile.name;
            }
        };
        BinaryThumbnailController.prototype.getFileName = function () {
            if (this.srcField) {
                return this.srcField.fileName;
            }
            else {
                return this.srcFile.name;
            }
        };
        BinaryThumbnailController.prototype.getFileExt = function () {
            var filename = this.srcField ? this.srcField.fileName : this.srcFile.name;
            if (filename) {
                var parts = filename.split('.');
                return parts[parts.length - 1];
            }
        };
        /**
         * Calculate and set values for the objects which are bound to the resize preview DOM CSS.
         */
        BinaryThumbnailController.prototype.calculateStyles = function () {
            var container = this.$element[0].getBoundingClientRect();
            var isSmallerThanContainer = this.transform.cropw * this.transform.scale < container.width;
            if (!this.transform || !this.transform.cropw) {
                this.styles.content = {
                    position: 'relative'
                };
            }
            else {
                var scale_1 = this.transform.scale;
                var cropWidth_1 = this.transform.cropw * scale_1;
                var cropHeight_1 = this.transform.croph * scale_1;
                var scaleX = function (val) { return val * 100 / cropWidth_1 * scale_1; };
                var scaleY = function (val) { return val * 100 / cropHeight_1 * scale_1; };
                this.styles.content = {
                    position: 'absolute',
                    width: scaleX(this.transform.width) + '%',
                    height: scaleY(this.transform.height) + '%',
                    left: -scaleX(this.transform.cropx) + '%',
                    top: -scaleY(this.transform.cropy) + '%'
                };
                if (isSmallerThanContainer) {
                    // when the cropped / resized image is less wide than its container,
                    // we need to use absolute pixel values rather than percentages for the
                    // container.
                    this.styles.container = {
                        width: cropWidth_1 + 'px',
                        'padding-bottom': cropHeight_1 + 'px',
                    };
                }
                else {
                    this.styles.container = {
                        width: '100%',
                        'padding-bottom': scaleX(this.transform.croph) + '%',
                    };
                }
            }
        };
        return BinaryThumbnailController;
    }());
    function binaryThumbnailDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/binaryThumbnail/binaryThumbnail.html',
            controller: 'BinaryThumbnailController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                srcUrl: '=',
                srcField: '=',
                srcFile: '=',
                transform: '='
            }
        };
    }
    angular.module('meshAdminUi.projects')
        .directive('binaryThumbnail', binaryThumbnailDirective)
        .controller('BinaryThumbnailController', BinaryThumbnailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var BinaryFileInputController = (function () {
        function BinaryFileInputController() {
        }
        BinaryFileInputController.prototype.fileChanged = function () {
            this.onChange();
        };
        return BinaryFileInputController;
    }());
    function binaryFileInputDirective() {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/binaryFileInput/binaryFileInput.html',
            controller: 'BinaryFileInputController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                srcUrl: '=',
                srcField: '=',
                model: '=',
                onChange: '&',
                transform: '='
            }
        };
    }
    function binaryFilePreviewDirective(mu) {
        return {
            restrict: 'E',
            templateUrl: 'projects/components/binaryFileInput/binaryFilePreview.html',
            scope: {
                srcUrl: '=',
                srcField: '=',
                srcFile: '=',
                transform: '='
            },
            link: function (scope) {
                scope.isImageNode = function () {
                    return mu.isImageField(scope.field);
                };
                scope.isImageFile = function () {
                    return mu.isImageMimeType(scope.srcFile.type);
                };
                scope.getFileExt = function (filename) {
                    if (filename) {
                        var parts = filename.split('.');
                        return parts[parts.length - 1];
                    }
                };
            }
        };
    }
    binaryFilePreviewDirective.$inject = ["mu"];
    angular.module('meshAdminUi.common')
        .directive('binaryFilePreview', binaryFilePreviewDirective)
        .directive('binaryFileInput', binaryFileInputDirective)
        .controller('BinaryFileInputController', BinaryFileInputController);
})(meshAdminUi || (meshAdminUi = {}));

angular.module('meshAdminUi.login')
    .directive('animatedLogo', animatedLogoDirective);
function animatedLogoDirective() {
    function animatedMenuController() {
        var vm = this;
        vm.in = true;
        vm.toggle = function () {
            vm.in = !vm.in;
        };
    }
    return {
        restrict: 'E',
        templateUrl: 'login/components/animatedLogo/animatedLogo.html',
        replace: true,
        scope: {
            animateIn: '=',
            animateOut: '='
        },
        bindToController: true,
        controller: animatedMenuController,
        controllerAs: 'vm'
    };
}

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Directive work-in-progress (WIP) tabs which allow switching between open editor views.
     */
    function wipTabs() {
        return {
            restrict: 'E',
            templateUrl: 'common/components/wipTabs/wipTabs.html',
            controller: 'wipTabsController',
            controllerAs: 'vm',
            scope: {}
        };
    }
    var WipTabsController = (function () {
        function WipTabsController($scope, $q, $timeout, $mdDialog, dispatcher, editorService, i18nService, wipService, dataService, notifyService) {
            var _this = this;
            this.$q = $q;
            this.$timeout = $timeout;
            this.$mdDialog = $mdDialog;
            this.dispatcher = dispatcher;
            this.editorService = editorService;
            this.i18nService = i18nService;
            this.wipService = wipService;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.wipType = 'contents';
            this.lastIndex = 0;
            this.wips = [];
            this.modified = [];
            this.selectedIndex = 0;
            var changeHandler = function () { return _this.wipChangeHandler(); };
            this.lang = i18nService.getCurrentLang().code;
            this.dispatcher.subscribe(this.dispatcher.events.wipsChanged, changeHandler);
            this.dispatcher.subscribe(this.dispatcher.events.editorServiceNodeOpened, changeHandler);
            $scope.$on('$destroy', function () {
                _this.dispatcher.unsubscribeAll(changeHandler);
            });
            window.addEventListener('beforeunload', function () { return _this.persistOpenWipsLocally(); });
            this.wipChangeHandler(); // populate with WIPs on load.
        }
        WipTabsController.$inject = ["$scope", "$q", "$timeout", "$mdDialog", "dispatcher", "editorService", "i18nService", "wipService", "dataService", "notifyService"];
        /**
         * Has the WIP with the specified uuid been modified?
         */
        WipTabsController.prototype.isModified = function (uuid) {
            return -1 < this.modified.indexOf(uuid);
        };
        WipTabsController.prototype.open = function (uuid) {
            this.editorService.open(uuid);
            this.editorOpenHandler(uuid);
        };
        /**
         * Close a WIP tab and remove the WIP item from the wipService,
         * automatically switching to another tab or the list view.
         */
        WipTabsController.prototype.closeWip = function (index) {
            var _this = this;
            var wip = this.wips[index].item, projectName = this.wips[index].metadata.projectName, action;
            this.lastIndex = this.selectedIndex;
            if (this.wipService.isModified(this.wipType, wip)) {
                action = this.showDialog(wip).then(function (response) {
                    if (response === 'save') {
                        _this.notifyService.toast('SAVED_CHANGES');
                        _this.dataService.persistNode(projectName, wip)
                            .then(function () { return _this.dataService.updateNodeTags(projectName, wip, wip.tags); })
                            .then(function () { return _this.dispatcher.publish(_this.dispatcher.events.explorerContentsChanged); });
                    }
                });
            }
            else {
                action = this.$q.when();
            }
            return action.then(function () {
                _this.wipService.closeItem(_this.wipType, wip);
                _this.transitionIfCurrentTabClosed(index);
            });
        };
        /**
         * Close all open WIP tabs.
         */
        WipTabsController.prototype.closeAllWips = function () {
            var _this = this;
            if (0 < this.wips.length) {
                return this.closeWip(0)
                    .then(function () {
                    // $timeout is used to guard against race conditions where watchers may
                    // be trying to access removed object properties.
                    return _this.$timeout(function () { return _this.closeAllWips(); }, 0);
                });
            }
        };
        /**
         * Display the close confirmation dialog box. Returns a promise which is resolved
         * to 'save', 'discard', or rejected if user cancels.
         */
        WipTabsController.prototype.showDialog = function (wip) {
            return this.$mdDialog.show({
                templateUrl: 'common/components/wipTabs/wipTabsCloseDialog.html',
                controller: 'wipTabsDialogController',
                controllerAs: 'vm',
                locals: {
                    node: wip
                }
            });
        };
        /**
         * If the current tab has been closed, select the next tab in the editor pane.
         */
        WipTabsController.prototype.transitionIfCurrentTabClosed = function (closedTabIndex) {
            if (closedTabIndex === this.lastIndex) {
                if (0 < this.wips.length) {
                    var selectedIndex = this.selectedIndex !== -1 ? this.selectedIndex : 0;
                    this.editorService.open(this.wips[selectedIndex].item.uuid);
                }
                else {
                    this.editorService.closeAll();
                }
            }
        };
        /**
         * Callback to be invoked whenever the contents of the wipService changes
         * i.e. an item is added or removed. Keeps the UI in sync with the
         * wip store.
         */
        WipTabsController.prototype.wipChangeHandler = function () {
            this.wips = this.wipService.getOpenItems(this.wipType);
            this.modified = this.wipService.getModifiedItems(this.wipType);
            this.selectedIndex = this.indexByUuid(this.wips, this.editorService.getOpenNodeId());
        };
        WipTabsController.prototype.editorOpenHandler = function (uuid) {
            this.selectedIndex = this.indexByUuid(this.wips, uuid);
        };
        /**
         * Persist any open WIPs to the browser's localStorage.
         */
        WipTabsController.prototype.persistOpenWipsLocally = function () {
            this.wipService.persistLocal();
        };
        /**
         * Get the index of a given WIP item in the collection by its uuid.
         */
        WipTabsController.prototype.indexByUuid = function (collection, uuid) {
            if (collection.length === 0) {
                return -1;
            }
            return collection.map(function (wip) { return wip.item.uuid; }).indexOf(uuid);
        };
        return WipTabsController;
    }());
    /**
     * Controller used to set the return value of the close dialog box.
     */
    var WipTabsDialogController = (function () {
        function WipTabsDialogController($mdDialog, node) {
            this.$mdDialog = $mdDialog;
            this.node = node;
        }
        WipTabsDialogController.$inject = ["$mdDialog", "node"];
        WipTabsDialogController.prototype.save = function () {
            this.$mdDialog.hide('save');
        };
        WipTabsDialogController.prototype.discard = function () {
            this.$mdDialog.hide('discard');
        };
        WipTabsDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return WipTabsDialogController;
    }());
    angular.module('meshAdminUi.common')
        .directive('wipTabs', wipTabs)
        .controller('wipTabsController', WipTabsController)
        .controller('wipTabsDialogController', WipTabsDialogController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var TopMenuController = (function () {
        function TopMenuController($state, editorService, dispatcher, authService, i18nService) {
            var _this = this;
            this.$state = $state;
            this.editorService = editorService;
            this.dispatcher = dispatcher;
            this.authService = authService;
            this.i18nService = i18nService;
            this.isLoggedIn = authService.isLoggedIn();
            this.languages = i18nService.languages;
            this.lang = i18nService.getCurrentLang();
            this.profileImage = 'assets/images/example-profile-pic.jpg';
            dispatcher.subscribe(dispatcher.events.loginSuccess, function () {
                _this.isLoggedIn = true;
            });
            dispatcher.subscribe(dispatcher.events.logoutSuccess, function () {
                _this.isLoggedIn = false;
            });
            dispatcher.subscribe(dispatcher.events.languageChanged, function (event, lang) {
                _this.lang = lang;
            });
        }
        TopMenuController.$inject = ["$state", "editorService", "dispatcher", "authService", "i18nService"];
        TopMenuController.prototype.getUserName = function () {
            var currentUser = this.authService.getCurrentUser();
            if (!currentUser) {
                return;
            }
            if (currentUser.firstname && currentUser.lastname) {
                return currentUser.firstname + ' ' + currentUser.lastname;
            }
            else {
                return currentUser.username;
            }
        };
        TopMenuController.prototype.getInitials = function () {
            var currentUser = this.authService.getCurrentUser();
            if (!currentUser) {
                return;
            }
            if (currentUser.firstname && currentUser.lastname) {
                return currentUser.firstname[0] + currentUser.lastname[0];
            }
            else {
                return currentUser.username.substr(0, 2);
            }
        };
        TopMenuController.prototype.logout = function () {
            this.authService.logOut();
        };
        TopMenuController.prototype.updateLanguage = function (event, lang) {
            event.preventDefault();
            if (lang !== this.i18nService.getCurrentLang().code) {
                this.i18nService.setCurrentLang(lang);
                this.editorService.closeAll();
                this.$state.reload();
            }
        };
        ;
        return TopMenuController;
    }());
    /**
     * Directive for the top menu bar which is present on every screen apart from the login.
     */
    function topMenuDirective() {
        return {
            restrict: 'E',
            templateUrl: 'common/components/topMenu/topMenu.html',
            controller: 'TopMenuController',
            controllerAs: 'vm'
        };
    }
    angular.module('meshAdminUi.common')
        .directive('topMenu', topMenuDirective)
        .controller('TopMenuController', TopMenuController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    function tagDirective(mu) {
        return {
            restrict: 'E',
            templateUrl: 'common/components/tag/tag.html',
            scope: {
                source: '=',
                showClose: '=',
                onClose: '&'
            },
            link: function (scope) {
                scope.tagColor = function () {
                    var tagFamilyName = typeof scope.source.tagFamily === 'string' ? scope.source.tagFamily : scope.source.tagFamily.name;
                    return mu.stringToColor(tagFamilyName);
                };
                scope.close = function (event) {
                    scope.onClose({ $event: event });
                };
            }
        };
    }
    tagDirective.$inject = ["mu"];
    angular.module('meshAdminUi.common')
        .directive('tag', tagDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Behavioural directive that sets the height of the element to take up
     * the rest of the vertical space of the viewport.
     */
    function fitToViewportDirective($window, $timeout) {
        function fitToViewportLinkFn(scope, element) {
            function setSize() {
                var offsetTop = element[0].getBoundingClientRect().top, windowHeight = $window.innerHeight;
                element[0].style.height = (windowHeight - offsetTop) + 'px';
            }
            $timeout(setSize, 200);
            window.addEventListener('resize', setSize);
        }
        return {
            restrict: 'A',
            link: fitToViewportLinkFn
        };
    }
    fitToViewportDirective.$inject = ["$window", "$timeout"];
    angular.module('meshAdminUi.common')
        .directive('fitToViewport', fitToViewportDirective);
})(meshAdminUi || (meshAdminUi = {}));

angular.module('meshAdminUi.common')
    .controller('mhDropdownController', dropdownController)
    .directive('mhDropdown', dropdownDirective)
    .directive('mhDropdownLabel', dropdownLabelDirective)
    .directive('mhDropdownBody', dropdownBodyDirective);
/**
 * Controller for the mh-dropdown component. The component expects the following markup:
 * TODO: needs refactoring to proper TypeScript form.
 *
 * <mh-dropdown>
 *    <mh-dropdown-label>
 *        Label
 *    </mh-dropdown-label>
 *    <mh-dropdown-body>
 *        <ul>
 *            <li>Option 1</li>
 *            <li>Option 2</li>
 *        </ul>
 *    </mh-dropdown-body>
 * </mh-dropdown>
 *
 * Options:
 * =======
 *
 * sticky: boolean - if true, dropdown will not close unless clicked outside of.
 */
function dropdownController($scope, $element, $document, $window) {
    var vm = this, labelElement, caretHeight = 15;
    /**
     * Tracks the state of the dropdown
     */
    vm.isOpen = false;
    vm.align = vm.align || 'left';
    vm.sticky = vm.sticky || false;
    vm.toggle = toggle;
    vm.setLabelElement = setLabelElement;
    vm.getContentTop = getContentTop;
    /**
     * Toggle the open state of the dropdown and register a
     * global click handler to close.
     */
    function toggle() {
        vm.isOpen = !vm.isOpen;
        if (vm.isOpen) {
            $document.on('click', handleClick);
            angular.element($window).on('scroll resize', handleResizeScroll);
        }
    }
    /**
     * Keep a reference to the label element, used for
     * correct positioning of the dropdown content div.
     * @type {number}
     */
    function setLabelElement(element) {
        labelElement = element;
    }
    /**
     * Get the top position of the content div, which is based on the
     * height of the label element.
     */
    function getContentTop() {
        return labelElement.offsetHeight - caretHeight;
    }
    function handleClick(event) {
        var target = event.target;
        if (!$element[0].contains(target)) {
            closeDropdown();
        }
    }
    function handleResizeScroll() {
        closeDropdown();
    }
    /**
     * Close the dropdown and de-register the global event handlers.
     */
    function closeDropdown() {
        vm.isOpen = false;
        $document.off('click', handleClick);
        angular.element($window).off('scroll resize', handleResizeScroll);
        if (!$scope.$root.$$phase) {
            $scope.$digest();
        }
    }
}
dropdownController.$inject = ["$scope", "$element", "$document", "$window"];
/**
 * The container for the whole dropdown component. Its main purpose is to
 * establish an isolate scope and make the shared controller available
 * to the other parts of the component.
 *
 * @returns {ng.IDirective} Directive definition object
 */
function dropdownDirective() {
    return {
        restrict: 'E',
        template: '<div class="mh-dropdown" ng-transclude></div>',
        controller: 'mhDropdownController',
        controllerAs: 'vm',
        bindToController: true,
        transclude: true,
        replace: true,
        scope: {
            sticky: '=?'
        }
    };
}
/**
 * The dropdown label
 *
 * @returns {ng.IDirective} Directive definition object
 */
function dropdownLabelDirective() {
    function dropdownLabelLink(scope, element, attrs, dropdownCtrl) {
        dropdownCtrl.setLabelElement(element[0]);
        scope.dropdown = dropdownCtrl;
    }
    return {
        restrict: 'E',
        template: '<div class="mh-dropdown-label" ng-click="dropdown.toggle()" ng-class="{ open: dropdown.isOpen }"><a href="" class="a-plain" ng-transclude></a></div>',
        transclude: true,
        replace: true,
        require: '^^mhDropdown',
        link: dropdownLabelLink,
        scope: {}
    };
}
/**
 * The dropdown body.
 *
 * @returns {ng.IDirective} Directive definition object
 */
function dropdownBodyDirective() {
    function linkFn(scope, element, attrs, dropdownCtrl) {
        var arrowGutter = 15;
        var container = element[0];
        var parentContainer = container.parentElement;
        var clickHandler = function (e) {
            e.stopPropagation();
            if (!dropdownCtrl.sticky) {
                scope.$apply(function () {
                    dropdownCtrl.toggle();
                });
            }
        };
        container.remove();
        scope.dropdown = dropdownCtrl;
        scope.$watch('dropdown.isOpen', setPosition);
        scope.$on('$destroy', function () {
            element.off('click', clickHandler);
        });
        element.on('click', clickHandler);
        function setPosition(isOpen) {
            if (isOpen) {
                document.body.appendChild(container);
                var containerChild = container.children[0];
                var parentBox = parentContainer.getBoundingClientRect();
                var childBox = containerChild.getBoundingClientRect();
                var widthDelta = parentBox.width - childBox.width;
                var bodyTop = dropdownCtrl.getContentTop() + parentBox.top + window.pageYOffset + 7;
                var bodyHeight = containerChild.offsetHeight + arrowGutter * 2;
                var bodyWidth = childBox.width + arrowGutter * 2;
                container.style.width = bodyWidth + 'px';
                container.style.height = bodyHeight + 'px';
                /**
                 * Check to see if the dropdown body goes off the edge of the viewport,
                 * and adjust it if so.
                 */
                if ((parentBox.left + widthDelta) < 0) {
                    container.style.left = parentBox.left - arrowGutter + 'px';
                    container.style.right = 'auto';
                    container.classList.remove('left');
                    container.classList.add('right');
                }
                else {
                    container.style.left = (parentBox.left + widthDelta - arrowGutter) + 'px';
                }
                /**
                 * Check to see if the dropdown body goes off the bottom of the viewport,
                 * and adjust it if so.
                 */
                if (window.innerHeight + window.scrollY <= bodyTop + bodyHeight) {
                    console.log('Adjusting body', bodyTop + bodyHeight);
                    bodyTop = bodyTop - bodyHeight;
                    container.classList.remove('top');
                    container.classList.add('bottom');
                }
                else {
                    container.classList.remove('bottom');
                    container.classList.add('top');
                }
                container.style.top = bodyTop + 'px';
                container.classList.add('open');
            }
            else {
                container.remove();
                container.style.height = '0';
                container.classList.remove('open');
            }
        }
    }
    return {
        restrict: 'AE',
        template: '<div class="mh-dropdown-body left"><div class="contents" ng-transclude></div></div>',
        scope: {},
        replace: true,
        transclude: true,
        require: '^^mhDropdown',
        link: linkFn
    };
}

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Service to display a confirmation dialog box, with a confirm and cancel button.
     * By default, it will display generic "delete object"? values, but these can
     * be customised by specifying an `options` object.
     */
    var ConfirmActionDialog = (function () {
        function ConfirmActionDialog($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        ConfirmActionDialog.$inject = ["$mdDialog"];
        /**
         * Show a confirmation dialog box.
         */
        ConfirmActionDialog.prototype.show = function (options) {
            var locals = {
                title: options.title || 'CONFIRM_DELETE_TITLE',
                message: options.message || 'CONFIRM_DELETE_MESSAGE',
                confirmLabel: options.confirmLabel || 'DELETE',
                cancelLabel: options.cancelLabel || 'CANCEL',
                confirmButtonClass: options.confirmButtonClass || 'btn-warn'
            };
            return this.$mdDialog.show({
                templateUrl: 'common/components/dialog/confirmActionDialog.html',
                controller: 'confirmActionDialogController',
                controllerAs: 'vm',
                locals: locals,
                bindToController: true
            });
        };
        return ConfirmActionDialog;
    }());
    meshAdminUi.ConfirmActionDialog = ConfirmActionDialog;
    /**
     * Controller used to set the return value of the close dialog box.
     */
    var ConfirmActionDialogController = (function () {
        function ConfirmActionDialogController($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        ConfirmActionDialogController.$inject = ["$mdDialog"];
        ConfirmActionDialogController.prototype.confirm = function () {
            this.$mdDialog.hide(true);
        };
        ConfirmActionDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return ConfirmActionDialogController;
    }());
    angular.module('meshAdminUi.common')
        .service('confirmActionDialog', ConfirmActionDialog)
        .controller('confirmActionDialogController', ConfirmActionDialogController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var BreadcrumbsController = (function () {
        function BreadcrumbsController($scope, dataService, searchService, dispatcher) {
            var _this = this;
            this.dataService = dataService;
            this.searchService = searchService;
            this.dispatcher = dispatcher;
            this.breadcrumbs = [];
            var changeHandler = function (event, project, node) {
                _this.contextChangeHandler(project, node);
            };
            dispatcher.subscribe(dispatcher.events.contextChanged, changeHandler);
            $scope.$on('$destroy', function () { return dispatcher.unsubscribeAll(changeHandler); });
        }
        BreadcrumbsController.$inject = ["$scope", "dataService", "searchService", "dispatcher"];
        BreadcrumbsController.prototype.isGlobal = function () {
            return this.searchService.getParams().searchAll;
        };
        /**
         * Update the breadcrumbs array whenever the context changes (i.e. user moves to a new
         * node or project).
         */
        BreadcrumbsController.prototype.contextChangeHandler = function (currentProject, currentNode) {
            var _this = this;
            if (!currentProject.name || !currentNode.uuid) {
                this.breadcrumbs = [];
                return;
            }
            this.projectName = currentProject.name;
            if (this.projectName === '') {
                this.breadcrumbs = [];
            }
            else {
                return this.dataService.getNode(currentProject.name, currentNode.uuid)
                    .then(function (node) { return _this.dataService.getBreadcrumb(currentProject, node); })
                    .then(function (breadcrumbs) {
                    var breadcrumbLabels = breadcrumbs.map(function (node) {
                        return {
                            name: node.fields[node.displayField],
                            uuid: node.uuid
                        };
                    });
                    breadcrumbLabels.push({
                        name: currentProject.name,
                        uuid: currentProject.rootNode.uuid
                    });
                    _this.breadcrumbs = breadcrumbLabels.reverse();
                });
            }
        };
        return BreadcrumbsController;
    }());
    /**
     * Directive to generate breadcrumbs for navigating a project.
     */
    function breadcrumbsDirective() {
        return {
            restrict: 'E',
            templateUrl: 'common/components/breadcrumbs/breadcrumbs.html',
            controller: 'breadcrumbsController',
            controllerAs: 'vm',
            scope: {}
        };
    }
    angular.module('meshAdminUi.common')
        .directive('breadcrumbs', breadcrumbsDirective)
        .controller('breadcrumbsController', BreadcrumbsController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Admin menu component.
     *
     * @param adminMenuService
     * @returns {{restrict: string, templateUrl: string, controller: adminMenuController, controllerAs: string, scope: {}, replace: boolean}}
     */
    function adminMenuDirective(adminMenuService) {
        function adminMenuController() {
            var vm = this;
            vm.menu = adminMenuService;
        }
        return {
            restrict: 'E',
            templateUrl: 'common/components/adminMenu/adminMenu.html',
            controller: adminMenuController,
            controllerAs: 'vm',
            scope: {}
        };
    }
    adminMenuDirective.$inject = ["adminMenuService"];
    /**
     * Button which toggles the admin menu open/closed.
     *
     * @param adminMenuService
     * @returns {{restrict: string, controller: adminMenuButtonController, controllerAs: string, template: string, scope: {}}}
     */
    function adminMenuButtonDirective(adminMenuService) {
        function adminMenuButtonController() {
            var vm = this;
            vm.menu = adminMenuService;
        }
        return {
            restrict: 'E',
            controller: adminMenuButtonController,
            controllerAs: 'vm',
            template: '<div class="admin-menu-icon" ng-click="vm.menu.toggle()" ng-class="{ open: vm.menu.isOpen() }"><span></span></div>',
            scope: {}
        };
    }
    adminMenuButtonDirective.$inject = ["adminMenuService"];
    /**
     * This service provides communication between the admin menu button and the menu itself. It holds the open/closed
     * state of the menu, and performs related actions when that state changes to facilitate the working of the
     * component.
     */
    var AdminMenuService = (function () {
        function AdminMenuService($rootScope, $document, $state, contextService) {
            var _this = this;
            this.$rootScope = $rootScope;
            this.$document = $document;
            this.$state = $state;
            this.contextService = contextService;
            this.closeMenuHandler = function (event) {
                _this.closeAdminMenu(event);
            };
            $rootScope.$on('$stateChangeSuccess', function () {
                _this.openIfAdminState();
                if (!_this.isAdminState()) {
                    _this.$document.off('click', _this.closeMenuHandler);
                }
            });
        }
        AdminMenuService.$inject = ["$rootScope", "$document", "$state", "contextService"];
        /**
         * Is the admin menu currently open?
         */
        AdminMenuService.prototype.isOpen = function () {
            return this._isOpen;
        };
        /**
         * Toggle the open/closed state of the admin menu. Also registers/de-registers a
         * document-level click handler to cause auto-hide when not in one of the
         * admin states.
         */
        AdminMenuService.prototype.toggle = function () {
            this._isOpen = !this._isOpen;
            if (this._isOpen) {
                this.$document.on('click', this.closeMenuHandler);
            }
            else {
                this.$document.off('click', this.closeMenuHandler);
                if (this.isAdminState()) {
                    var projectName = this.contextService.getProject().name, nodeId = this.contextService.getCurrentNode().uuid;
                    if (projectName) {
                        this.$state.go('projects.node', { projectName: projectName, nodeId: nodeId });
                    }
                    else {
                        this.$state.go('projectsList');
                    }
                }
            }
        };
        /**
         * Handler to provide auto-hide when clicking off the menu when not in an admin state.
         * @param event
         */
        AdminMenuService.prototype.closeAdminMenu = function (event) {
            var _this = this;
            var target = event.target, appViewport = document.querySelector('.app-viewport');
            if (!this.isAdminState() && (target === appViewport || appViewport.contains(target))) {
                this.$rootScope.$apply(function () { return _this.toggle(); });
            }
        };
        AdminMenuService.prototype.openIfAdminState = function () {
            this._isOpen = this.isAdminState();
        };
        /**
         * Returns true is the current router state is one of the "admin" states as defined in
         * the admin.config.js state definitions.
         */
        AdminMenuService.prototype.isAdminState = function () {
            return !!(this.$state.current.data && this.$state.current.data.isAdminState === true);
        };
        return AdminMenuService;
    }());
    angular.module('meshAdminUi.common')
        .directive('adminMenu', adminMenuDirective)
        .directive('adminMenuButton', adminMenuButtonDirective)
        .service('adminMenuService', AdminMenuService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var TagListController = (function () {
        function TagListController($q, $mdDialog, confirmActionDialog, notifyService, dataService) {
            var _this = this;
            this.$q = $q;
            this.$mdDialog = $mdDialog;
            this.confirmActionDialog = confirmActionDialog;
            this.notifyService = notifyService;
            this.dataService = dataService;
            this.tags = [];
            this.projectGroups = [];
            dataService.getProjects()
                .then(function (response) {
                return $q.all(response.data.map(function (project) {
                    return $q.all({
                        project: $q.when(project),
                        tagFamilies: dataService.getTagFamilies(project.name),
                        tags: dataService.getProjectTags(project.name)
                    });
                }));
            })
                .then(function (result) {
                result.forEach(function (item) {
                    _this.projectGroups.push({
                        project: item.project,
                        tagFamilies: item.tagFamilies.data
                    });
                    var flatten = function (arr) { return arr.reduce(function (arr, t) { return arr.concat(t); }, []); };
                    _this.tags = _this.tags.concat(flatten(item.tags));
                });
            });
        }
        TagListController.$inject = ["$q", "$mdDialog", "confirmActionDialog", "notifyService", "dataService"];
        TagListController.prototype.addTagFamilyDialog = function (project) {
            var _this = this;
            this.showDialog('CREATE_NEW_TAG_FAMILY')
                .then(function (name) {
                var tagFamily = { name: name };
                return _this.dataService.persistTagFamily(project.name, tagFamily);
            })
                .then(function (tagFamily) {
                var index = _this.projectGroups.map(function (group) { return group.project.uuid; }).indexOf(project.uuid);
                _this.projectGroups[index].tagFamilies.push(tagFamily);
                _this.notifyService.toast('TAG_FAMILY_CREATED');
            });
        };
        TagListController.prototype.editTagFamilyDialog = function (project, tagFamily) {
            var _this = this;
            this.showDialog('EDIT_TAG_FAMILY', tagFamily.name)
                .then(function (name) {
                tagFamily.name = name;
                return _this.dataService.persistTagFamily(project.name, tagFamily);
            })
                .then(function (tagFamily) {
                var projectGroup = _this.getProjectGroupIndex(project), index = projectGroup.tagFamilies.map(function (tf) { return tf.uuid; }).indexOf(tagFamily.uuid);
                projectGroup.tagFamilies[index] = tagFamily;
                _this.notifyService.toast('TAG_FAMILY_UPDATED');
            });
        };
        TagListController.prototype.deleteTagFamilyDialog = function (event, project, tagFamily) {
            var _this = this;
            event.stopPropagation();
            event.preventDefault();
            this.confirmActionDialog.show({
                title: 'DELETE_TAG_FAMILY',
                message: 'DELETE_TAG_FAMILY_AND_ALL_TAGS'
            })
                .then(function () {
                return _this.dataService.deleteTagFamily(project.name, tagFamily);
            })
                .then(function () {
                var projectGroup = _this.getProjectGroupIndex(project), index = projectGroup.tagFamilies.map(function (tf) { return tf.uuid; }).indexOf(tagFamily.uuid);
                projectGroup.tagFamilies.splice(index, 1);
                _this.notifyService.toast("TAG_FAMILY_DELETED");
            });
        };
        /**
         * Get the projectgroup object at corresponding to the given project.
         */
        TagListController.prototype.getProjectGroupIndex = function (project) {
            var index = this.projectGroups.map(function (group) { return group.project.uuid; }).indexOf(project.uuid);
            return this.projectGroups[index];
        };
        TagListController.prototype.addTagDialog = function (project, tagFamily) {
            var _this = this;
            this.showDialog('ADD_TAG')
                .then(function (name) {
                var tag = {
                    name: name,
                    tagFamily: {
                        name: tagFamily.name,
                        uuid: tagFamily.uuid
                    }
                };
                return _this.dataService.persistTag(project.name, tag);
            })
                .then(function (tag) {
                _this.tags.push(tag);
                _this.notifyService.toast('NEW_TAG_CREATED');
            });
        };
        TagListController.prototype.editTagDialog = function (project, tag) {
            var _this = this;
            this.showDialog('EDIT_TAG', tag.name)
                .then(function (name) {
                tag.name = name;
                return _this.dataService.persistTag(project.name, tag);
            })
                .then(function (newTag) {
                var index = _this.tags.map(function (tag) { return tag.uuid; }).indexOf(newTag.uuid);
                _this.tags[index] = newTag;
                _this.notifyService.toast("UPDATED_TAG");
            });
        };
        TagListController.prototype.deleteTag = function (event, project, tag) {
            var _this = this;
            event.stopPropagation();
            event.preventDefault();
            return this.dataService.deleteTag(project.name, tag)
                .then(function () {
                var index = _this.tags.map(function (t) { return t.uuid; }).indexOf(tag.uuid);
                _this.tags.splice(index, 1);
                _this.notifyService.toast("DELETED_TAG");
            });
        };
        TagListController.prototype.getTagsByFamily = function (tagFamily) {
            return this.tags.filter(function (tag) { return tag.tagFamily.uuid === tagFamily.uuid; });
        };
        TagListController.prototype.showDialog = function (title, currentName) {
            return this.$mdDialog.show({
                controller: 'TagListDialogController',
                controllerAs: 'vm',
                templateUrl: 'admin/tags/list/tagEditDialog.html',
                locals: {
                    title: title,
                    currentName: currentName
                }
            });
        };
        return TagListController;
    }());
    var TagListDialogController = (function () {
        function TagListDialogController($mdDialog, title, currentName) {
            this.$mdDialog = $mdDialog;
            this.title = title;
            this.currentName = currentName;
            this.name = currentName;
        }
        TagListDialogController.$inject = ["$mdDialog", "title", "currentName"];
        TagListDialogController.prototype.create = function (name) {
            this.$mdDialog.hide(name);
        };
        TagListDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return TagListDialogController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('TagListDialogController', TagListDialogController)
        .controller('TagListController', TagListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var UserListController = (function () {
        function UserListController($q, $timeout, notifyService, mu, dataService) {
            var _this = this;
            this.$q = $q;
            this.$timeout = $timeout;
            this.notifyService = notifyService;
            this.mu = mu;
            this.dataService = dataService;
            this.filterFn = function (value) {
                var filterProps = ['username', 'firstname', 'lastname', 'emailAddress'];
                return _this.mu.matchProps(value, filterProps, _this.userFilter);
            };
            $q.all([
                // TODO: implement paging
                dataService.getGroups({ perPage: 10000 }),
                dataService.getUsers({ perPage: 10000 })
            ])
                .then(function (dataArray) {
                _this.groups = dataArray[0].data;
                _this.users = dataArray[1].data;
            });
        }
        UserListController.$inject = ["$q", "$timeout", "notifyService", "mu", "dataService"];
        UserListController.prototype.validateGroup = function (group, user) {
            var _this = this;
            var userAlreadyInGroup = user.groups.map(function (group) { return group; }).indexOf(group.name) > -1;
            if (userAlreadyInGroup || !group.name) {
                this.notifyService.toast('USER_ALREADY_IN_GROUP', { name: group.name });
                this.$timeout(function () { return user.groups = user.groups.filter(function (group) { return typeof group !== 'undefined'; }); });
            }
            else {
                // group is valid, so add the user to this group.
                this.dataService.addUserToGroup(user.uuid, group.uuid)
                    .then(function () { return _this.notifyService.toast('USER_ADDED_TO_GROUP', { userName: _this.userDisplayName(user), groupName: group.name }); });
                return group;
            }
        };
        UserListController.prototype.removeGroup = function (group, user) {
            var _this = this;
            this.dataService.removeUserFromGroup(user.uuid, group.uuid)
                .then(function () { return _this.notifyService.toast('USER_REMOVED_FROM_GROUP', { userName: _this.userDisplayName(user), groupName: group.name }); });
        };
        UserListController.prototype.userDisplayName = function (user) {
            if (user.firstname && user.lastname) {
                return user.firstname + ' ' + user.lastname;
            }
            else {
                return user.username;
            }
        };
        UserListController.prototype.displayChipName = function (chip) {
            return (chip && chip.name) ? chip.name : chip;
        };
        /**
         * Search for thing.
         */
        UserListController.prototype.filterBy = function (collection, query) {
            return query ? collection.filter(this.createFilterFor(query)) : [];
        };
        /**
         * Create filter function for a query string
         */
        UserListController.prototype.createFilterFor = function (query) {
            var lowercaseQuery = angular.lowercase(query);
            return function (item) { return item.name.toLowerCase().indexOf(lowercaseQuery) === 0; };
        };
        return UserListController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('UserListController', UserListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     *
     */
    var UserDetailController = (function () {
        function UserDetailController($state, $stateParams, confirmActionDialog, dataService, notifyService) {
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.confirmActionDialog = confirmActionDialog;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.isNew = false;
            this.modified = false;
            this.getUserData();
        }
        UserDetailController.$inject = ["$state", "$stateParams", "confirmActionDialog", "dataService", "notifyService"];
        /**
         * Persist the user data back to the server.
         */
        UserDetailController.prototype.persist = function (user) {
            var _this = this;
            this.dataService.persistUser(user)
                .then(function (response) {
                if (_this.isNew) {
                    _this.notifyService.toast('NEW_USER_CREATED');
                    _this.isNew = false;
                    _this.$state.go('admin.users.detail', { uuid: response.uuid });
                }
                else {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.modified = false;
                }
            });
        };
        /**
         * Delete the open content, displaying a confirmation dialog first before making the API call.
         */
        UserDetailController.prototype.remove = function (user) {
            var _this = this;
            this.showDeleteDialog()
                .then(function () {
                return _this.dataService.deleteUser(user);
            })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.$state.go('admin.users.list');
            });
        };
        /**
         * @returns {ng.IPromise}
         */
        UserDetailController.prototype.showDeleteDialog = function () {
            return this.confirmActionDialog.show({
                title: 'Delete User?',
                message: 'Are you sure you want to delete this user?'
            });
        };
        UserDetailController.prototype.canDelete = function () {
            if (this.user) {
                return this.user.permissions && this.user.permissions.delete && !this.isNew;
            }
        };
        /**
         * Get the user data from the server, or in the case of a new user,
         * create an empty user object.
         */
        UserDetailController.prototype.getUserData = function () {
            var _this = this;
            var userId = this.$stateParams.uuid;
            if (userId) {
                return this.dataService.getUser(userId)
                    .then(function (data) { return _this.user = data; });
            }
            else {
                this.user = this.createEmptyUser();
                this.isNew = true;
            }
        };
        /**
         * Create an empty user object.
         */
        UserDetailController.prototype.createEmptyUser = function () {
            return {
                firstname: '',
                lastname: '',
                emailAddress: '',
                username: '',
                password: '',
                groups: []
            };
        };
        return UserDetailController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('UserDetailController', UserDetailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var SchemaListController = (function () {
        function SchemaListController(dataService, mu) {
            var _this = this;
            this.dataService = dataService;
            this.mu = mu;
            this.selected = {};
            this.filterFn = function (value) {
                return _this.mu.matchProps(value, ['name'], _this.schemaFilter);
            };
            // TODO: implement paging
            this.populateSchemas();
        }
        SchemaListController.$inject = ["dataService", "mu"];
        SchemaListController.prototype.populateSchemas = function () {
            var _this = this;
            this.dataService.getSchemas({ perPage: 10000 })
                .then(function (response) { return _this.schemas = response.data; });
        };
        /**
         * Returns false if at least one schema is selected.
         */
        SchemaListController.prototype.selectionEmpty = function () {
            var _this = this;
            return 0 === Object.keys(this.selected).filter(function (key) { return _this.selected[key]; }).length;
        };
        SchemaListController.prototype.clearSelection = function () {
            this.selected = {};
        };
        return SchemaListController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('SchemaListController', SchemaListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     *
     */
    var SchemaDetailController = (function () {
        function SchemaDetailController($state, $stateParams, schemaValidatorService, schemaUpdateService, confirmActionDialog, dataService, notifyService) {
            var _this = this;
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.schemaValidatorService = schemaValidatorService;
            this.schemaUpdateService = schemaUpdateService;
            this.confirmActionDialog = confirmActionDialog;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.isNew = false;
            this.isValid = true;
            this.modified = false;
            this.contentLoaded = false;
            this.lastError = '';
            this.microschemas = [];
            this.schemas = [];
            /**
             * Set up the ACE editor.
             */
            this.aceLoaded = function (editor) {
                editor.setFontSize('14px');
                editor.$blockScrolling = Infinity;
            };
            /**
             * Handle data-binding from the ACE editor instance.
             */
            this.aceChanged = function () {
                if (!_this.contentLoaded) {
                    _this.contentLoaded = true;
                    return;
                }
                _this.modified = true;
                _this.checkErrors();
            };
            this.getSchemaData();
            dataService.getMicroschemas()
                .then(function (response) { return _this.microschemas = response.data; });
            dataService.getSchemas()
                .then(function (response) { return _this.schemas = response.data; });
        }
        SchemaDetailController.$inject = ["$state", "$stateParams", "schemaValidatorService", "schemaUpdateService", "confirmActionDialog", "dataService", "notifyService"];
        /**
         * Persist the user data back to the server.
         */
        SchemaDetailController.prototype.persist = function (schema) {
            var _this = this;
            this.extendSchemaWithJsonValues(this.schemaJson);
            if (this.isNew) {
                this.dataService.createSchema(schema)
                    .then(function (response) {
                    _this.notifyService.toast('NEW_SCHEMA_CREATED');
                    _this.isNew = false;
                    _this.$state.go('admin.schemas.detail', { uuid: response.uuid });
                })
                    .catch(function (error) { return _this.notifyService.toast([error.data.message, error.data.internalMessage]); });
            }
            else {
                this.dataService.forceSchemaUpdate(schema)
                    .then(function () {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.modified = false;
                })
                    .catch(function (error) {
                    if (error) {
                        _this.notifyService.toast([error.data.message, error.data.internalMessage]);
                    }
                });
            }
        };
        /**
         * Delete the open content, displaying a confirmation dialog first before making the API call.
         */
        SchemaDetailController.prototype.remove = function (schema) {
            var _this = this;
            return this.showDeleteDialog()
                .then(function () { return _this.dataService.deleteSchema(schema); })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.$state.go('admin.schemas.list');
            });
        };
        /**
         */
        SchemaDetailController.prototype.showDeleteDialog = function () {
            return this.confirmActionDialog.show({
                title: 'CONFIRM_DELETE_SCHEMA_TITLE',
                message: 'CONFIRM_DELETE_SCHEMA_MESSAGE'
            });
        };
        SchemaDetailController.prototype.canDelete = function () {
            if (this.schema) {
                return this.schema.permissions && this.schema.permissions.delete && !this.isNew;
            }
        };
        /**
         * Callback invoked by the schemaEditor component when changes are made.
         */
        SchemaDetailController.prototype.schemaChanged = function (schema) {
            this.schema = schema;
            this.schemaJson = this.schemaToJson(this.schema);
            this.modified = true;
        };
        /**
         * Get the user data from the server, or in the case of a new user,
         * create an empty user object.
         */
        SchemaDetailController.prototype.getSchemaData = function () {
            var _this = this;
            var uuid = this.$stateParams.uuid;
            if (uuid && uuid !== 'new') {
                return this.dataService.getSchema(uuid)
                    .then(function (data) {
                    _this.schema = data;
                    _this.schemaJson = _this.schemaToJson(_this.schema);
                    _this.checkErrors();
                });
            }
            else {
                this.schema = this.createEmptySchema();
                this.schemaJson = this.schemaToJson(this.schema);
                this.isNew = true;
            }
        };
        /**
         * Converts the schema object into a json string to be used in the editor.
         */
        SchemaDetailController.prototype.schemaToJson = function (schema) {
            var removeHashKey = function (f) {
                delete f.$$hashKey;
                return f;
            };
            var jsonObj = {
                name: schema.name || '',
                displayField: schema.displayField || '',
                segmentField: schema.segmentField || '',
                container: schema.container || false,
                binary: schema.binary || false,
                fields: schema.fields.map(removeHashKey) || []
            };
            return JSON.stringify(jsonObj, null, '\t');
        };
        /**
         * Takes the code from the json editor and merges the values back with the original schema object.
         */
        SchemaDetailController.prototype.extendSchemaWithJsonValues = function (json) {
            var jsonObject = JSON.parse(json);
            angular.extend(this.schema, jsonObject);
            if (this.schema.segmentField === '' || this.schema.segmentField === null) {
                delete this.schema.segmentField;
            }
        };
        /**
         * Run validation on the schema and set the error message and isValid status accordingly
         */
        SchemaDetailController.prototype.checkErrors = function () {
            var _this = this;
            this.lastError = '';
            var setLastError = function (message) { return _this.lastError = message; };
            this.isValid = this.schemaValidatorService.validateSchemaJson(this.schemaJson, setLastError);
        };
        /**
         * When the tab containing the ACE JSON editor is deselected, run validation and display
         * a toast message on any errors.
         */
        SchemaDetailController.prototype.deselectJsonTab = function () {
            var _this = this;
            var displayErrors = function (message) {
                _this.notifyService.toast(message);
            };
            if (!this.isValid) {
                this.schemaValidatorService.validateSchemaJson(this.schemaJson, displayErrors);
            }
            else {
                this.extendSchemaWithJsonValues(this.schemaJson);
            }
        };
        /**
         * Create an empty user object.
         */
        SchemaDetailController.prototype.createEmptySchema = function () {
            return {
                binary: false,
                displayField: '',
                segmentField: '',
                fields: [],
                container: false,
                name: ''
            };
        };
        return SchemaDetailController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('SchemaDetailController', SchemaDetailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var RoleListController = (function () {
        function RoleListController(dataService, mu) {
            var _this = this;
            this.dataService = dataService;
            this.mu = mu;
            this.filterFn = function (value) {
                return _this.mu.matchProps(value, ['name'], _this.rolesFilter);
            };
            // TODO: implement paging
            dataService.getRoles({ perPage: 10000 })
                .then(function (data) { return _this.roles = data.data; });
        }
        RoleListController.$inject = ["dataService", "mu"];
        return RoleListController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('RoleListController', RoleListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var NodeType;
    (function (NodeType) {
        NodeType[NodeType["PROJECT"] = 0] = "PROJECT";
        NodeType[NodeType["SCHEMA"] = 1] = "SCHEMA";
        NodeType[NodeType["MICROSCHEMA"] = 2] = "MICROSCHEMA";
        NodeType[NodeType["GROUP"] = 3] = "GROUP";
        NodeType[NodeType["USER"] = 4] = "USER";
        NodeType[NodeType["ROLE"] = 5] = "ROLE";
        NodeType[NodeType["NODE"] = 6] = "NODE";
    })(NodeType || (NodeType = {}));
    var RoleDetailController = (function () {
        function RoleDetailController($q, $state, $stateParams, mu, confirmActionDialog, notifyService, dataService) {
            var _this = this;
            this.$q = $q;
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.mu = mu;
            this.confirmActionDialog = confirmActionDialog;
            this.notifyService = notifyService;
            this.dataService = dataService;
            this.rootPermissions = {};
            this.tagItemPermissions = {};
            this.tagItemRootPermissions = {};
            this.roleId = $stateParams.uuid;
            this.isNew = this.roleId === undefined || this.roleId === 'new';
            this.modified = false;
            this.queryParams = {
                "role": $stateParams.uuid
            };
            if (!this.isNew) {
                this.getRoleData();
                $q.all([
                    dataService.getSchemas(this.queryParams),
                    dataService.getMicroschemas(this.queryParams),
                    dataService.getProjects(this.queryParams),
                    dataService.getRoles(this.queryParams),
                    dataService.getGroups(this.queryParams),
                    dataService.getUsers(this.queryParams),
                    this.populateTagItems()
                ])
                    .then(function (dataArray) {
                    _this.schemas = dataArray[0].data;
                    _this.microschemas = dataArray[1].data;
                    _this.projects = dataArray[2].data;
                    _this.roles = dataArray[3].data;
                    _this.groups = dataArray[4].data;
                    _this.users = dataArray[5].data;
                    _this.tagItems = dataArray[6];
                    _this.getProjectTagFamilyRootPerms(_this.projects);
                })
                    .then(function () { return _this.setTagItemPermissions(); });
                var types_1 = ['schemas', 'microschemas', 'projects', 'roles', 'groups', 'users'];
                var rootPermPromises = types_1.map(function (type) { return dataService.getPermissions(_this.roleId, type); });
                $q.all(rootPermPromises)
                    .then(function (result) {
                    result.forEach(function (result, index) {
                        _this.rootPermissions[types_1[index]] = result.permissions;
                    });
                });
            }
        }
        RoleDetailController.$inject = ["$q", "$state", "$stateParams", "mu", "confirmActionDialog", "notifyService", "dataService"];
        RoleDetailController.prototype.getProjectTagFamilyRootPerms = function (projects) {
            var _this = this;
            var promises = projects.map(function (project) {
                var path = "projects/" + project.uuid + "/tagFamilies";
                return _this.dataService.getPermissions(_this.roleId, path);
            });
            return this.$q.all(promises)
                .then(function (results) {
                results.forEach(function (result, index) {
                    _this.tagItemRootPermissions[projects[index].uuid] = result;
                });
            });
        };
        /**
         * Populate the this.items array by recur sing through all projects/tagFamilies/tags and flattening the
         * results into an array.
         *
         */
        RoleDetailController.prototype.populateTagItems = function () {
            var _this = this;
            return this.dataService.getProjects(this.queryParams)
                .then(function (response) {
                var promises = [];
                response.data.forEach(function (project) {
                    promises.push(project);
                    promises = promises.concat(_this.populateTagFamilies(project));
                });
                return _this.$q.all(promises);
            })
                .then(function (result) { return _this.mu.flatten(result); });
        };
        /**
         * Return an array of tagFamilies for the project with the tags of each tagFamily following the
         * tagFamily in the array.
         */
        RoleDetailController.prototype.populateTagFamilies = function (project) {
            var _this = this;
            return this.dataService.getTagFamilies(project.name, this.queryParams)
                .then(function (response) {
                var promises = [];
                project.tagFamilies = response.data;
                project.tagFamilies.forEach(function (tagFamily) {
                    promises.push(_this.$q.when(tagFamily));
                    promises = promises.concat([_this.dataService.getTagFamilyTags(project.name, tagFamily.uuid, _this.queryParams)
                            .then(function (result) { return result.data; })]);
                });
                return _this.$q.all(promises);
            });
        };
        RoleDetailController.prototype.setTagItemPermissions = function () {
            var _this = this;
            this.tagItems.forEach(function (item) {
                _this.tagItemPermissions[item.uuid] = item.rolePerms;
            });
        };
        /**
         * Get the project data from the server, or in the case of a new project,
         * create an empty project object.
         */
        RoleDetailController.prototype.getRoleData = function () {
            var _this = this;
            if (this.roleId) {
                return this.dataService.getRole(this.roleId)
                    .then(function (data) { return _this.role = data; });
            }
            else {
                this.role = this.createEmptyRole();
                this.isNew = true;
            }
        };
        RoleDetailController.prototype.persist = function (role) {
            var _this = this;
            this.dataService.persistRole(role)
                .then(function (response) {
                if (_this.isNew) {
                    _this.notifyService.toast('NEW_ROLE_CREATED');
                    _this.isNew = false;
                    _this.$state.go('admin.roles.detail', { uuid: response.uuid });
                }
                else {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.modified = false;
                }
            });
        };
        /**
         * Create an empty project object.
         */
        RoleDetailController.prototype.createEmptyRole = function () {
            return {
                name: '',
                groups: []
            };
        };
        RoleDetailController.prototype.isReadonly = function () {
            return !this.role || this.role.name === 'admin' || !this.role.permissions.update;
        };
        /**
         * Can the current user delete this role?
         */
        RoleDetailController.prototype.canDelete = function () {
            return !!(this.role && this.role.permissions && this.role.permissions.delete);
        };
        /**
         * Delete the role
         */
        RoleDetailController.prototype.remove = function (role) {
            var _this = this;
            this.showDeleteDialog()
                .then(function () { return _this.dataService.deleteRole(role); })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.$state.go('admin.roles.list');
            });
        };
        /**
         * Confirm deletion of the role
         */
        RoleDetailController.prototype.showDeleteDialog = function () {
            return this.confirmActionDialog.show({
                title: 'CONFIRM_DELETE_ROLE_TITLE',
                message: 'CONFIRM_DELETE_ROLE_MESSAGE'
            });
        };
        RoleDetailController.prototype.setTagPermissions = function (project, permissions, tagOrFamily) {
            var _this = this;
            if (tagOrFamily && tagOrFamily.tagFamily) {
                // it is a tag
                return this.dataService.setTagPermissions(this.role.uuid, project.uuid, permissions, tagOrFamily);
            }
            else {
                // it is a tagFamily
                var uuid = tagOrFamily && tagOrFamily.uuid;
                return this.dataService.setTagFamilyPermissions(this.role.uuid, project.uuid, permissions, uuid)
                    .then(function () {
                    if (permissions.recursive) {
                        // we need to update the tags since they have been recursively changed.
                        _this.populateTagItems()
                            .then(function (items) { return _this.tagItems = items; })
                            .then(function () { return _this.setTagItemPermissions(); });
                    }
                });
            }
        };
        RoleDetailController.prototype.setNodePermissions = function (node, project, permissions) {
            var _this = this;
            this.dataService.setNodePermissions(this.role.uuid, project.uuid, node.uuid, permissions)
                .then(function () { return _this.notifyService.toast('PERMISSIONS_SET_ON_NODE', { name: node.fields[node.displayField] }); });
        };
        RoleDetailController.prototype.setProjectPermissions = function (permissions, project) {
            return this.executeSetPermissions(NodeType.PROJECT, permissions, project);
        };
        RoleDetailController.prototype.setSchemaPermissions = function (permissions, schema) {
            return this.executeSetPermissions(NodeType.SCHEMA, permissions, schema);
        };
        RoleDetailController.prototype.setMicroschemaPermissions = function (permissions, microschema) {
            return this.executeSetPermissions(NodeType.MICROSCHEMA, permissions, microschema);
        };
        RoleDetailController.prototype.setUserPermissions = function (permissions, user) {
            return this.executeSetPermissions(NodeType.USER, permissions, user);
        };
        RoleDetailController.prototype.setGroupPermissions = function (permissions, group) {
            return this.executeSetPermissions(NodeType.GROUP, permissions, group);
        };
        RoleDetailController.prototype.setRolePermissions = function (permissions, role) {
            return this.executeSetPermissions(NodeType.ROLE, permissions, role);
        };
        RoleDetailController.prototype.executeSetPermissions = function (type, permissions, node) {
            var _this = this;
            var uuid = (typeof node === 'undefined') ? undefined : node.uuid, name = (typeof node === 'undefined') ? undefined : node.name || node.username, nodeType, promise;
            switch (type) {
                case NodeType.PROJECT:
                    promise = this.dataService.setProjectPermissions(this.role.uuid, permissions, uuid);
                    nodeType = 'project';
                    break;
                case NodeType.SCHEMA:
                    promise = this.dataService.setSchemaPermissions(this.role.uuid, permissions, uuid);
                    nodeType = 'schema';
                    break;
                case NodeType.MICROSCHEMA:
                    promise = this.dataService.setMicroschemaPermissions(this.role.uuid, permissions, uuid);
                    nodeType = 'microschema';
                    break;
                case NodeType.USER:
                    promise = this.dataService.setUserPermissions(this.role.uuid, permissions, uuid);
                    nodeType = 'user';
                    break;
                case NodeType.GROUP:
                    promise = this.dataService.setGroupPermissions(this.role.uuid, permissions, uuid);
                    nodeType = 'group';
                    break;
                case NodeType.ROLE:
                    promise = this.dataService.setRolePermissions(this.role.uuid, permissions, uuid);
                    nodeType = 'role';
                    break;
                default:
            }
            return promise.then(function () { return _this.notifyPermissionsSuccess(nodeType, name); });
        };
        RoleDetailController.prototype.notifyPermissionsSuccess = function (nodeType, name) {
            this.notifyService.toast('PERMISSIONS_SET_ON_TYPE', { type: nodeType, name: name });
        };
        return RoleDetailController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('RoleDetailController', RoleDetailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ProjectListController = (function () {
        function ProjectListController(dataService, mu) {
            var _this = this;
            this.dataService = dataService;
            this.mu = mu;
            this.filterFn = function (value) {
                return _this.mu.matchProps(value, ['name'], _this.projectFilter);
            };
            dataService.getProjects()
                .then(function (response) { return _this.projects = response.data; });
        }
        ProjectListController.$inject = ["dataService", "mu"];
        return ProjectListController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('ProjectListController', ProjectListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     */
    var ProjectDetailController = (function () {
        function ProjectDetailController($q, $state, $stateParams, confirmActionDialog, dataService, notifyService) {
            this.$q = $q;
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.confirmActionDialog = confirmActionDialog;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.isNew = false;
            this.modified = false;
            this.projectSchemas = {};
            this.projectMicroschemas = {};
            this.getProjectData();
        }
        ProjectDetailController.$inject = ["$q", "$state", "$stateParams", "confirmActionDialog", "dataService", "notifyService"];
        /**
         * Persist the project data back to the server.
         */
        ProjectDetailController.prototype.persist = function () {
            var _this = this;
            this.dataService.persistProject(this.project)
                .then(function (response) {
                if (_this.isNew) {
                    _this.notifyService.toast('NEW_PROJECT_CREATED');
                    _this.isNew = false;
                    _this.$state.go('admin.projects.detail', { uuid: response.uuid });
                }
                else {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.modified = false;
                }
            })
                .catch(function (error) {
                _this.notifyService.toast(error.data.message);
            });
        };
        /**
         * Delete the open content, displaying a confirmation dialog first before making the API call.
         */
        ProjectDetailController.prototype.remove = function (project) {
            var _this = this;
            this.showDeleteDialog()
                .then(function () { return _this.dataService.deleteProject(project); })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.$state.go('admin.projects.list');
            });
        };
        /**
         * The project name should not have any spaces in it.
         */
        ProjectDetailController.prototype.nameChanged = function () {
            this.modified = true;
            this.project.name = this.project.name.replace(/\s+/, '');
        };
        /**
         */
        ProjectDetailController.prototype.showDeleteDialog = function () {
            return this.confirmActionDialog.show({
                title: 'CONFIRM_DELETE_PROJECT_TITLE',
                message: 'CONFIRM_DELETE_PROJECT_MESSAGE'
            });
        };
        ProjectDetailController.prototype.canDelete = function () {
            if (this.project) {
                return this.project.permissions && this.project.permissions.delete && !this.isNew;
            }
        };
        /**
         * Get the project data from the server, or in the case of a new project,
         * create an empty project object.
         */
        ProjectDetailController.prototype.getProjectData = function () {
            var _this = this;
            var projectId = this.$stateParams.uuid;
            if (projectId && projectId !== 'new') {
                return this.$q.all([
                    this.dataService.getProject(this.$stateParams.uuid),
                    this.dataService.getSchemas(),
                    this.dataService.getMicroschemas()
                ])
                    .then(function (responses) {
                    _this.project = responses[0];
                    _this.schemas = responses[1].data;
                    _this.microschemas = responses[2].data;
                    return _this.$q.all([
                        _this.dataService.getProjectSchemas(_this.project.name),
                        _this.dataService.getProjectMicroschemas(_this.project.name)
                    ]);
                })
                    .then(function (response) {
                    var currentSchemaUuids = response[0].data.map(function (schema) { return schema.uuid; });
                    _this.schemas.forEach(function (schema) {
                        _this.projectSchemas[schema.uuid] = -1 < currentSchemaUuids.indexOf(schema.uuid);
                    });
                    var currentMicroschemaUuids = response[1].data.map(function (microschema) { return microschema.uuid; });
                    _this.microschemas.forEach(function (microschema) {
                        _this.projectMicroschemas[microschema.uuid] = -1 < currentMicroschemaUuids.indexOf(microschema.uuid);
                    });
                });
            }
            else {
                this.project = this.createEmptyProject();
                this.isNew = true;
            }
        };
        ProjectDetailController.prototype.toggleSchema = function (schema) {
            var _this = this;
            var add = this.projectSchemas[schema.uuid];
            var promise;
            if (add) {
                promise = this.dataService.addSchemaToProject(schema.uuid, this.project.name);
            }
            else {
                promise = this.dataService.removeSchemaFromProject(schema.uuid, this.project.name);
            }
            promise.then(function () {
                var token = add ? 'SCHEMA_ADDED_TO_PROJECT' : 'SCHEMA_REMOVED_FROM_PROJECT';
                _this.notifyService.toast(token, { name: schema.name });
            });
        };
        ProjectDetailController.prototype.toggleMicroschema = function (microschema) {
            var _this = this;
            var add = this.projectMicroschemas[microschema.uuid];
            var promise;
            if (add) {
                promise = this.dataService.addMicroschemaToProject(microschema.uuid, this.project.name);
            }
            else {
                promise = this.dataService.removeMicroschemaFromProject(microschema.uuid, this.project.name);
            }
            promise.then(function () {
                var token = add ? 'MICROSCHEMA_ADDED_TO_PROJECT' : 'MICROSCHEMA_REMOVED_FROM_PROJECT';
                _this.notifyService.toast(token, { name: microschema.name });
            });
        };
        /**
         * Create an empty project object.
         */
        ProjectDetailController.prototype.createEmptyProject = function () {
            return {
                name: '',
                rootNode: {
                    uuid: ''
                },
                schema: {
                    name: 'folder'
                }
            };
        };
        return ProjectDetailController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('ProjectDetailController', ProjectDetailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var MicroschemaListController = (function () {
        function MicroschemaListController(dataService, notifyService, schemaImportExportService, mu) {
            var _this = this;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.schemaImportExportService = schemaImportExportService;
            this.mu = mu;
            this.selected = {};
            this.filterFn = function (value) {
                return _this.mu.matchProps(value, ['name'], _this.microschemaFilter);
            };
            this.populateMicroschemas();
        }
        MicroschemaListController.$inject = ["dataService", "notifyService", "schemaImportExportService", "mu"];
        MicroschemaListController.prototype.populateMicroschemas = function () {
            var _this = this;
            this.dataService.getMicroschemas()
                .then(function (response) { return _this.microschemas = response.data; });
        };
        MicroschemaListController.prototype.clearSelection = function () {
            this.selected = {};
        };
        /**
         * Returns false if at least one schema is selected.
         */
        MicroschemaListController.prototype.selectionEmpty = function () {
            var _this = this;
            return 0 === Object.keys(this.selected).filter(function (key) { return _this.selected[key]; }).length;
        };
        return MicroschemaListController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('MicroschemaListController', MicroschemaListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     *
     */
    var MicroschemaDetailController = (function () {
        function MicroschemaDetailController($state, $stateParams, schemaValidatorService, confirmActionDialog, schemaUpdateService, dataService, notifyService) {
            var _this = this;
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.schemaValidatorService = schemaValidatorService;
            this.confirmActionDialog = confirmActionDialog;
            this.schemaUpdateService = schemaUpdateService;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.isNew = false;
            this.isValid = true;
            this.modified = false;
            this.contentLoaded = false;
            this.lastError = '';
            this.schemas = [];
            /**
             * Set up the ACE editor.
             */
            this.aceLoaded = function (editor) {
                editor.setFontSize('14px');
                editor.$blockScrolling = Infinity;
            };
            /**
             * Handle data-binding from the ACE editor instance.
             */
            this.aceChanged = function () {
                if (!_this.contentLoaded) {
                    _this.contentLoaded = true;
                    return;
                }
                _this.modified = true;
                _this.checkErrors();
            };
            this.getMicroschemaData();
            dataService.getSchemas()
                .then(function (response) { return _this.schemas = response.data; });
        }
        MicroschemaDetailController.$inject = ["$state", "$stateParams", "schemaValidatorService", "confirmActionDialog", "schemaUpdateService", "dataService", "notifyService"];
        /**
         * Persist the user data back to the server.
         */
        MicroschemaDetailController.prototype.persist = function (microschema) {
            var _this = this;
            this.extendMicroschemaWithJsonValues(this.microschemaJson);
            if (this.isNew) {
                this.dataService.createMicroschema(microschema)
                    .then(function (response) {
                    _this.notifyService.toast('NEW_MICROSCHEMA_CREATED');
                    _this.isNew = false;
                    _this.$state.go('admin.microschemas.detail', { uuid: response.uuid });
                })
                    .catch(function (error) { return _this.notifyService.toast([error.data.message, error.data.internalMessage]); });
            }
            else {
                this.dataService.forceMicroschemaUpdate(microschema)
                    .then(function () {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.modified = false;
                })
                    .catch(function (error) {
                    if (error) {
                        _this.notifyService.toast([error.data.message, error.data.internalMessage]);
                    }
                });
            }
        };
        /**
         * Delete the open content, displaying a confirmation dialog first before making the API call.
         */
        MicroschemaDetailController.prototype.remove = function (microschema) {
            var _this = this;
            return this.showDeleteDialog()
                .then(function () { return _this.dataService.deleteMicroschema(microschema); })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.$state.go('admin.microschemas.list');
            });
        };
        /**
         */
        MicroschemaDetailController.prototype.showDeleteDialog = function () {
            return this.confirmActionDialog.show({
                title: 'CONFIRM_DELETE_MICROSCHEMA_TITLE',
                message: 'CONFIRM_DELETE_MICROSCHEMA_MESSAGE'
            });
        };
        MicroschemaDetailController.prototype.canDelete = function () {
            if (this.microschema) {
                return this.microschema.permissions && this.microschema.permissions.delete && !this.isNew;
            }
        };
        /**
         * Callback invoked by the schemaEditor component when changes are made.
         */
        MicroschemaDetailController.prototype.microschemaChanged = function (microschema) {
            this.microschema = microschema;
            this.microschemaJson = this.microschemaToJson(this.microschema);
            this.modified = true;
        };
        /**
         * Get the user data from the server, or in the case of a new user,
         * create an empty user object.
         */
        MicroschemaDetailController.prototype.getMicroschemaData = function () {
            var _this = this;
            var uuid = this.$stateParams.uuid;
            if (uuid && uuid !== 'new') {
                return this.dataService.getMicroschema(uuid)
                    .then(function (data) {
                    _this.microschema = data;
                    _this.microschemaJson = _this.microschemaToJson(_this.microschema);
                    _this.checkErrors();
                });
            }
            else {
                this.microschema = this.createEmptyMicroschema();
                this.microschemaJson = this.microschemaToJson(this.microschema);
                this.isNew = true;
            }
        };
        /**
         * Converts the microschema object into a json string to be used in the editor.
         */
        MicroschemaDetailController.prototype.microschemaToJson = function (microschema) {
            var removeHashKey = function (f) {
                delete f.$$hashKey;
                return f;
            };
            var jsonObj = {
                name: microschema.name || '',
                fields: microschema.fields.map(removeHashKey) || []
            };
            return JSON.stringify(jsonObj, null, '\t');
        };
        /**
         * Takes the code from the json editor and merges the values back with the original microschema object.
         */
        MicroschemaDetailController.prototype.extendMicroschemaWithJsonValues = function (json) {
            var jsonObject = JSON.parse(json);
            angular.extend(this.microschema, jsonObject);
        };
        /**
         * Run validation on the microschema and set the error message and isValid status accordingly
         */
        MicroschemaDetailController.prototype.checkErrors = function () {
            var _this = this;
            this.lastError = '';
            var setLastError = function (message) { return _this.lastError = message; };
            this.isValid = this.schemaValidatorService.validateMichroschemaJson(this.microschemaJson, setLastError);
        };
        /**
         * When the tab containing the ACE JSON editor is deselected, run validation and display
         * a toast message on any errors.
         */
        MicroschemaDetailController.prototype.deselectJsonTab = function () {
            var _this = this;
            var displayErrors = function (message) {
                _this.notifyService.toast(message);
            };
            if (!this.isValid) {
                this.schemaValidatorService.validateSchemaJson(this.microschemaJson, displayErrors);
            }
            else {
                this.extendMicroschemaWithJsonValues(this.microschemaJson);
            }
        };
        /**
         * Create an empty microschema object.
         */
        MicroschemaDetailController.prototype.createEmptyMicroschema = function () {
            return {
                fields: [],
                name: ''
            };
        };
        return MicroschemaDetailController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('MicroschemaDetailController', MicroschemaDetailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var GroupListController = (function () {
        function GroupListController($q, $timeout, notifyService, mu, dataService) {
            var _this = this;
            this.$q = $q;
            this.$timeout = $timeout;
            this.notifyService = notifyService;
            this.mu = mu;
            this.dataService = dataService;
            this.filterFn = function (value) {
                return _this.mu.matchProps(value, ['name'], _this.groupFilter);
            };
            $q.all([
                // TODO: implement paging
                dataService.getGroups({ perPage: 10000 }),
                dataService.getRoles({ perPage: 10000 })
            ])
                .then(function (dataArray) {
                _this.groups = dataArray[0].data;
                _this.roles = dataArray[1].data;
            });
        }
        GroupListController.$inject = ["$q", "$timeout", "notifyService", "mu", "dataService"];
        GroupListController.prototype.validateRole = function (role, group) {
            var _this = this;
            // TODO: group.roles will be a noderef rather than a string.
            var groupAlreadyHasRole = group.roles.map(function (role) { return role; }).indexOf(role.name) > -1;
            if (groupAlreadyHasRole || !role.name) {
                this.notifyService.toast('GROUP_ALREADY_ASSIGNED_TO_ROLE', { name: role.name });
                this.$timeout(function () { return group.roles = group.roles.filter(function (role) { return typeof role !== 'undefined'; }); });
            }
            else {
                this.dataService.addGroupToRole(group.uuid, role.uuid)
                    .then(function () { return _this.notifyService.toast('GROUP_ADDED_TO_ROLE', { groupName: group.name, roleName: role.name }); });
                return role;
            }
        };
        GroupListController.prototype.removeRole = function (role, group) {
            var _this = this;
            this.dataService.removeGroupFromRole(group.uuid, role.uuid)
                .then(function () { return _this.notifyService.toast('GROUP_REMOVED_FROM_ROLE', { groupName: group.name, roleName: role.name }); });
        };
        GroupListController.prototype.displayChipName = function (chip) {
            return (chip && chip.name) ? chip.name : chip;
        };
        /**
         * Search for thing.
         */
        GroupListController.prototype.filterBy = function (collection, query) {
            return query ? collection.filter(this.createFilterFor(query)) : [];
        };
        /**
         * Create filter function for a query string
         */
        GroupListController.prototype.createFilterFor = function (query) {
            var lowercaseQuery = angular.lowercase(query);
            return function (item) { return item.name.toLowerCase().indexOf(lowercaseQuery) === 0; };
        };
        return GroupListController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('GroupListController', GroupListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     *
     */
    var GroupDetailController = (function () {
        function GroupDetailController($state, $stateParams, confirmActionDialog, dataService, notifyService) {
            this.$state = $state;
            this.$stateParams = $stateParams;
            this.confirmActionDialog = confirmActionDialog;
            this.dataService = dataService;
            this.notifyService = notifyService;
            this.isNew = false;
            this.modified = false;
            this.getGroupData();
        }
        GroupDetailController.$inject = ["$state", "$stateParams", "confirmActionDialog", "dataService", "notifyService"];
        /**
         * Persist the user data back to the server.
         */
        GroupDetailController.prototype.persist = function (group) {
            var _this = this;
            this.dataService.persistGroup(group)
                .then(function (response) {
                if (_this.isNew) {
                    _this.notifyService.toast('NEW_GROUP_CREATED');
                    _this.isNew = false;
                    _this.$state.go('admin.groups.detail', { uuid: response.uuid });
                }
                else {
                    _this.notifyService.toast('SAVED_CHANGES');
                    _this.modified = false;
                }
            });
        };
        /**
         * Delete the open content, displaying a confirmation dialog first before making the API call.
         */
        GroupDetailController.prototype.remove = function (group) {
            var _this = this;
            return this.showDeleteDialog()
                .then(function () { return _this.dataService.deleteGroup(group.uuid); })
                .then(function () {
                _this.notifyService.toast('DELETED');
                _this.$state.go('admin.groups.list');
            });
        };
        /**
         */
        GroupDetailController.prototype.showDeleteDialog = function () {
            return this.confirmActionDialog.show({
                title: 'CONFIRM_DELETE_GROUP_TITLE',
                message: 'CONFIRM_DELETE_GROUP_MESSAGE'
            });
        };
        GroupDetailController.prototype.canDelete = function () {
            if (this.group) {
                return this.group.permissions && this.group.permissions.delete && !this.isNew;
            }
        };
        /**
         * Get the user data from the server, or in the case of a new user,
         * create an empty user object.
         */
        GroupDetailController.prototype.getGroupData = function () {
            var _this = this;
            var uuid = this.$stateParams.uuid;
            if (uuid && uuid !== 'new') {
                return this.dataService.getGroup(uuid)
                    .then(function (data) { return _this.group = data; });
            }
            else {
                this.group = this.createEmptyGroup();
                this.isNew = true;
            }
        };
        /**
         * Create an empty user object.
         */
        GroupDetailController.prototype.createEmptyGroup = function () {
            return {
                name: '',
                roles: []
            };
        };
        return GroupDetailController;
    }());
    angular.module('meshAdminUi.admin')
        .controller('GroupDetailController', GroupDetailController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var TagPermissionsSelectorController = (function () {
        function TagPermissionsSelectorController($timeout) {
            var _this = this;
            this.$timeout = $timeout;
            this.collapsed = true;
            this.filter = '';
            this.itemPermissions = {};
            this.rootPermissions = {};
            /**
             * Filter function for matching node names.
             */
            this.filterTags = function (item) {
                var name;
                if (_this.filter !== '') {
                    if (item.name) {
                        name = item.name;
                    }
                    else {
                        name = item.fields.name;
                    }
                    return name.toLowerCase().indexOf(_this.filter.toLowerCase()) > -1;
                }
                else {
                    return true;
                }
            };
            this.queryParams = { "role": this.roleId };
            this.items = this.items || [];
        }
        TagPermissionsSelectorController.$inject = ["$timeout"];
        /**
         * Toggle whether the tags in a tag family should be expanded.
         */
        TagPermissionsSelectorController.prototype.toggleExpand = function (item) {
            if (!item.rootNode && !item.tagFamily) {
                if (this.expand === item.uuid) {
                    this.expand = '';
                }
                else {
                    this.expand = item.uuid;
                }
            }
        };
        TagPermissionsSelectorController.prototype.toggle = function (item, recursive, isProjectRoot) {
            var _this = this;
            if (recursive === void 0) { recursive = false; }
            if (isProjectRoot === void 0) { isProjectRoot = false; }
            this.$timeout(function () {
                var permObject = isProjectRoot ? _this.rootPermissions[item.uuid] : _this.itemPermissions[item.uuid], permissions = {
                    permissions: permObject,
                    recursive: recursive
                };
                if (!isProjectRoot) {
                    var project = _this.getItemProject(item);
                    _this.onToggle({ permissions: permissions, item: item, project: project });
                }
                else {
                    _this.onToggle({ permissions: permissions, project: item });
                }
            });
        };
        /**
         * Given a tag or tag family in the this.items array, return the project that it belongs to
         * (which is equivalent to the closest preceding project in the array).
         */
        TagPermissionsSelectorController.prototype.getItemProject = function (item) {
            var itemIndex = this.items.map(function (item) { return item.uuid; }).indexOf(item.uuid);
            var projectsIndexes = this.items.reduce(function (acc, curr, index) {
                if (curr.rootNode && curr.rootNode.uuid) {
                    acc.push(index);
                }
                return acc;
            }, []);
            var projectIndex = projectsIndexes.filter(function (index) { return index < itemIndex; }).reverse()[0];
            return this.items[projectIndex];
        };
        return TagPermissionsSelectorController;
    }());
    function tagPermissionsSelectorDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/tagPermissionsSelector/tagPermissionsSelector.html',
            controller: 'TagPermissionsSelectorController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                items: '=',
                itemPermissions: '=',
                rootPermissions: '=',
                roleId: '=',
                isReadonly: '=',
                onToggle: '&'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('tagPermissionsSelector', tagPermissionsSelectorDirective)
        .controller('TagPermissionsSelectorController', TagPermissionsSelectorController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var SchemaUpdateService = (function () {
        function SchemaUpdateService($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        SchemaUpdateService.$inject = ["$mdDialog"];
        /**
         * Opens the node schema update dialog in order to show the results of the schema diff.
         */
        SchemaUpdateService.prototype.openDialog = function (schema, changeset) {
            return this.$mdDialog.show({
                templateUrl: 'admin/components/schemaUpdateDialog/schemaUpdateDialog.html',
                controller: 'SchemaUpdateDialogController',
                controllerAs: 'dialog',
                locals: {
                    schema: schema,
                    changeset: changeset
                },
                bindToController: true
            });
        };
        return SchemaUpdateService;
    }());
    meshAdminUi.SchemaUpdateService = SchemaUpdateService;
    var SchemaUpdateDialogController = (function () {
        function SchemaUpdateDialogController($mdDialog) {
            this.$mdDialog = $mdDialog;
        }
        SchemaUpdateDialogController.$inject = ["$mdDialog"];
        SchemaUpdateDialogController.prototype.update = function () {
            this.$mdDialog.hide(this.changeset);
        };
        SchemaUpdateDialogController.prototype.cancel = function () {
            this.$mdDialog.cancel();
        };
        return SchemaUpdateDialogController;
    }());
    var SchemaUpdateFormController = (function () {
        function SchemaUpdateFormController(i18n) {
            this.i18n = i18n;
        }
        SchemaUpdateFormController.$inject = ["i18n"];
        /**
         * Returns a human-readable description of the change (in the imperative mood i.e. "Add a field named 'Foo'")
         */
        SchemaUpdateFormController.prototype.describeChange = function (change) {
            var description = '';
            var props = change.properties;
            switch (change.operation) {
                case 'ADDFIELD':
                    description = this.i18n('ADD_SCHEMA_FIELD_NAMED', { fieldName: props.field });
                    break;
                case 'REMOVEFIELD':
                    description = this.i18n('REMOVE_SCHEMA_FIELD', { fieldName: props.field });
                    break;
                case 'UPDATEFIELD':
                    var keys = Object.keys(props).filter(function (k) { return k !== 'field'; });
                    var values = keys.map(function (k) { return ("\"" + props[k].toString() + "\""); });
                    description = this.i18n('UPDATE_SCHEMA_FIELD', { fieldName: props.field, keys: keys.join(', '), values: values.join(', ') });
                    break;
                case 'CHANGEFIELDTYPE':
                    description = this.i18n('CHANGE_SCHEMA_FIELD_TYPE', { fieldName: props.field, type: props.newType });
                    break;
                case 'UPDATESCHEMA':
                    var key = Object.keys(props)[0];
                    description = this.i18n('UPDATE_SCHEMA_PROPERTY', { key: key, value: props[key] });
                    break;
                default:
                    description = this.i18n('SCHEMA_OPERATION_NOT_RECOGNIZED', { name: change.operation });
            }
            return description;
        };
        /**
         * Returns a CSS class based on the change operation
         */
        SchemaUpdateFormController.prototype.getChangeClass = function (change) {
            switch (change.operation) {
                case 'ADDFIELD':
                    return 'change-add';
                case 'REMOVEFIELD':
                    return 'change-remove';
                case 'CHANGEFIELDTYPE':
                case 'UPDATESCHEMA':
                case 'UPDATEFIELD':
                    return 'change-update';
                default:
                    return '';
            }
        };
        return SchemaUpdateFormController;
    }());
    function schemaUpdateFormDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaUpdateDialog/schemaUpdateForm.html',
            controller: 'SchemaUpdateFormController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                changeset: '='
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .service('schemaUpdateService', SchemaUpdateService)
        .controller('SchemaUpdateDialogController', SchemaUpdateDialogController)
        .controller('SchemaUpdateFormController', SchemaUpdateFormController)
        .directive('schemaUpdateForm', schemaUpdateFormDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var SchemaImportButtonController = (function () {
        function SchemaImportButtonController(schemaImportExportService, notifyService) {
            this.schemaImportExportService = schemaImportExportService;
            this.notifyService = notifyService;
        }
        SchemaImportButtonController.$inject = ["schemaImportExportService", "notifyService"];
        /**
         * Process the imported files.
         */
        SchemaImportButtonController.prototype.processImport = function (files) {
            var _this = this;
            var onErrors = function (errors) { return _this.notifyService.toast(errors); };
            var importFn = function () {
                if (_this.type === 'microschema') {
                    return _this.schemaImportExportService.importMicroschemas(files, _this.currentSchemas, onErrors);
                }
                else {
                    return _this.schemaImportExportService.importSchemas(files, _this.currentSchemas, onErrors);
                }
            };
            importFn().then(function (results) {
                _this.notifyService.toast('IMPORTED_SCHEMAS', { count: results.length });
                if (_this.onImport) {
                    _this.onImport();
                }
            });
        };
        return SchemaImportButtonController;
    }());
    function schemaImportButtonDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaImportButton/schemaImportButton.html',
            controller: 'SchemaImportButtonController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                onImport: '&',
                currentSchemas: '=',
                type: '@'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('schemaImportButton', schemaImportButtonDirective)
        .controller('SchemaImportButtonController', SchemaImportButtonController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var SchemaExportButtonController = (function () {
        function SchemaExportButtonController($scope, schemaImportExportService) {
            var _this = this;
            this.schemaImportExportService = schemaImportExportService;
            this.filename = '';
            $scope.$watchCollection(function () { return _this.selected; }, function () {
                if (_this.schemas instanceof Array) {
                    _this.filename = _this.makeFilename();
                }
            });
        }
        SchemaExportButtonController.$inject = ["$scope", "schemaImportExportService"];
        /**
         * Make a default filename suggestion based on the selected schema(s)
         */
        SchemaExportButtonController.prototype.makeFilename = function () {
            var _this = this;
            var isSchema = this.schemas[0] && this.schemas[0].hasOwnProperty('displayField');
            var prefix = isSchema ? 'mesh-schema-' : 'mesh-microschema-';
            var d = new Date();
            var dateSuffix = "-" + d.getFullYear() + "-" + (d.getMonth() + 1) + "-" + d.getDate();
            var selectedSchemas = this.schemas.filter(function (s) { return _this.selected[s.uuid]; });
            var name = (selectedSchemas.length === 1) ? selectedSchemas[0].name : 'bundle';
            return prefix + name + dateSuffix;
        };
        SchemaExportButtonController.prototype.export = function () {
            this.schemaImportExportService.exportSelected(this.filename, this.schemas, this.selected);
            if (this.onExport) {
                this.onExport();
            }
        };
        return SchemaExportButtonController;
    }());
    function schemaExportButtonDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaExportButton/schemaExportButton.html',
            controller: 'SchemaExportButtonController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                schemas: '=',
                selected: '=',
                disabled: '=',
                onExport: '&'
            },
            transclude: true
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('schemaExportButton', schemaExportButtonDirective)
        .controller('SchemaExportButtonController', SchemaExportButtonController);
})(meshAdminUi || (meshAdminUi = {}));

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Component representing a single editable field in the schema editor.
     */
    var SchemaFieldController = (function () {
        function SchemaFieldController($scope) {
            var _this = this;
            this.types = [
                { name: 'string', icon: 'icon-type' },
                { name: 'number', icon: 'icon-hash' },
                { name: 'html', icon: 'icon-code' },
                { name: 'boolean', icon: 'icon-check-box' },
                { name: 'date', icon: 'icon-event' },
                { name: 'node', icon: 'icon-link' },
                { name: 'list', icon: 'icon-list' },
                { name: 'micronode', icon: 'icon-view-quilt' },
                { name: 'binary', icon: 'icon-attach-file' }
            ];
            this.microschemaAllow = '';
            this.microschemasAllow = [];
            this.schemasAllow = [];
            this.stringsAllow = '';
            $scope.$watch(function () { return _this.field.allow; }, function () {
                var f = _this.field;
                var isMicronode = f.type === 'micronode';
                var isMicronodeList = f.listType && f.listType === 'micronode';
                var isNodeOrNodeList = f.type === 'node' || (f.listType && f.listType === 'node');
                var isStringOrStringList = f.type === 'string' || (f.listType && f.listType === 'string');
                var allowIsArray = f.allow instanceof Array;
                if (isMicronode && allowIsArray) {
                    _this.microschemaAllow = f.allow[0];
                }
                if (isMicronodeList && allowIsArray) {
                    _this.microschemasAllow = f.allow;
                }
                if (isNodeOrNodeList && allowIsArray) {
                    _this.schemasAllow = f.allow;
                }
                if (isStringOrStringList && allowIsArray) {
                    if (_this.stringsAllow.split(',').length !== f.allow.length) {
                        _this.stringsAllow = f.allow.join(', ');
                    }
                }
            });
        }
        SchemaFieldController.$inject = ["$scope"];
        SchemaFieldController.prototype.getTypeIcon = function (field) {
            return this.types.filter(function (type) { return type.name === field.type; })[0].icon;
        };
        /**
         * We cannot use ng-model to directly bind to the "allow" property of a 'micronode', since
         * it would update the "allow" with a string value rather than an array.
         * Therefore we manually create an array with a single string element.
         */
        SchemaFieldController.prototype.updateMicronodeAllow = function (value) {
            this.field.allow = [value];
            this.onChange();
        };
        /**
         * Update the `allow` array, limiting the values only to those that are in the available selection.
         */
        SchemaFieldController.prototype.updateAllowArray = function (values, available) {
            var availableNames = available.map(function (s) { return s.name; });
            this.field.allow = values.filter(function (name) { return -1 < availableNames.indexOf(name); });
            this.onChange();
        };
        SchemaFieldController.prototype.updateStringAllow = function (value) {
            this.field.allow = value.split(',').map(function (s) { return s.trim(); });
            this.onChange();
        };
        return SchemaFieldController;
    }());
    var MicroschemaFieldController = (function (_super) {
        __extends(MicroschemaFieldController, _super);
        function MicroschemaFieldController($scope) {
            _super.call(this, $scope);
            // micronodes and binary fields are not a valid type in a microschema field.
            this.types = this.types.filter(function (type) { return type.name !== 'micronode' && type.name !== 'binary'; });
        }
        MicroschemaFieldController.$inject = ["$scope"];
        return MicroschemaFieldController;
    }(SchemaFieldController));
    function schemaFieldDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaEditor/schemaField.html',
            controller: 'SchemaFieldController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                field: '=',
                schemas: '=',
                microschemas: '=',
                onChange: '&',
                onDelete: '&'
            }
        };
    }
    function microschemaFieldDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaEditor/schemaField.html',
            controller: 'MicroschemaFieldController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                field: '=',
                schemas: '=',
                onChange: '&',
                onDelete: '&'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('schemaField', schemaFieldDirective)
        .directive('microschemaField', microschemaFieldDirective)
        .controller('SchemaFieldController', SchemaFieldController)
        .controller('MicroschemaFieldController', MicroschemaFieldController);
})(meshAdminUi || (meshAdminUi = {}));

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var meshAdminUi;
(function (meshAdminUi) {
    /**
     * A common controller for both the schemaEditor and microschemaEditor. Most of the functionality is shared, except
     * that in the case of microschemas, certain fields are not needed.
     */
    var SchemaEditorBaseController = (function () {
        function SchemaEditorBaseController() {
        }
        /**
         * Add a new empty field and update the schema.
         */
        SchemaEditorBaseController.prototype.addField = function () {
            this.baseSchema.fields.push({
                name: '',
                label: '',
                type: 'string',
                required: false
            });
            this.schemaChanged();
        };
        /**
         * Remove the field at index from the schema
         */
        SchemaEditorBaseController.prototype.removeField = function (index) {
            this.baseSchema.fields.splice(index, 1);
            this.schemaChanged();
        };
        /**
         * Invoke the onChange callback and pass it the updated schema.
         */
        SchemaEditorBaseController.prototype.schemaChanged = function () {
            this.baseSchema.fields.forEach(this.cleanFields);
            this.onChange({ schema: this.baseSchema });
        };
        /**
         * Removes unused properties from a field definition.
         */
        SchemaEditorBaseController.prototype.cleanFields = function (field) {
            if (field.type !== 'list') {
                delete field.listType;
            }
            if (field.type !== 'micronode' && field.listType !== 'micronode' &&
                field.type !== 'node' && field.listType !== 'node' &&
                field.type !== 'string' && field.listType !== 'string') {
                delete field.allow;
            }
            if (field.type !== 'number' && field.listType !== 'number') {
                delete field.min;
                delete field.max;
                delete field.step;
            }
            if (field.hasOwnProperty('$$mdSelectId')) {
                delete field['$$mdSelectId'];
            }
            return field;
        };
        ;
        /**
         * Filter for fields which are valid displayField values,
         * which may be used as the value of the `displayName` or `segmentName` property.
         *
         * TODO: allow other field types: https://jira.gentics.com/browse/CL-408
         */
        SchemaEditorBaseController.prototype.validDisplayFieldsFilter = function (field) {
            var validTypes = ['string'];
            return -1 < validTypes.indexOf(field.type);
        };
        /**
         * Filter for fields which are valid displayField values,
         * which may be used as the value of the `displayName` or `segmentName` property.
         *
         * TODO: allow other field types: https://jira.gentics.com/browse/CL-408
         */
        SchemaEditorBaseController.prototype.validSegmentFieldsFilter = function (field) {
            var validTypes = ['string', 'binary'];
            return -1 < validTypes.indexOf(field.type);
        };
        /**
         * Record the index of the item that is being dragged.
         */
        SchemaEditorBaseController.prototype.startDrag = function (index) {
            this.dragStartIndex = index;
        };
        /**
         * Remove the original dragged item from the list.
         */
        SchemaEditorBaseController.prototype.endDrag = function (item, dragEndIndex, list) {
            var indexToSplice;
            if (dragEndIndex < this.dragStartIndex) {
                indexToSplice = this.dragStartIndex + 1;
            }
            else {
                indexToSplice = this.dragStartIndex;
            }
            list.splice(dragEndIndex, 0, item); // add the new position
            list.splice(indexToSplice, 1); // remove the old position
            this.schemaChanged();
        };
        return SchemaEditorBaseController;
    }());
    var SchemaEditorController = (function (_super) {
        __extends(SchemaEditorController, _super);
        function SchemaEditorController($scope) {
            var _this = this;
            _super.call(this);
            var unwatch = $scope.$watch(function () { return _this.schema; }, function (val) {
                if (val !== undefined) {
                    _this.baseSchema = val;
                    _this.displayFieldValue = _this.schema.fields.filter(function (f) { return f.name === _this.schema.displayField; })[0];
                    _this.segmentFieldValue = _this.schema.fields.filter(function (f) { return f.name === _this.schema.segmentField; })[0];
                    unwatch();
                }
            });
        }
        SchemaEditorController.$inject = ["$scope"];
        SchemaEditorController.prototype.updateDisplayField = function () {
            this.schema.displayField = this.displayFieldValue.name;
            this.schemaChanged();
        };
        SchemaEditorController.prototype.updateSegmentField = function () {
            this.schema.segmentField = this.segmentFieldValue.name;
            this.schemaChanged();
        };
        return SchemaEditorController;
    }(SchemaEditorBaseController));
    var MicroschemaEditorController = (function (_super) {
        __extends(MicroschemaEditorController, _super);
        function MicroschemaEditorController($scope) {
            var _this = this;
            _super.call(this);
            var unwatch = $scope.$watch(function () { return _this.microschema; }, function (val) {
                if (val !== undefined) {
                    _this.baseSchema = val;
                    unwatch();
                }
            });
        }
        MicroschemaEditorController.$inject = ["$scope"];
        return MicroschemaEditorController;
    }(SchemaEditorBaseController));
    function schemaEditorDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaEditor/schemaEditor.html',
            controller: 'SchemaEditorController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                schema: '=',
                schemas: '=',
                microschemas: '=',
                onChange: '&'
            }
        };
    }
    function microschemaEditorDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/schemaEditor/microschemaEditor.html',
            controller: 'MicroschemaEditorController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                microschema: '=',
                schemas: '=',
                onChange: '&'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('schemaEditor', schemaEditorDirective)
        .directive('microschemaEditor', microschemaEditorDirective)
        .controller('SchemaEditorController', SchemaEditorController)
        .controller('MicroschemaEditorController', MicroschemaEditorController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    function recursiveToggleCheckboxDirective() {
        function recursiveToggleCheckboxController() {
            var vm = this;
            vm.recursiveClick = recursiveClick;
            vm.changed = changed;
            function recursiveClick() {
                vm.model = !vm.model;
                vm.onRecursiveClick();
            }
            function changed($event) {
                vm.onChange({ $event: $event });
            }
        }
        return {
            restrict: 'E',
            templateUrl: 'admin/components/recursiveToggleCheckbox/recursiveToggleCheckbox.html',
            controller: recursiveToggleCheckboxController,
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                model: '=',
                label: '@',
                onChange: '&',
                ngDisabled: '=',
                onRecursiveClick: '&'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('recursiveToggleCheckbox', recursiveToggleCheckboxDirective);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var PermissionsTableController = (function () {
        function PermissionsTableController($scope, mu) {
            var _this = this;
            this.$scope = $scope;
            this.mu = mu;
            this.collapsed = true;
            this.filter = '';
            this.itemPermissions = {};
            /**
             * Filter by item name.
             */
            this.itemNameFilter = function (item) {
                var name;
                if (_this.filter !== '') {
                    name = _this.displayItemName(item);
                    return name.toLowerCase().indexOf(_this.filter.toLowerCase()) > -1;
                }
                else {
                    return true;
                }
            };
            /**
             * The `item` array will be populated async when the data arrives from the server call, so we
             * need to watch it and once it is populated, process the array and cancel the watcher.
             */
            var cancelItemsWatcher = $scope.$watch(function () { return _this.items; }, function (newVal) {
                if (newVal) {
                    _this.items = newVal;
                    _this.items.forEach(function (item) {
                        _this.itemPermissions[item.uuid] = item.rolePerms;
                    });
                    cancelItemsWatcher();
                }
            });
            this.rootPermissions = this.rootPermissions || {};
        }
        PermissionsTableController.$inject = ["$scope", "mu"];
        /**
         * Evaluate the expression provided in the `itemNameField` field against the provided
         * `item` object and return the result.
         */
        PermissionsTableController.prototype.displayItemName = function (item) {
            return this.$scope.$eval(this.itemNameField, { item: item });
        };
        PermissionsTableController.prototype.toggle = function (item) {
            var permObject = this.itemPermissions[item.uuid], permissions = {
                permissions: permObject,
                recursive: false
            };
            this.onToggle({ permissions: permissions, item: item });
        };
        PermissionsTableController.prototype.toggleRootPerm = function () {
            var permissions = {
                permissions: this.rootPermissions,
                recursive: false
            };
            this.onToggle({ permissions: permissions });
        };
        return PermissionsTableController;
    }());
    function permissionsTableDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/permissionsTable/permissionsTable.html',
            controller: 'PermissionsTableController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                isReadonly: '=',
                rootName: '@',
                rootPermissions: '=',
                items: '=',
                itemNameField: '@',
                onToggle: '&'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('permissionsTable', permissionsTableDirective)
        .controller('PermissionsTableController', PermissionsTableController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var NodePermissionsSelectorController = (function () {
        function NodePermissionsSelectorController(dataService, $q, $timeout, mu) {
            var _this = this;
            this.dataService = dataService;
            this.$q = $q;
            this.$timeout = $timeout;
            this.mu = mu;
            this.currentProject = null;
            this.currentNode = null;
            this.breadcrumbsBase = [{
                    name: 'Projects',
                    uuid: null
                }];
            this.collapsed = true;
            this.filter = '';
            this.nodes = [];
            this.nodePermissions = {};
            /**
             * Filter function for matching node names.
             */
            this.filterNodes = function (node) {
                var name;
                if (_this.filter !== '') {
                    if (_this.isProjectNode(node)) {
                        name = node.name;
                    }
                    else {
                        name = node.fields[node.displayField];
                    }
                    return name.toLowerCase().indexOf(_this.filter.toLowerCase()) > -1;
                }
                else {
                    return true;
                }
            };
            this.queryParams = { 'role': this.roleId };
            this.populateContents();
        }
        NodePermissionsSelectorController.$inject = ["dataService", "$q", "$timeout", "mu"];
        /**
         * Open the selected node and display its child folders and contents.
         */
        NodePermissionsSelectorController.prototype.openNode = function (event, node) {
            event.preventDefault();
            if (this.isProjectNode(node)) {
                this.currentProject = node;
                this.currentNode = { uuid: node.rootNode.uuid };
            }
            else if (node.project) {
                this.currentProject = node.project;
                this.currentNode = node;
            }
            else {
                this.currentNode = node;
            }
            this.populateContents();
        };
        /**
         * Fetch the folders and contents which are children of the currentNodeId, and
         * concatenate into a single array, and populate the breadcrumbs.
         */
        NodePermissionsSelectorController.prototype.populateContents = function () {
            var _this = this;
            this.nodes.length = 0;
            if (this.currentNode === null || this.currentNode.uuid === null) {
                this.loadProjects();
            }
            else {
                this.dataService.getChildNodes(this.currentProject.name, this.currentNode.uuid, this.queryParams)
                    .then(function (data) {
                    _this.nodes = data.data;
                    _this.createNodePermissionsArray(_this.nodes);
                    return _this.dataService.getBreadcrumb(_this.currentProject, _this.currentNode);
                })
                    .then(function (breadcrumbs) { return _this.populateBreadcrumbs(breadcrumbs); });
            }
        };
        /**
         * Fetch all projects and then get the root node for each, and load the rootNodes into the nodes collection.
         */
        NodePermissionsSelectorController.prototype.loadProjects = function () {
            var _this = this;
            var projects;
            this.dataService.getProjects(this.queryParams)
                .then(function (response) {
                projects = response.data;
                return _this.$q.all(projects.map(function (project) {
                    return _this.dataService.getNode(project.name, project.rootNode.uuid, _this.queryParams);
                }));
            })
                .then(function (nodes) {
                _this.nodes = nodes;
                _this.nodes.forEach(function (node, i) {
                    node.name = projects[i].name;
                    node.project = projects[i];
                });
                _this.createNodePermissionsArray(_this.nodes);
                _this.breadcrumbs = _this.breadcrumbsBase;
            });
        };
        /**
         * populate the NodePermissions object
         * @param nodes
         */
        NodePermissionsSelectorController.prototype.createNodePermissionsArray = function (nodes) {
            var _this = this;
            this.nodePermissions = {};
            nodes.forEach(function (node) {
                _this.nodePermissions[node.uuid] = node.rolePerms;
            });
        };
        /**
         * Create the breadcrumbs array by concatenating the fetched breadcrumb data with
         *  the `breadcrumbBase` array.
         */
        NodePermissionsSelectorController.prototype.populateBreadcrumbs = function (data) {
            var _this = this;
            var breadcrumbs = this.breadcrumbsBase.slice(0);
            if (!this.currentNode.project) {
                breadcrumbs = this.breadcrumbsBase.concat([this.currentProject]);
            }
            this.breadcrumbs = breadcrumbs.concat(data.map(function (item) { return _this.replaceRootnodeWithProjectName(item); }));
        };
        /**
         * Used to map over the breadcrumb array and replace the rootNode with the
         * current project name.
         */
        NodePermissionsSelectorController.prototype.replaceRootnodeWithProjectName = function (item) {
            if (item.name === 'rootNode') {
                item.name = this.currentProject.name;
            }
            return item;
        };
        NodePermissionsSelectorController.prototype.isProjectNode = function (node) {
            return node.hasOwnProperty('rootNode');
        };
        /**
         * Call the onToggle function with the required args. It is inside a $timeout() to ensure that the
         * model has had a chance to update before making the call.
         */
        NodePermissionsSelectorController.prototype.toggle = function (node, recursive) {
            var _this = this;
            this.$timeout(function () {
                var permObject = _this.nodePermissions[node.uuid];
                var permissions = {
                    permissions: permObject,
                    recursive: recursive
                };
                var project = (node.project) ? node.project : _this.currentProject;
                _this.onToggle({ node: node, project: project, permissions: permissions });
            });
        };
        return NodePermissionsSelectorController;
    }());
    function nodePermissionsSelectorDirective() {
        return {
            restrict: 'E',
            templateUrl: 'admin/components/nodePermissionsSelector/nodePermissionsSelector.html',
            controller: 'nodePermissionsSelectorController',
            controllerAs: 'vm',
            bindToController: true,
            scope: {
                isReadonly: '=',
                roleId: '=',
                onToggle: '&'
            }
        };
    }
    angular.module('meshAdminUi.admin')
        .directive('nodePermissionsSelector', nodePermissionsSelectorDirective)
        .controller('nodePermissionsSelectorController', NodePermissionsSelectorController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var ProjectsListController = (function () {
        function ProjectsListController(dataService) {
            this.dataService = dataService;
            this.populate();
        }
        ProjectsListController.$inject = ["dataService"];
        ProjectsListController.prototype.populate = function () {
            var _this = this;
            this.dataService.getProjects()
                .then(function (data) {
                _this.projects = data.data;
            });
        };
        return ProjectsListController;
    }());
    angular.module('meshAdminUi.projects')
        .controller('ProjectsListController', ProjectsListController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     *
     */
    var ProjectExplorerController = (function () {
        function ProjectExplorerController($scope, $q, dataService, searchService, mu, contextService, dispatcher, currentNode) {
            var _this = this;
            this.$scope = $scope;
            this.$q = $q;
            this.dataService = dataService;
            this.searchService = searchService;
            this.mu = mu;
            this.contextService = contextService;
            this.dispatcher = dispatcher;
            this.currentNode = currentNode;
            this.itemsPerPage = 10;
            this.contents = [];
            this.childrenSchemas = [];
            this.tagsArray = {};
            this.projectName = contextService.getProject().name;
            this.createPermission = currentNode.permissions.create;
            var updateContents = function () {
                dataService.getNode(contextService.getProject().name, currentNode.uuid)
                    .then(function (node) {
                    _this.currentNode = node;
                    return _this.getChildrenSchemas();
                })
                    .then(function () { return _this.populateChildNodes(); });
            };
            var updateNode = function (event, nodeUuid) {
                dataService.getNode(contextService.getProject().name, nodeUuid)
                    .then(function (node) {
                    var bundleIndex = _this.contents.map(function (bundle) { return bundle.schema.uuid; }).indexOf(node.schema.uuid);
                    if (-1 < bundleIndex) {
                        var nodeIndex = _this.contents[bundleIndex].data.map(function (node) { return node.uuid; }).indexOf(node.uuid);
                        _this.contents[bundleIndex].data[nodeIndex] = node;
                        _this.tagsArray[node.uuid] = node.tags;
                    }
                });
            };
            var searchParamsHandler = function () { return _this.populateChildNodes(); };
            dispatcher.subscribe(dispatcher.events.explorerSearchParamsChanged, searchParamsHandler);
            dispatcher.subscribe(dispatcher.events.explorerContentsChanged, updateContents);
            dispatcher.subscribe(dispatcher.events.explorerNodeTagsChanged, updateNode);
            $scope.$on('$destroy', function () { return dispatcher.unsubscribeAll(updateContents, searchParamsHandler, updateNode); });
            this.getChildrenSchemas()
                .then(function () { return _this.populateChildNodes(); });
        }
        ProjectExplorerController.$inject = ["$scope", "$q", "dataService", "searchService", "mu", "contextService", "dispatcher", "currentNode"];
        /**
         * The node.childrenInfo property contains the name and uuid of the children's schemas, but we need the
         * full schema object in order to do proper search queries, since we need to know the displayField of
         * each child type.
         */
        ProjectExplorerController.prototype.getChildrenSchemas = function () {
            var _this = this;
            var promises = Object.keys(this.currentNode.childrenInfo).map(function (schemaName) {
                return _this.dataService.getSchema(_this.currentNode.childrenInfo[schemaName].schemaUuid);
            });
            return this.$q.all(promises)
                .then(function (results) {
                _this.childrenSchemas = results;
            });
        };
        /**
         * Converts the tags object for each node in the bundle into an array
         * and stores them in this.tagsArray[nodeUuid].
         */
        ProjectExplorerController.prototype.populateTagsArray = function (bundles) {
            var _this = this;
            return bundles.forEach(function (bundle) {
                bundle.data.forEach(function (node) {
                    _this.tagsArray[node.uuid] = node.tags;
                });
            });
        };
        /**
         * Fill the vm with the child children of the current node.
         */
        ProjectExplorerController.prototype.populateChildNodes = function () {
            var _this = this;
            var projectName = this.contextService.getProject().name, searchParams = this.searchService.getParams(), queryParams = {
                perPage: this.itemsPerPage
            };
            var bundleParams = this.childrenSchemas.map(function (schemaRef) {
                return {
                    schema: schemaRef,
                    page: 1
                };
            });
            return this.dataService.getNodeBundles(projectName, this.currentNode, bundleParams, searchParams, queryParams)
                .then(function (response) {
                _this.contents = response;
                _this.populateTagsArray(response);
            });
        };
        /**
         * Reload a single bundle with a new page of data and update the contents list.
         */
        ProjectExplorerController.prototype.pageChanged = function (newPageNumber, schemaUuid) {
            var _this = this;
            var bundleParams = [{
                    schema: this.childrenSchemas.filter(function (schema) { return schema.uuid === schemaUuid; })[0],
                    page: newPageNumber
                }];
            var searchParams = this.searchService.getParams();
            return this.dataService.getNodeBundles(this.projectName, this.currentNode, bundleParams, searchParams, {
                perPage: this.itemsPerPage
            }).then(function (result) {
                var index = _this.contents.map(function (bundle) { return bundle.schema.uuid; }).indexOf(schemaUuid);
                _this.contents[index].data = result[0].data;
                _this.contents[index]._metainfo = result[0]._metainfo;
            });
        };
        return ProjectExplorerController;
    }());
    angular.module('meshAdminUi.projects')
        .controller('ProjectExplorerController', ProjectExplorerController);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Specify the prefix for the app's localStorage entries.
     */
    function localStorageConfig(localStorageServiceProvider) {
        localStorageServiceProvider.setPrefix('mesh');
    }
    localStorageConfig.$inject = ["localStorageServiceProvider"];
    /**
     * Service to track work-in-progress (WIP) items, i.e. objects which have
     * been opened for editing but have not had their changes persisted back
     * to the server. This service allows the user to edit multiple open objects
     * at once, and tracks whether they need to be persisted.
     *
     * In any of these methods that have an `item` requirement, `item` must be an
     * object with a `uuid` property.
     */
    var WipService = (function () {
        function WipService($q, i18nService, localStorageService, mu, dispatcher) {
            this.$q = $q;
            this.i18nService = i18nService;
            this.localStorageService = localStorageService;
            this.mu = mu;
            this.dispatcher = dispatcher;
            this.wipStore = {};
            this.modifiedWips = {};
            this.populateFromLocalStorage();
        }
        WipService.$inject = ["$q", "i18nService", "localStorageService", "mu", "dispatcher"];
        /**
         * Generates a temporary unique ID for newly-created items so that
         * they can be referenced before they are saved to the server (at
         * which point they should get a new, server-generated ID).
         */
        WipService.prototype.generateTempId = function () {
            return this.mu.generateGuid();
        };
        /**
         * Add a new wip item of specified type to the store.
         * Metadata is an optional argument which can be used to store
         * arbitrary key-value data that can be retrieved later.
         */
        WipService.prototype.openItem = function (type, item, metadata) {
            this.validateItem(item);
            var lang = this.i18nService.getCurrentLang().code;
            if (!this.wipStore[lang][type]) {
                this.wipStore[lang][type] = {};
                this.modifiedWips[lang][type] = {};
            }
            if (this.wipStore[lang][type][item.uuid]) {
                console.warn('wipStore#openItem: "' + type + '" with uuid "' +
                    item.uuid + '" is already open.');
            }
            this.wipStore[lang][type][item.uuid] = {
                item: item,
                metadata: metadata || {}
            };
            this.dispatcher.publish(this.dispatcher.events.wipsChanged);
        };
        /**
         * Updates an open wip item by extending it with the properties of the new object.
         * Preserves the object reference.
         */
        WipService.prototype.updateItem = function (type, item) {
            this.validateItem(item);
            if (!this.isOpen(type, item.uuid)) {
                throw new Error('wipStore#updateItem: "' + type + '" with uuid "' +
                    item.uuid + '" is not open in the wipStore.');
            }
            angular.extend(this.getItem(type, item.uuid), item);
        };
        /**
         * Set a key-value pair on the wip item's metadata object.
         */
        WipService.prototype.setMetadata = function (type, uuid, key, value) {
            var lang = this.i18nService.getCurrentLang().code;
            if (this.wipStore[lang][type] && this.wipStore[lang][type][uuid]) {
                this.wipStore[lang][type][uuid].metadata[key] = value;
            }
        };
        /**
         * Get the metadata associated with the wip item.
         */
        WipService.prototype.getMetadata = function (type, uuid) {
            var lang = this.i18nService.getCurrentLang().code;
            this.checkItemInWipStore(type, uuid);
            return this.wipStore[lang][type][uuid].metadata;
        };
        /**
         * Mark the item as having been modified,
         * which determines its behaviour on closing.
         */
        WipService.prototype.setAsModified = function (type, item) {
            var lang = this.i18nService.getCurrentLang().code;
            this.validateItem(item);
            this.checkItemInWipStore(type, item);
            this.modifiedWips[lang][type][item.uuid] = true;
            this.dispatcher.publish(this.dispatcher.events.wipsChanged);
        };
        /**
         * Set the item to be unmodified. Useful in the situation where a modified item
         * is persisted to the server, but we want to keep it open in the wipStore but
         * indicate that it is unmodified and okay to close.
         */
        WipService.prototype.setAsUnmodified = function (type, item) {
            var lang = this.i18nService.getCurrentLang().code;
            this.validateItem(item);
            this.checkItemInWipStore(type, item);
            this.modifiedWips[lang][type][item.uuid] = false;
            this.dispatcher.publish(this.dispatcher.events.wipsChanged);
        };
        /**
         * Checks to see if the item in the store has been modified.
         */
        WipService.prototype.isModified = function (type, item) {
            var lang = this.i18nService.getCurrentLang().code;
            this.validateItem(item);
            this.checkItemInWipStore(type, item);
            return this.modifiedWips[lang][type][item.uuid] === true;
        };
        /**
         * Remove an existing item of the specified type from the store.
         */
        WipService.prototype.closeItem = function (type, item) {
            var lang = this.i18nService.getCurrentLang().code;
            var deferred = this.$q.defer();
            this.validateItem(item);
            this.checkItemInWipStore(type, item);
            delete this.wipStore[lang][type][item.uuid];
            delete this.modifiedWips[lang][type][item.uuid];
            deferred.resolve(item);
            this.dispatcher.publish(this.dispatcher.events.wipsChanged);
            return deferred.promise;
        };
        /**
         * Get an array of all open items of the specified type.
         */
        WipService.prototype.getOpenItems = function (type) {
            var lang = this.i18nService.getCurrentLang().code;
            var itemsArray = [];
            if (this.wipStore[lang] && this.wipStore[lang][type]) {
                for (var key in this.wipStore[lang][type]) {
                    if (this.wipStore[lang][type].hasOwnProperty(key)) {
                        itemsArray.push(this.wipStore[lang][type][key]);
                    }
                }
            }
            return itemsArray;
        };
        /**
         * Is the item of the given type and UUID open in the wip store?
         */
        WipService.prototype.isOpen = function (type, uuid) {
            var lang = this.i18nService.getCurrentLang().code;
            return !!(this.wipStore[lang][type] && this.wipStore[lang][type].hasOwnProperty(uuid));
        };
        /**
         * Get an array of uuids of each item of the specified type which has been modified.
         */
        WipService.prototype.getModifiedItems = function (type) {
            var lang = this.i18nService.getCurrentLang().code;
            var itemsArray = [];
            if (this.modifiedWips[lang] && this.modifiedWips[lang][type]) {
                for (var key in this.modifiedWips[lang][type]) {
                    if (this.modifiedWips[lang][type].hasOwnProperty(key) && this.modifiedWips[lang][type][key] === true) {
                        itemsArray.push(key);
                    }
                }
            }
            return itemsArray;
        };
        /**
         * Returns the wip content specified by the uuid if it exists in the contents store.
         */
        WipService.prototype.getItem = function (type, uuid) {
            var lang = this.i18nService.getCurrentLang().code;
            return this.wipStore[lang][type] && this.wipStore[lang][type][uuid] && this.wipStore[lang][type][uuid].item;
        };
        /**
         * Throws an exception if the item object does not have a `uuid` property.
         */
        WipService.prototype.validateItem = function (item) {
            if (!item.uuid) {
                throw new Error('wipService: `item` does not have a `uuid` property.');
            }
        };
        /**
         * Check to see whether the item exists in the wipStore object. If not, an exception
         * is thrown.
         */
        WipService.prototype.checkItemInWipStore = function (type, itemOrUuid) {
            var lang = this.i18nService.getCurrentLang().code;
            var id = typeof itemOrUuid === 'string' ? itemOrUuid : itemOrUuid.uuid;
            if (!this.wipStore[lang][type]) {
                throw new Error('wipService: the wipStore does not contain any items of the type "' + type + '".');
            }
            else if (!this.wipStore[lang][type][id]) {
                throw new Error('wipService: item of type "' + type + '" with uuid "' + id + '" not found.');
            }
        };
        /**
         * Load any wips which have been persisted to localStorage via persistToLocalStorage().
         */
        WipService.prototype.populateFromLocalStorage = function () {
            var _this = this;
            var localWipstore = this.localStorageService.get('wipStore'), localModifiedWips = this.localStorageService.get('modifiedWips');
            if (localWipstore) {
                this.wipStore = localWipstore;
            }
            if (localModifiedWips) {
                this.modifiedWips = localModifiedWips;
            }
            this.i18nService.getAvailableLanguages().forEach(function (lang) {
                if (!_this.wipStore[lang.code]) {
                    _this.wipStore[lang.code] = {};
                }
                if (!_this.modifiedWips[lang.code]) {
                    _this.modifiedWips[lang.code] = {};
                }
            });
        };
        /**
         * Persist the open wips to the browser's localStorage.
         */
        WipService.prototype.persistLocal = function () {
            this.localStorageService.set('wipStore', this.wipStore);
            this.localStorageService.set('modifiedWips', this.modifiedWips);
        };
        /**
         * Clear any wip data from the browser's localStorage.
         */
        WipService.prototype.clearLocal = function () {
            this.localStorageService.remove('wipStore');
            this.localStorageService.remove('modifiedWips');
        };
        return WipService;
    }());
    meshAdminUi.WipService = WipService;
    angular.module('meshAdminUi.common.wipService', ['LocalStorageModule'])
        .config(localStorageConfig)
        .service('wipService', WipService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    function selectiveCacheProvider() {
        var baseUrl = '', cacheableGroups = {};
        this.$get = ["$cacheFactory", function ($cacheFactory) {
            return new SelectiveCache($cacheFactory, baseUrl, cacheableGroups);
        }];
        /**
         * Set the base url which will be stripped from keys when performing
         * regexp matching. Can also be done in the service itself.
         * @param value
         */
        this.setBaseUrl = function (value) {
            baseUrl = value;
        };
        /**
         * Set the urls which will have caching enabled. Should be a hash of { groupName: urlRegexp }:
         * {
         *   'contents': /^\/contents.+/,
         *   'users': /^\/users\/[0-9]{5}$/
         * }
         *
         * So in this case, requests to the "contents" or "users" urls (which would match
         * the regexp patterns defined above) would be cached. No other requests would be.
         *
         * Then the cached entries can be selectively removed later on with the
         * selectiveCache.remove() method, passing in the group name.
         *
         * E.g. to remove all cache entries to do with contents, you would do:
         *
         * selectiveCache.remove('contents');
         *
         * @param {Object.<String, RegExp>} values
         */
        this.setCacheableGroups = function (values) {
            for (var key in values) {
                if (values.hasOwnProperty(key)) {
                    if (!(values[key] instanceof RegExp)) {
                        throw new Error('selectiveCacheProvider#setCacheableGroups():  "' +
                            values[key] + '" should be a regular expression.');
                    }
                }
            }
            cacheableGroups = values;
        };
    }
    /**
     * This is a wrapper around Angular's $cacheFactory cache, which provides the following enhancements:
     *
     * 1. It can be configured (see provider above) to only cache groups of urls that match the provided regexps.
     * 2. These urls groups can then be selectively removed from the cache by passing the name of the group
     *    to the .remove() method.
     *
     * @param $cacheFactory
     * @param {String} baseUrl
     * @param {Object.<String, RegExp>} cacheableGroups
     * @constructor
     */
    var SelectiveCache = (function () {
        function SelectiveCache($cacheFactory, baseUrl, cacheableGroups) {
            this.$cacheFactory = $cacheFactory;
            this.baseUrl = baseUrl;
            this.cacheableGroups = cacheableGroups;
            this.cachedKeys = [];
            this.ngCache = $cacheFactory('selectiveCache');
            this.cachedKeys = [];
            this.cachableRegexps = this.objectValues(cacheableGroups);
        }
        /**
         * Put a key-value pair into the cache as long as the key matches
         * one of the allowed urls set in `cacheableUrls`.
         */
        SelectiveCache.prototype.put = function (key, value) {
            var _this = this;
            var keyMinusBase = key.replace(this.baseUrl, '');
            this.cachableRegexps.forEach(function (regexp) {
                if (-1 < keyMinusBase.search(regexp)) {
                    _this.cachedKeys.push(key);
                    return _this.ngCache.put(key, value);
                }
            });
        };
        /**
         * Get the value associated with the key.
         */
        SelectiveCache.prototype.get = function (key) {
            return this.ngCache.get(key);
        };
        /**
         * Remove cache entries that match the regex defined by `group`
         * (see selectiveCacheProvider.setCacheableGroups()).
         * Throws an exception if the group was not already defined in the provider.
         * Returns the number of cache keys removed.
         */
        SelectiveCache.prototype.remove = function (group) {
            var _this = this;
            if (this.cacheableGroups[group] instanceof RegExp) {
                var keyRegexp = this.cacheableGroups[group], filterFn = this.matches(keyRegexp), matching = this.cachedKeys.filter(filterFn);
                matching.forEach(function (match) { return _this.removeFromCache(match); });
                return matching.length;
            }
        };
        /**
         * Returns a filter function which matches each passed element against the
         * regex.
         */
        SelectiveCache.prototype.matches = function (regexp) {
            var _this = this;
            return function (key) {
                var keyMinusBase = key.replace(_this.baseUrl, '');
                return -1 < keyMinusBase.search(regexp);
            };
        };
        /**
         * Remove the key from the cache.
         */
        SelectiveCache.prototype.removeFromCache = function (key) {
            this.ngCache.remove(key);
            this.removeFromCachedKeys(key);
        };
        /**
         * Remove the specified key from the cachedKeys array.
         */
        SelectiveCache.prototype.removeFromCachedKeys = function (key) {
            var index = this.cachedKeys.indexOf(key);
            if (index > -1) {
                this.cachedKeys.splice(index, 1);
            }
        };
        /**
         * Delegate directly to Angular's cache method.
         */
        SelectiveCache.prototype.removeAll = function () {
            return this.ngCache.removeAll();
        };
        /**
         * Delegate directly to Angular's cache method.
         */
        SelectiveCache.prototype.info = function () {
            return this.ngCache.info();
        };
        /**
         * Duplicate of the selectiveCacheProvider#setBaseUrl()
         */
        SelectiveCache.prototype.setBaseUrl = function (value) {
            this.baseUrl = value;
        };
        /**
         * Get the values from an object as an array;
         */
        SelectiveCache.prototype.objectValues = function (obj) {
            var values = [];
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    values.push(obj[key]);
                }
            }
            return values;
        };
        return SelectiveCache;
    }());
    meshAdminUi.SelectiveCache = SelectiveCache;
    angular.module('meshAdminUi.common')
        .provider('selectiveCache', selectiveCacheProvider);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Service to keep a track of the currently-applied search settings
     */
    var SearchService = (function () {
        function SearchService() {
            this.searchTerm = '';
            this.tagFilters = [];
            this.searchAll = false;
        }
        SearchService.prototype.setParams = function (params) {
            if (params.hasOwnProperty('searchTerm')) {
                this.searchTerm = params.searchTerm;
            }
            if (params.tagFilters) {
                this.tagFilters = params.tagFilters;
            }
            if (params.hasOwnProperty('searchAll')) {
                this.searchAll = params.searchAll;
            }
        };
        SearchService.prototype.getParams = function () {
            return {
                searchTerm: this.searchTerm,
                tagFilters: this.tagFilters,
                searchAll: this.searchAll
            };
        };
        return SearchService;
    }());
    meshAdminUi.SearchService = SearchService;
    angular.module('meshAdminUi.common')
        .service('searchService', SearchService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Returns a string of type `[fieldName, fieldType], ...` for use in error messages.
     */
    var fieldPropString = function (fields) {
        var props = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            props[_i - 1] = arguments[_i];
        }
        // get the first valid (truthy) value of the props passed in.
        var propVal = function (field) { return props.map(function (prop) { return field[prop]; }).filter(function (x) { return !!x; })[0]; };
        var templateMap = 0 < props.length ? function (f) { return ("[" + f.name + " : " + propVal(f) + "]"); } : function (f) { return ("[" + f.name + "]"); };
        return fields.map(templateMap).join(', ');
    };
    var validTypes = [
        'string',
        'number',
        'html',
        'boolean',
        'date',
        'list',
        'select',
        'node',
        'micronode',
        'binary'
    ];
    // ensure the schema name is set
    function schemaNameIsSet(obj, error) {
        if (!obj.name || obj.name.trim() === '') {
            error('ERR_SCHEMA_NAME_EMPTY');
            return false;
        }
        return true;
    }
    // ensure the schema name is valid
    function schemaNameIsValid(obj, error) {
        var validNameRe = /^[_a-zA-Z][_a-zA-Z0-9]*$/;
        if (obj.name && !validNameRe.test(obj.name)) {
            error('ERR_SCHEMA_NAME_INVALID');
            return false;
        }
        return true;
    }
    // ensure the displayField is set
    function displayFieldIsSet(obj, error) {
        if (obj.displayField === undefined || obj.displayField === '') {
            error('ERR_SCHEMA_PLEASE_SPECIFY_DISPLAY_FIELD');
            return false;
        }
        return true;
    }
    // ensure the segmentField is set
    function segmentFieldIsSet(obj, error) {
        if (obj.segmentField === undefined || obj.segmentField === '') {
            error('ERR_SCHEMA_PLEASE_SPECIFY_SEGMENT_FIELD');
            return false;
        }
        return true;
    }
    // ensure at least one field has been defined
    function fieldsIsNotEmpty(obj, error) {
        if (!obj.fields || obj.fields.length === 0) {
            error('ERR_SCHEMA_MUST_HAVE_AT_LEAST_ONE_FIELD');
            return false;
        }
        return true;
    }
    // ensure displayField matches an actual field name
    function displayFieldMatchesFieldName(obj, error) {
        var fieldNames = obj.fields.map(function (field) { return field.name; });
        if (fieldNames.indexOf(obj.displayField) === -1) {
            error('ERR_SCHEMA_DISPLAY_FIELD_DOES_NOT_MATCH', { value: obj.displayField });
            return false;
        }
        return true;
    }
    // ensure segmentField matches an actual field name
    function segmentFieldMatchesFieldName(obj, error) {
        var fieldNames = obj.fields.map(function (field) { return field.name; });
        if (obj.segmentField && obj.segmentField !== '' && fieldNames.indexOf(obj.segmentField) === -1) {
            error('ERR_SCHEMA_SEGMENT_FIELD_DOES_NOT_MATCH', { value: obj.segmentField });
            return false;
        }
        return true;
    }
    // ensure each field has a name and type
    function fieldsHaveNameAndType(obj, error) {
        var badFields = obj.fields.filter(function (field) { return !field.name || !field.type; });
        if (0 < badFields.length) {
            error('ERR_SCHEMA_REQUIRE_NAME_AND_TYPE');
            return false;
        }
        return true;
    }
    // ensure only valid field types are used
    function allFieldTypesValid(obj, error) {
        var badFieldTypes = obj.fields.filter(function (field) { return -1 === validTypes.indexOf(field.type); });
        if (0 < badFieldTypes.length) {
            var names = fieldPropString(badFieldTypes, 'type');
            error('ERR_SCHEMA_INVALID_TYPES', { names: names });
            return false;
        }
        return true;
    }
    // ensure there are no duplicate field names
    function fieldNamesAreUnique(obj, error) {
        var fieldNames = obj.fields.map(function (field) { return field.name; });
        var duplicateNames = fieldNames
            .sort()
            .reduce(function (duplciates, curr, index, arr) {
            var last = arr[index - 1];
            if (last !== undefined && last === curr) {
                duplciates.push(curr);
            }
            return duplciates;
        }, []);
        if (0 < duplicateNames.length) {
            var names = duplicateNames.map(function (n) { return ("[" + n + "]"); }).join(', ');
            error('ERR_SCHEMA_DUPLICATE_FIELD_NAMES', { names: names });
            return false;
        }
        return true;
    }
    // ensure a list type has listType set to a valid type
    function listTypesAreValid(obj, error) {
        var listFields = obj.fields.filter(function (field) { return field.type === 'list'; });
        if (0 < listFields.length) {
            var badListFields = listFields.filter(function (field) { return -1 === validTypes.indexOf(field.listType); });
            if (0 < badListFields.length) {
                var names = fieldPropString(badListFields, 'listType');
                error('ERR_SCHEMA_INVALID_LIST_TYPE', { names: names });
                return false;
            }
        }
        return true;
    }
    // ensure any micronode types or listTypes have an allow property defined.
    function micronodesHaveAllowProperty(obj, error) {
        var micronodeFields = obj.fields.filter(function (field) { return field.type === 'micronode' || field.listType === 'micronode'; });
        if (0 < micronodeFields.length) {
            var isArray_1 = function (x) { return x instanceof Array; };
            var badMicronodeFields = micronodeFields.filter(function (field) { return field.allow === undefined || !isArray_1(field.allow); });
            if (0 < badMicronodeFields.length) {
                var names = fieldPropString(badMicronodeFields);
                error('ERR_SCHEMA_ALLOW_PROPERTY_MISSING', { names: names });
                return false;
            }
        }
        return true;
    }
    var SchemaValidatorService = (function () {
        function SchemaValidatorService(i18n) {
            this.i18n = i18n;
        }
        SchemaValidatorService.$inject = ["i18n"];
        /**
         * Validate the json of a schema
         */
        SchemaValidatorService.prototype.validateSchemaJson = function (json, onError) {
            var validatorFns = [
                schemaNameIsSet,
                schemaNameIsValid,
                displayFieldIsSet,
                fieldsIsNotEmpty,
                displayFieldMatchesFieldName,
                segmentFieldMatchesFieldName,
                fieldsHaveNameAndType,
                allFieldTypesValid,
                fieldNamesAreUnique,
                listTypesAreValid,
                micronodesHaveAllowProperty
            ];
            return this.doValidation(json, validatorFns, onError);
        };
        /**
         * Validate the json of a microschema
         */
        SchemaValidatorService.prototype.validateMichroschemaJson = function (json, onError) {
            var validatorFns = [
                schemaNameIsSet,
                schemaNameIsValid,
                fieldsIsNotEmpty,
                fieldsHaveNameAndType,
                allFieldTypesValid,
                fieldNamesAreUnique,
                listTypesAreValid
            ];
            return this.doValidation(json, validatorFns, onError);
        };
        SchemaValidatorService.prototype.doValidation = function (json, validators, onError) {
            var errorFn = this.createErrorFn(onError);
            var schema = this.jsonToObject(json, errorFn);
            if (schema === undefined) {
                return false;
            }
            var isValid = true;
            if (!schema.fields) {
                schema.fields = [];
            }
            validators.forEach(function (validator) {
                if (!validator(schema, errorFn)) {
                    isValid = false;
                }
            });
            return isValid;
        };
        /**
         * Returns a function which invokes the passed onError function only if it is defined.
         */
        SchemaValidatorService.prototype.createErrorFn = function (onError) {
            var _this = this;
            return function (message, values) {
                if (onError !== undefined) {
                    var translated = _this.i18n(message, values);
                    onError(translated);
                }
            };
        };
        /**
         * Convert a json string to an object and check that it is valid;
         */
        SchemaValidatorService.prototype.jsonToObject = function (json, error) {
            var obj;
            try {
                obj = JSON.parse(json);
            }
            catch (e) {
                // JSON must be well-formed.
                error('JSON_IS_INVALID');
            }
            return obj;
        };
        return SchemaValidatorService;
    }());
    meshAdminUi.SchemaValidatorService = SchemaValidatorService;
    angular.module('meshAdminUi.common')
        .service('schemaValidatorService', SchemaValidatorService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var Type;
    (function (Type) {
        Type[Type["Schema"] = 0] = "Schema";
        Type[Type["Microschema"] = 1] = "Microschema";
    })(Type || (Type = {}));
    /**
     * Service used to import and export schemas and microschemas in .json format.
     */
    var SchemaImportExportService = (function () {
        function SchemaImportExportService(dataService, $q, mu, schemaValidatorService) {
            this.dataService = dataService;
            this.$q = $q;
            this.mu = mu;
            this.schemaValidatorService = schemaValidatorService;
        }
        SchemaImportExportService.$inject = ["dataService", "$q", "mu", "schemaValidatorService"];
        /**
         * Download each of the selected schemas as a .json bundle.
         */
        SchemaImportExportService.prototype.exportSelected = function (filename, schemas, selected) {
            var bundle = schemas
                .filter(function (schema) { return selected[schema.uuid]; })
                .map(function (schema) {
                var schemaClone = angular.copy(schema);
                delete schemaClone.uuid;
                delete schemaClone.permissions;
                return schemaClone;
            })
                .reduce(function (arr, schema) {
                arr.push(schema);
                return arr;
            }, []);
            // sort by schema name
            bundle.sort(function (a, b) {
                if (a.name < b.name) {
                    return -1;
                }
                if (a.name > b.name) {
                    return 1;
                }
                return 0;
            });
            // if there is only a single schema in the bundle, extract it from the array
            if (bundle.length === 1) {
                bundle = bundle[0];
            }
            this.downloadFile(filename + '.json', JSON.stringify(bundle, null, 4));
        };
        /**
         * Trigger a browser download of arbitrary text content, from http://stackoverflow.com/a/18197341/772859
         */
        SchemaImportExportService.prototype.downloadFile = function (filename, text) {
            var element = document.createElement('a');
            element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(text));
            element.setAttribute('download', filename);
            element.style.display = 'none';
            document.body.appendChild(element);
            element.click();
            document.body.removeChild(element);
        };
        SchemaImportExportService.prototype.importSchemas = function (files, currentSchemas, onErrors) {
            return this.processImport(Type.Schema, files, currentSchemas, onErrors);
        };
        SchemaImportExportService.prototype.importMicroschemas = function (files, currentMicroschemas, onErrors) {
            return this.processImport(Type.Microschema, files, currentMicroschemas, onErrors);
        };
        /**
         * Get the content of the selected files, validate it as a valid schema, and then POST to Mesh.
         */
        SchemaImportExportService.prototype.processImport = function (type, files, currentSchemas, onErrors) {
            var _this = this;
            return this.getFileTextContents(files)
                .then(function (results) { return _this.validateImports(type, results, onErrors); })
                .then(function (results) { return _this.renameDuplicates(results, currentSchemas); })
                .then(function (validResults) {
                var promises = validResults.map(function (result) {
                    return _this.persistToMesh(type, result.content);
                });
                return _this.$q.all(promises);
            });
        };
        /**
         * Given an array of File objects, reads their contents and returns and array of results.
         */
        SchemaImportExportService.prototype.getFileTextContents = function (files) {
            var _this = this;
            if (!files || files.length === 0) {
                return this.$q.when([]);
            }
            var promises = files.map(function (file) {
                var reader = new FileReader();
                var deferred = _this.$q.defer();
                reader.onload = function () {
                    var schemas = _this.fileContentsToSchemaArray(reader.result);
                    deferred.resolve(schemas.map(function (s) {
                        return {
                            content: JSON.stringify(s),
                            schemaName: s.name,
                            fileName: file.name
                        };
                    }));
                };
                reader.readAsText(file);
                return deferred.promise;
            });
            return this.$q.all(promises)
                .then(function (arrays) { return _this.mu.flatten(arrays); });
        };
        /**
         * Accepts a JSON string representation of either a single Schema or an array of Schemas, and returns an
         * array of Schemas.
         */
        SchemaImportExportService.prototype.fileContentsToSchemaArray = function (contents) {
            var jsObj = JSON.parse(contents);
            return (jsObj instanceof Array) ? jsObj : [jsObj];
        };
        /**
         * Validate the schemas of the imported JSON files.
         */
        SchemaImportExportService.prototype.validateImports = function (type, results, onErrors) {
            var _this = this;
            var errors = [];
            var validateFn = function (json, onError) {
                if (type === Type.Schema) {
                    return _this.schemaValidatorService.validateSchemaJson(json, onError);
                }
                else {
                    return _this.schemaValidatorService.validateMichroschemaJson(json, onError);
                }
            };
            var valid = results.filter(function (result) {
                var isValid = true;
                if (result.fileName.indexOf('.json') === -1) {
                    errors.push(result.fileName + ": File must be have .json extension");
                    return false;
                }
                validateFn(result.content, function (msg) {
                    errors.push(result.fileName + ": [" + result.schemaName + "] " + msg);
                    isValid = false;
                });
                return isValid;
            });
            if (0 < errors.length) {
                onErrors(errors);
            }
            return valid;
        };
        /**
         * Check for any duplicate schema names and rename any duplicates found with a trailing number.
         */
        SchemaImportExportService.prototype.renameDuplicates = function (results, currentSchemas) {
            var existingNames = currentSchemas.map(function (s) { return s.name; });
            var renameIfDuplicate = function (name) {
                var numberSuffixRe = /\((\d+)\)$/;
                while (-1 < existingNames.indexOf(name)) {
                    if (!numberSuffixRe.test(name)) {
                        name += '(1)';
                    }
                    name = name.replace(numberSuffixRe, function (match, n) { return ("(" + ++n + ")"); });
                }
                existingNames.push(name);
                return name;
            };
            return results.map(function (result) {
                var schema = JSON.parse(result.content);
                schema.name = renameIfDuplicate(schema.name);
                result.content = JSON.stringify(schema);
                result.schemaName = schema.name;
                return result;
            });
        };
        SchemaImportExportService.prototype.persistToMesh = function (type, json) {
            if (type === Type.Schema) {
                return this.dataService.createSchema(JSON.parse(json));
            }
            else {
                return this.dataService.createMicroschema(JSON.parse(json));
            }
        };
        return SchemaImportExportService;
    }());
    meshAdminUi.SchemaImportExportService = SchemaImportExportService;
    angular.module('meshAdminUi.common')
        .service('schemaImportExportService', SchemaImportExportService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Service that deals with notifications for the user. There are two types of
     * notification: toast and message.
     *
     * Toast is for transient status updates that are displayed for a short time to the
     * user upon completion of some action, e.g. saving changes.
     *
     * Messages are for more persistent notifications that require user-intervention to
     * view and dismiss. The presence of such messages would be indicated by some
     * visual alert in the UI, which the user can then click through to get to
     * a list of current messages.
     */
    var NotifyService = (function () {
        function NotifyService($mdToast, i18n) {
            this.$mdToast = $mdToast;
            this.i18n = i18n;
        }
        NotifyService.$inject = ["$mdToast", "i18n"];
        /**
         * Display a toast (popup) message containing the specified text.
         *
         * @param message - the string message or translation token to display
         * @param values {Object} - interpolation object for translation.
         */
        NotifyService.prototype.toast = function (message, values) {
            var _this = this;
            if (values === void 0) { values = {}; }
            var parsedMessage;
            if (typeof message === 'string') {
                parsedMessage = this.i18n(message, values);
            }
            else if (message instanceof Array) {
                parsedMessage = message.map(function (m) { return _this.i18n(m, values); }).join('<br>');
            }
            else {
                parsedMessage = JSON.stringify(message, null, '\t');
            }
            var popup = this.$mdToast.simple()
                .content(parsedMessage)
                .position('top right');
            this.$mdToast.show(popup);
        };
        NotifyService.prototype.message = function (text) {
            // TODO: implement
        };
        return NotifyService;
    }());
    meshAdminUi.NotifyService = NotifyService;
    angular.module('meshAdminUi.common')
        .service('notifyService', NotifyService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * A collection of static utility methods for use throughout the app.
     */
    var MeshUtils = (function () {
        function MeshUtils() {
        }
        /**
         * Given an object `item` with a `rolePerms` property of type ['create', 'read', 'update' 'delete'], this function
         * will return an object with each permission as a key set to `true`.
         * TODO: rendered redundant due to change in Mesh API. Remove once refactored out of all app code.
         */
        MeshUtils.prototype.rolePermissionsArrayToKeys = function (item) {
            var permissions = {}, perms = ['create', 'read', 'update', 'delete'];
            if (!(item.hasOwnProperty('rolePerms') && item.rolePerms instanceof Array)) {
                throw new Error('meshUtils#rolePermissionsArrayToKeys: argument must have a "rolePerms" property of type Array.');
            }
            perms.forEach(function (perm) {
                if (item.hasOwnProperty(perm)) {
                    throw new Error('meshUtils#rolePermissionsArrayToKeys: item already has a key "' + perm + '".');
                }
                else {
                    permissions[perm] = item.rolePerms.indexOf(perm) > -1;
                }
            });
            return permissions;
        };
        /**
         * Transform the object found in the "tags" property of a node into an array of tag objects.
         */
        MeshUtils.prototype.nodeTagsObjectToArray = function (tagsObject) {
            if (typeof tagsObject === 'undefined') {
                return [];
            }
            var makeSimpleTag = function (item, tagsObject, familyName) {
                return {
                    uuid: item.uuid,
                    fields: { name: item.name },
                    tagFamily: {
                        name: familyName,
                        uuid: tagsObject[familyName].uuid
                    }
                };
            };
            return Object.keys(tagsObject).reduce(function (prev, curr) {
                var familyTags = tagsObject[curr].items.map(function (item) { return makeSimpleTag(item, tagsObject, curr); });
                return prev.concat(familyTags);
            }, []);
        };
        MeshUtils.prototype.stringToColor = function (input) {
            var safeColors = ['#D1D5FF', '#FFFBD1', '#EAE3FF', '#E3FFF3', '#E3EEFF', '#FFE3EA'];
            var value = input.split('').reduce(function (prev, curr) {
                return prev + curr.charCodeAt(0);
            }, 0);
            return safeColors[value % safeColors.length];
        };
        MeshUtils.prototype.getBinaryFileUrl = function (projectName, nodeUuid, languageCode, fieldName, sha512sum, imageOptions) {
            var queryParams = "?cs=" + (sha512sum && sha512sum.substring(0, 8).toLowerCase());
            if (imageOptions !== undefined) {
                queryParams = Object.keys(imageOptions).reduce(function (queryString, key) {
                    return queryString + ("&" + key + "=" + imageOptions[key]);
                }, queryParams);
            }
            return meshUiConfig.apiUrl + projectName + ("/nodes/" + nodeUuid + "/binary/" + (fieldName + queryParams));
        };
        /**
         * Given a node, check for any binary fields if one if found, return the first
         * in an object with key (field name) and value (binary field properties).
         */
        MeshUtils.prototype.getFirstBinaryField = function (node) {
            var binaryFieldKey;
            var binaryFieldValue;
            if (node) {
                for (var key in node.fields) {
                    var field = node.fields[key];
                    if (field && field.fileSize) {
                        if (binaryFieldValue === undefined) {
                            binaryFieldKey = key;
                            binaryFieldValue = field;
                        }
                    }
                }
                return {
                    key: binaryFieldKey,
                    value: binaryFieldValue
                };
            }
        };
        /**
         * Reads the mime type of a binary field and returns true if it is an image.
         */
        MeshUtils.prototype.isImageField = function (field) {
            if (field.mimeType !== undefined) {
                return this.isImageMimeType(field.mimeType);
            }
            return false;
        };
        MeshUtils.prototype.isImageMimeType = function (mimeType) {
            return (/^image\//.test(mimeType));
        };
        /**
         * Deep clone an object, but leave any File objects intact since they
         * cannot be copied.
         *
         * Based on http://stackoverflow.com/a/728694/772859
         * Note that this will not handle object containing circular references.
         */
        MeshUtils.prototype.safeClone = function (obj) {
            var copy;
            // Handle the 3 simple types, and null or undefined
            if (null == obj || "object" != typeof obj)
                return obj;
            // Handle Date
            if (obj instanceof Date) {
                copy = new Date();
                copy.setTime(obj.getTime());
                return copy;
            }
            // Handle Array
            if (obj instanceof Array) {
                copy = [];
                for (var i = 0, len = obj.length; i < len; i++) {
                    copy[i] = this.safeClone(obj[i]);
                }
                return copy;
            }
            // File objects cannot be copied - so just return it.
            if (obj instanceof File) {
                return obj;
            }
            // Handle Object
            if (obj instanceof Object) {
                copy = {};
                for (var attr in obj) {
                    if (obj.hasOwnProperty(attr)) {
                        copy[attr] = this.safeClone(obj[attr]);
                    }
                }
                return copy;
            }
            throw new Error("MeshUtils#safeClone(): Unable to copy obj! Its type isn't supported.");
        };
        /**
         * Helper for filtering nodes by the display name against a string query.
         */
        MeshUtils.prototype.nodeFilterFn = function (node, query) {
            if (!query) {
                return true;
            }
            return -1 < node.fields[node.displayField].toLowerCase().indexOf(query.toLowerCase());
        };
        /**
         * Flatten an n-dimensional array.
         */
        MeshUtils.prototype.flatten = function (mdArray) {
            var _this = this;
            var flatArray = [];
            if (!(mdArray instanceof Array)) {
                throw new Error('meshUtils#flatten: argument must be of type Array, got ' + typeof mdArray);
            }
            mdArray.forEach(function (item) {
                if (item instanceof Array) {
                    flatArray = flatArray.concat(_this.flatten(item));
                }
                else {
                    flatArray.push(item);
                }
            });
            return flatArray;
        };
        /**
         * Generate a GUID
         */
        MeshUtils.prototype.generateGuid = function () {
            var s4 = function () {
                return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
            };
            return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
        };
        /**
         * From http://davidwalsh.name/javascript-debounce-function
         */
        MeshUtils.prototype.debounce = function (func, wait, immediate) {
            var timeout;
            return function () {
                var context = this, args = arguments;
                var later = function () {
                    timeout = null;
                    if (!immediate)
                        func.apply(context, args);
                };
                var callNow = immediate && !timeout;
                clearTimeout(timeout);
                timeout = setTimeout(later, wait);
                if (callNow)
                    func.apply(context, args);
            };
        };
        /**
         * Returns a function which can be used to generate predicate functions to be used with the
         * AngularJS "filter" filter: https://docs.angularjs.org/api/ng/filter/filter.
         *
         * By default, using the `item in collection | filter: query` expression will make Angular check
         * *all* properties of the `item` object. If we want to only filter by selected properties, we need to
         * write our own predicate function. The following function abstracts this step away and would be used like so:
         *
         * ```
         * // controller
         * public filterString: string; // data-bound to some UI input
         *
         * public myFilter = (value) => {
         *   return this.mu.matchProps(value, ['foo', 'bar'], this.filterString);
         * }
         * ```
         */
        MeshUtils.prototype.matchProps = function (obj, properties, filterText) {
            if (filterText === null || filterText === undefined) {
                return true;
            }
            for (var _i = 0, properties_1 = properties; _i < properties_1.length; _i++) {
                var property = properties_1[_i];
                if (obj[property]) {
                    if (typeof obj[property] === 'string') {
                        var match = -1 < obj[property].toLowerCase().indexOf(filterText.toLowerCase());
                        if (match) {
                            return true;
                        }
                    }
                }
            }
            return false;
        };
        /**
         * Given an array of language codes (['de', 'fr', 'en', 'az']), sort them
         * alphabetically, placing the currentLang first if given.
         */
        MeshUtils.prototype.sortLanguages = function (availableLangs, currentLang) {
            var sortCurrentLangFirst = function (a, b) {
                if (a === currentLang) {
                    return -1;
                }
                else if (b === currentLang) {
                    return 1;
                }
                return a < b ? -1 : 1;
            };
            var indentity = function (x) { return x; };
            var extractCode = function (lang) { return lang.code; };
            var mapFn = typeof availableLangs[0] === 'string' ? indentity : extractCode;
            return availableLangs.map(mapFn).sort(sortCurrentLangFirst);
        };
        /**
         * Given a string value, append the suffix to the end.
         * If the value has periods in it (as in a file name), then insert
         * the suffix before the file extension:
         *
         * foo => foo_de
         * foo.html => foo.de.html
         */
        MeshUtils.prototype.addSuffixToString = function (value, suffix, delimiter) {
            if (delimiter === void 0) { delimiter = '_'; }
            var parts = value.split('.');
            if (1 < parts.length) {
                parts.splice(-1, 0, suffix);
                return parts.join('.');
            }
            else {
                return value + delimiter + suffix;
            }
        };
        /**
         * Clones a node and changes the fields which should be unique in a given parentNode (i.e. displayField,
         * segmentField) by adding a suffix.
         *
         * Returns a copy of the node passed in.
         */
        MeshUtils.prototype.safeCloneNode = function (node, schema, suffix) {
            var nodeClone = angular.copy(node);
            var displayField = schema.displayField;
            var segmentField = schema.segmentField;
            if (typeof node.fields[displayField] === 'string') {
                nodeClone.fields[displayField] += " (" + suffix + ")";
            }
            if (segmentField && segmentField !== displayField && node.fields[segmentField]) {
                if (node.fields[segmentField].type === 'binary') {
                    nodeClone.fields[segmentField].fileName = this.addSuffixToString(node.fields[segmentField].fileName, suffix);
                }
                else if (node.fields[segmentField] !== undefined) {
                    nodeClone.fields[segmentField] = this.addSuffixToString(nodeClone.fields[segmentField], suffix);
                }
            }
            // Display a warning if there are any binary fields - these cannot be handled properly
            // until the dedicated translation endpoint is implemented in Mesh.
            var firstBinaryField = this.getFirstBinaryField(node);
            if (firstBinaryField.key !== undefined) {
                console.warn("Note: binary fields cannot yet be copied.");
            }
            return nodeClone;
        };
        return MeshUtils;
    }());
    meshAdminUi.MeshUtils = MeshUtils;
    angular.module('meshAdminUi.common')
        .service('mu', MeshUtils);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Central event-dispatching system. To be used instead of littering $scope.$on() calls around the code.
     */
    var Dispatcher = (function () {
        function Dispatcher($rootScope) {
            this.$rootScope = $rootScope;
            this.events = {
                loginSuccess: 'mesh:loginSuccess',
                logoutSuccess: 'mesh:logoutSuccess',
                wipsChanged: 'mesh:wipsChanged',
                contextChanged: 'mesh:contextChanged',
                editorServiceNodeOpened: 'mesh:editorServiceNodeOpened',
                editorServiceNodeClosed: 'mesh:editorServiceNodeClosed',
                explorerSearchParamsChanged: 'mesh:explorerSearchParamsChanged',
                explorerContentsChanged: 'mesh:explorerContentsChanged',
                explorerNodeTagsChanged: 'mesh:explorerNodeTagsChanged',
                languageChanged: 'mesh:languageChanged'
            };
            this.callbacks = [];
        }
        Dispatcher.$inject = ["$rootScope"];
        Dispatcher.prototype.subscribe = function (event, callback) {
            this.validateEvent(event);
            var unsubscribe = this.$rootScope.$on(event, callback);
            this.callbacks.push({
                fn: callback,
                unsubscribe: unsubscribe
            });
        };
        /**
         * Unsubscribe the callback functions from all events it is subscribed to.
         */
        Dispatcher.prototype.unsubscribeAll = function () {
            var _this = this;
            var callbacks = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                callbacks[_i - 0] = arguments[_i];
            }
            callbacks.forEach(function (callback) {
                _this.callbacks.forEach(function (obj) {
                    if (obj.fn === callback) {
                        obj.unsubscribe();
                    }
                });
                _this.callbacks = _this.callbacks.filter(function (obj) { return obj.fn !== callback; });
            });
        };
        Dispatcher.prototype.publish = function (event) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            this.validateEvent(event);
            (_a = this.$rootScope).$broadcast.apply(_a, [event].concat(args));
            var _a;
        };
        /**
         * Ensure the event string is valid as per the events defined in the static "events" object.
         */
        Dispatcher.prototype.validateEvent = function (event) {
            for (var label in this.events) {
                var validEvent = this.events[label];
                if (validEvent === event) {
                    return true;
                }
            }
            throw new Error("despatcher#validateEvent: The event \"" + event + "\" is not valid.");
        };
        return Dispatcher;
    }());
    meshAdminUi.Dispatcher = Dispatcher;
    angular.module('meshAdminUi.common')
        .service('dispatcher', Dispatcher);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    angular.module('meshAdminUi.common')
        .config(dataServiceConfig)
        .provider('dataService', dataServiceProvider);
    /**
     * The dataServiceProvider is used to configure and create the DataService which is used
     * for all requests to the API.
     */
    function dataServiceProvider() {
        var apiUrl;
        this.setApiUrl = setApiUrl;
        this.$get = ["$http", "$q", "Upload", "selectiveCache", "i18nService", "authService", "mu", function ($http, $q, Upload, selectiveCache, i18nService, authService, mu) {
            return new DataService($http, $q, Upload, selectiveCache, i18nService, authService, mu, apiUrl);
        }];
        /**
         * Allow config of the API url in the app's config phase.
         * @param value
         */
        function setApiUrl(value) {
            apiUrl = value;
        }
    }
    /**
     * The data service itself which is responsible for all requests to the API.
     */
    var DataService = (function () {
        function DataService($http, $q, Upload, selectiveCache, i18nService, authService, mu, apiUrl) {
            var _this = this;
            this.$http = $http;
            this.$q = $q;
            this.Upload = Upload;
            this.selectiveCache = selectiveCache;
            this.i18nService = i18nService;
            this.authService = authService;
            this.mu = mu;
            this.apiUrl = apiUrl;
            this.successHandler = function (response) {
                if (response && response.data) {
                    return response.data;
                }
                else {
                    return response;
                }
            };
            this.errorHandler = function (err) {
                if (err.status === 401) {
                    // user is not authorized - log out and return to login
                    console.warn("Session has expired, logging out");
                    _this.authService.setAsLoggedOut();
                    return {};
                }
                else {
                    throw err;
                }
            };
            selectiveCache.setBaseUrl(apiUrl);
            $http.defaults.cache = selectiveCache;
        }
        DataService.prototype.meshGet = function (url, params, config) {
            config = this.makeConfigObject(params, config);
            return this.$http.get(this.makeFullUrl(url), config)
                .then(this.successHandler, this.errorHandler);
        };
        DataService.prototype.meshPut = function (url, data, params, config) {
            config = this.makeConfigObject(params, config);
            return this.$http.put(this.makeFullUrl(url), data, config)
                .then(this.successHandler, this.errorHandler);
        };
        DataService.prototype.meshPost = function (url, data, params, config) {
            config = this.makeConfigObject(params, config);
            return this.$http.post(this.makeFullUrl(url), data, config)
                .then(this.successHandler, this.errorHandler);
        };
        DataService.prototype.meshDelete = function (url, params, config) {
            config = this.makeConfigObject(params, config);
            return this.$http.delete(this.makeFullUrl(url), config)
                .then(this.successHandler, this.errorHandler);
        };
        /**
         * Attach the given URL to the configured API URL and url-encode it.
         */
        DataService.prototype.makeFullUrl = function (url) {
            return this.apiUrl + encodeURI(url);
        };
        DataService.prototype.makeConfigObject = function (params, config) {
            params = params || {};
            if (params.lang !== false) {
                params.lang = params.lang || this.i18nService.getCurrentLang().code;
            }
            config = config || {};
            config.params = params;
            return config;
        };
        /**
         * Get all projects as a list.
         */
        DataService.prototype.getProjects = function (queryParams) {
            return this.meshGet('projects', queryParams);
        };
        /**
         * Get the details of a single project specified by uuid.
         */
        DataService.prototype.getProject = function (uuid, queryParams) {
            return this.meshGet('projects/' + uuid, queryParams);
        };
        DataService.prototype.getProjectByName = function (name, queryParams) {
            return this.getProjects(queryParams)
                .then(function (response) {
                return response.data.filter(function (project) { return project.name === name; })[0];
            });
        };
        /**
         * Persist the project back to the server.
         */
        DataService.prototype.persistProject = function (project) {
            this.clearCache('projects');
            this.clearCache('tags');
            if (project.uuid) {
                // this is an existing project
                return this.meshPost('projects/' + project.uuid, project);
            }
            else {
                // this is a plain object (newly-created)
                return this.meshPost('projects', project);
            }
        };
        /**
         * Delete the project from the server.
         */
        DataService.prototype.deleteProject = function (project) {
            this.clearCache('projects');
            return this.meshDelete('projects/' + project.uuid);
        };
        /**
         * Get the uuid of the specified project.
         */
        DataService.prototype.getProjectId = function (projectName) {
            return this.getProjectProperty(projectName, 'uuid');
        };
        /**
         * Get the root tag uuid of the specified project.
         */
        DataService.prototype.getProjectRootNodeId = function (projectName) {
            return this.getProjectProperty(projectName, 'rootNode')
                .then(function (rootNode) { return rootNode.uuid; });
        };
        /**
         * Get the value of the specified property for a project matching
         * projectName. If no matching project is found, the promise is
         * rejected.
         */
        DataService.prototype.getProjectProperty = function (projectName, propertyName) {
            var deferred = this.$q.defer();
            this.getProjects().then(function (projects) {
                var filtered = projects.data.filter(function (project) {
                    return project.name === projectName;
                });
                if (filtered[0] && filtered[0][propertyName]) {
                    deferred.resolve(filtered[0][propertyName]);
                }
                else {
                    deferred.reject('Property "' + propertyName + '" of project "' + projectName + '" not found.');
                }
            });
            return deferred.promise;
        };
        /**
         *
         */
        DataService.prototype.getUsers = function (queryParams) {
            return this.meshGet('users', queryParams);
        };
        /**
         *
         */
        DataService.prototype.getUser = function (uuid, queryParams) {
            return this.meshGet('users/' + uuid, queryParams);
        };
        /**
         * Persist the user back to the server.
         */
        DataService.prototype.persistUser = function (user) {
            var isNew = !user.hasOwnProperty('created');
            this.clearCache('users');
            return isNew ? this.createUser(user) : this.updateUser(user);
        };
        DataService.prototype.createUser = function (user) {
            return this.meshPost('users', user);
        };
        DataService.prototype.updateUser = function (user) {
            return this.meshPost('users/' + user.uuid, user);
        };
        DataService.prototype.deleteUser = function (user) {
            this.clearCache('users');
            return this.meshDelete('users/' + user.uuid);
        };
        DataService.prototype.addUserToGroup = function (userId, groupId) {
            this.clearCache('users');
            return this.meshPost('groups/' + groupId + '/users/' + userId, {});
        };
        DataService.prototype.removeUserFromGroup = function (userId, groupId) {
            this.clearCache('users');
            return this.meshDelete('groups/' + groupId + '/users/' + userId);
        };
        /**
         *
         */
        DataService.prototype.getGroups = function (queryParams) {
            return this.meshGet('groups', queryParams);
        };
        DataService.prototype.getGroup = function (uuid, queryParams) {
            return this.meshGet('groups/' + uuid, queryParams);
        };
        DataService.prototype.deleteGroup = function (uuid, queryParams) {
            this.clearCache('groups');
            return this.meshDelete('groups/' + uuid, queryParams);
        };
        /**
         * Persist the group back to the server.
         */
        DataService.prototype.persistGroup = function (group) {
            var isNew = !group.hasOwnProperty('created');
            this.clearCache('users');
            this.clearCache('groups');
            return isNew ? this.createGroup(group) : this.updateGroup(group);
        };
        DataService.prototype.createGroup = function (group) {
            return this.meshPost('groups', group);
        };
        DataService.prototype.updateGroup = function (group) {
            return this.meshPost('groups/' + group.uuid, group);
        };
        DataService.prototype.addGroupToRole = function (groupUuid, roleUuid) {
            this.clearCache('groups');
            this.clearCache('roles');
            return this.meshPost('groups/' + groupUuid + '/roles/' + roleUuid, {});
        };
        DataService.prototype.removeGroupFromRole = function (groupUuid, roleUuid) {
            this.clearCache('groups');
            this.clearCache('roles');
            return this.meshDelete('groups/' + groupUuid + '/roles/' + roleUuid);
        };
        /**
         * Get the child nodes for the parent node in the given project.
         */
        DataService.prototype.getChildNodes = function (projectName, parentNodeId, queryParams) {
            var _this = this;
            if (queryParams === void 0) { queryParams = {}; }
            var url = projectName + '/nodes/' + parentNodeId + '/children';
            var currLangCode = this.i18nService.getCurrentLang().code;
            var availableLangs = this.i18nService.getAvailableLanguages();
            queryParams.lang = this.mu.sortLanguages(availableLangs, currLangCode).join(',');
            return this.meshGet(url, queryParams)
                .then(function (result) {
                result.data.sort(_this.sortNodesBySchemaName);
                return result;
            });
        };
        /**
         */
        DataService.prototype.getNodeBundles = function (projectName, node, bundleParams, searchParams, queryParams) {
            var _this = this;
            var sortByIsContainer = function (a, b) {
                if (a.schema.container === true && b.schema.container === false) {
                    return -1;
                }
                else if (a.schema.container === false && b.schema.container === true) {
                    return 1;
                }
                return 0;
            };
            var searchOperation;
            if (searchParams.searchAll) {
                var searchAll = function () {
                    var query = _this.createNodeSearchQuery(projectName, node, searchParams);
                    var bundleQueryParams = angular.extend(queryParams, { page: bundleParams[0].page });
                    return _this.searchNodes(query, projectName, bundleQueryParams);
                };
                searchOperation = [searchAll()];
            }
            else {
                searchOperation = bundleParams
                    .sort(sortByIsContainer)
                    .map(function (bundleParam) {
                    var query = _this.createNodeSearchQuery(projectName, node, searchParams, bundleParam);
                    var bundleQueryParams = angular.extend(queryParams, { page: bundleParam.page });
                    return _this.searchNodes(query, projectName, bundleQueryParams);
                });
            }
            return this.$q.all(searchOperation)
                .then(function (results) {
                return results
                    .map(function (listResponse, index) {
                    return angular.extend(listResponse, { schema: bundleParams[index].schema });
                })
                    .filter(function (bundle) { return 0 < bundle.data.length; });
            });
        };
        DataService.prototype.createNodeSearchQuery = function (projectName, node, searchParams, bundleParam) {
            var query = {
                filter: {
                    bool: {
                        must: []
                    }
                },
                sort: [
                    { "created": "asc" }
                ]
            };
            if (bundleParam && bundleParam.schema) {
                query.filter.bool.must.push({ "term": { "schema.uuid": bundleParam.schema.uuid } });
            }
            if (!searchParams.searchAll) {
                query.filter.bool.must.push({ "term": { "parentNode.uuid": node.uuid } });
            }
            if (searchParams.searchTerm && searchParams.searchTerm !== '') {
                query.query = {
                    match_phrase: {
                        'displayField.value': searchParams.searchTerm
                    }
                };
            }
            if (searchParams.tagFilters && 0 < searchParams.tagFilters.length) {
                query.filter.bool.must.push({
                    "nested": {
                        "path": "tags",
                        "query": {
                            "bool": {
                                "must": searchParams.tagFilters.map(function (tag) {
                                    return {
                                        "term": {
                                            "tags.name.raw": tag.name,
                                        }
                                    };
                                })
                            }
                        }
                    }
                });
            }
            return query;
        };
        /**
         * Search for nodes matching the given query.
         * If the projectName is specified, the search will be limited
         * @param query
         * @param queryParams
         */
        DataService.prototype.searchNodes = function (query, projectName, queryParams) {
            var _this = this;
            queryParams.lang = '*';
            var url = projectName + "/search/nodes";
            return this.meshPost(url, query, queryParams)
                .then(function (response) {
                // because we are search for all languages (*), Mesh will return multiple nodes of the same
                // UUID if it exists in more than one language. In this case we need to filter out all but one
                // language version so as to not list multiple versions on the same node.
                var currentLang = _this.i18nService.getCurrentLang().code;
                var defaultLang = _this.i18nService.getDefaultLang().code;
                response.data = _this.filterNodesByLanguage(response.data, currentLang, defaultLang);
                return response;
            });
        };
        /**
         * Given an array of Nodes, this method will look for any duplicated uuids and ensure there that only
         * one of them is returned. The rule on which one to pick is as follows:
         * 1. If the node exists in the current language, keep that one.
         * 2. Else if the node exists in the default language, keep that one.
         * 3. Else return the node whose language code comes first alphabetically.
         *
         */
        DataService.prototype.filterNodesByLanguage = function (nodes, currentLang, defaultLang) {
            // return an array of nodes with the given UUID.
            var nodesWithUuid = function (uuid) { return nodes.filter(function (node) { return node.uuid === uuid; }); };
            // an array of unique uuids
            var uuids = nodes
                .map(function (node) { return node.uuid; })
                .filter(function (uuid, i, uuids) { return uuids.indexOf(uuid) === i; });
            var filtered = [];
            uuids.forEach(function (uuid) {
                var instances = nodesWithUuid(uuid);
                var node;
                if (instances.length === 1) {
                    node = instances;
                }
                else {
                    var availableLangs = instances.map(function (node) { return node.language; });
                    if (-1 < availableLangs.indexOf(currentLang)) {
                        // use the current lang
                        node = instances[availableLangs.indexOf(currentLang)];
                    }
                    else if (-1 < availableLangs.indexOf(defaultLang)) {
                        // use the default lang
                        node = instances[availableLangs.indexOf(defaultLang)];
                    }
                    else {
                        // use the alphabetical first lang
                        var alphabeticalFirstIndex = availableLangs.indexOf(availableLangs.sort()[0]);
                        node = instances[alphabeticalFirstIndex];
                    }
                }
                filtered = filtered.concat(node);
            });
            return filtered;
        };
        /**
         * Get a single node.
         */
        DataService.prototype.getNode = function (projectName, uuid, queryParams) {
            queryParams = queryParams || {};
            var currLangCode = this.i18nService.getCurrentLang().code;
            var availableLangs = this.i18nService.getAvailableLanguages();
            queryParams.lang = this.mu.sortLanguages(availableLangs, currLangCode).join(',');
            return this.meshGet(projectName + '/nodes/' + uuid, queryParams);
        };
        /**
         * Create or update the node object on the server.
         */
        DataService.prototype.persistNode = function (projectName, node, queryParams) {
            var isNew = !node.hasOwnProperty('created');
            var sanitizedNode = this.sanitizeFields(node);
            this.clearCache('nodes');
            return isNew ? this.createNode(projectName, sanitizedNode, queryParams) : this.updateNode(projectName, sanitizedNode, queryParams);
        };
        DataService.prototype.createNode = function (projectName, node, queryParams) {
            var _this = this;
            return this.meshPost(projectName + '/nodes', node, queryParams)
                .then(function (newNode) {
                return _this.uploadBinaryFields(projectName, node.fields, newNode.uuid, newNode.version.number, 'POST')
                    .then(function (result) {
                    if (result === false) {
                        // no uploads were required
                        return newNode;
                    }
                    else {
                        return result;
                    }
                })
                    .then(function (newNode) {
                    return _this.transformBinaryFields(projectName, node.fields, newNode.uuid, newNode.version.number)
                        .then(function () { return newNode; });
                });
            });
        };
        DataService.prototype.updateNode = function (projectName, node, queryParams) {
            var _this = this;
            return this.meshPost(projectName + '/nodes/' + node.uuid, node, queryParams)
                .then(function (newNode) {
                return _this.uploadBinaryFields(projectName, node.fields, newNode.uuid, newNode.version.number, 'PUT')
                    .then(function (result) {
                    if (result === false) {
                        // no uploads were required
                        return newNode;
                    }
                    else {
                        return result;
                    }
                })
                    .then(function (newNode) {
                    return _this.transformBinaryFields(projectName, node.fields, newNode.uuid, newNode.version.number)
                        .then(function () { return newNode; });
                });
            }, function (err) {
                if (err.status === 409) {
                    // a conflict occurred, so we will brute-force update the node by simply setting the version
                    // to the latest version and re-posting.
                    console.warn("Version conflict detected, forcing update to this version", err.data);
                    node.version.number = err.data.properties.newVersion;
                    return _this.updateNode(projectName, node, queryParams);
                }
            });
        };
        DataService.prototype.publishNode = function (projectName, node) {
            this.clearCache('nodes');
            return this.meshPost(projectName + '/nodes/' + node.uuid + '/published', {});
        };
        /**
         * Perform any needed clean-up of fields before sending the object to Mesh.
         */
        DataService.prototype.sanitizeFields = function (node) {
            // Returns true if obj is an object with no properties.
            var isEmptyObject = function (obj) { return typeof obj === 'object' && Object.keys(obj).length === 0; };
            for (var fieldName in node.fields) {
                var fieldVal = node.fields[fieldName];
                if (fieldVal instanceof Array) {
                    // Mesh errors if a list of nodes contains an empty object, so filter
                    // any of these out.
                    node.fields[fieldName] = fieldVal.filter(function (item) { return !isEmptyObject(item); });
                }
            }
            return node;
        };
        /**
         * Given a node, inspect each field for any that have a File value. This means a file has been selected
         * in the editor for upload.
         */
        DataService.prototype.uploadBinaryFields = function (projectName, fields, nodeUuid, versionNumber, method) {
            var _this = this;
            var binaryFields = Object.keys(fields)
                .filter(function (key) { return fields[key] instanceof File; })
                .map(function (key) {
                return {
                    name: key,
                    file: fields[key]
                };
            });
            if (0 < binaryFields.length) {
                var uploads = binaryFields
                    .map(function (field) { return _this.uploadBinaryFile(projectName, nodeUuid, field.name, field.file, versionNumber, method); });
                return this.$q.all(uploads)
                    .then(function () { return _this.getNode(projectName, nodeUuid); });
            }
            else {
                return this.$q.when(false);
            }
        };
        /**
         * Uploads a binary file to a specified field of a node.
         */
        DataService.prototype.uploadBinaryFile = function (projectName, nodeUuid, fieldName, binaryFile, versionNumber, method) {
            if (method === void 0) { method = 'POST'; }
            var lang = this.i18nService.getCurrentLang().code;
            return this.Upload.upload({
                url: this.apiUrl + projectName + ("/nodes/" + nodeUuid + "/binary/" + fieldName),
                method: method,
                data: {
                    file: binaryFile,
                    filename: binaryFile.name,
                    language: lang,
                    version: versionNumber
                }
            });
        };
        /**
         * Check for any binary fields that have a `transform` property, and make a binary transform request
         * for any found.
         */
        DataService.prototype.transformBinaryFields = function (projectName, fields, nodeUuid, version) {
            var _this = this;
            var isBinary = function (obj) { return obj.hasOwnProperty('fileSize') || obj instanceof File; };
            var hasTransform = function (obj) { return obj.hasOwnProperty('transform') && 0 < Object.keys(obj.transform).length; };
            var binaryFieldsWithTransform = Object.keys(fields)
                .filter(function (key) { return fields[key] && isBinary(fields[key]) && hasTransform(fields[key]); })
                .map(function (key) {
                return {
                    name: key,
                    value: fields[key]
                };
            });
            if (0 < binaryFieldsWithTransform.length) {
                var promises = binaryFieldsWithTransform
                    .map(function (field) { return _this.transformBinary(projectName, nodeUuid, field.name, field.value.transform, version); });
                return this.$q.all(promises);
            }
            else {
                return this.$q.when(false);
            }
        };
        /**
         * Send a POST request to a binary field's `transform` endpoint.
         */
        DataService.prototype.transformBinary = function (projectName, nodeUuid, fieldName, transformParams, version) {
            var lang = this.i18nService.getCurrentLang().code;
            var params = angular.copy(transformParams);
            // set the width and height of the image in pixels according to
            // the crop and scale data.
            params.width = params.cropw * params.scale;
            params.height = params.croph * params.scale;
            params.language = lang;
            params.version = { number: version };
            return this.meshPost(projectName + ("/nodes/" + nodeUuid + "/binaryTransform/" + fieldName), params);
        };
        /**
         * Remove the content from the server.
         */
        DataService.prototype.deleteNode = function (projectName, node) {
            this.clearCache('nodes');
            var uuid = this.toUuid(node);
            return this.meshDelete(projectName + '/nodes/' + uuid);
        };
        DataService.prototype.deleteNodeLanguage = function (projectName, node, langCode) {
            this.clearCache('nodes');
            var uuid = this.toUuid(node);
            return this.meshDelete(projectName + '/nodes/' + uuid + '/languages/' + langCode);
        };
        /**
         * Delete multiple nodes in sequence. Returns the node uuids that were deleted.
         */
        DataService.prototype.deleteNodes = function (projectName, nodes, deleteAllLanguages) {
            if (deleteAllLanguages === void 0) { deleteAllLanguages = false; }
            var uuids = this.toUuids(nodes), uuidsClone = uuids.slice(0);
            return this.$q.when(this.deleteNext(projectName, uuids, deleteAllLanguages))
                .then(function () { return uuidsClone; });
        };
        /**
         * Recursively deletes the selected content items as specified by the indices in the
         * this.selectedItems array. Done recursively in order to allow each DELETE request to
         * complete before sending the next. When done in parallel, deleting more than a few
         * items at once causes server error.
         */
        DataService.prototype.deleteNext = function (projectName, uuids, deleteAllLanguages) {
            var _this = this;
            if (uuids === void 0) { uuids = []; }
            if (uuids.length === 0) {
                return;
            }
            else {
                var uuid_1 = uuids.pop();
                var deleteOperation = void 0;
                if (deleteAllLanguages === true) {
                    deleteOperation = function () { return _this.deleteNode(projectName, uuid_1); };
                }
                else {
                    deleteOperation = function () { return _this.deleteNodeLanguage(projectName, uuid_1, _this.i18nService.getCurrentLang().code); };
                }
                return deleteOperation().then(function () { return _this.deleteNext(projectName, uuids, deleteAllLanguages); });
            }
        };
        /**
         * Move a node to be a child of another node given by uuid.
         */
        DataService.prototype.moveNode = function (projectName, node, destinationUuid) {
            this.clearCache('nodes');
            var uuid = this.toUuid(node);
            return this.meshPost(projectName + '/nodes/' + uuid + '/moveTo/' + destinationUuid, {});
        };
        /**
         * Recursively moves a list of nodes to the destination node given by uuid.
         */
        DataService.prototype.moveNodes = function (projectName, nodes, destinationUuid) {
            var uuids = this.toUuids(nodes), uuidsClone = uuids.slice(0);
            return this.$q.when(this.moveNext(projectName, destinationUuid, uuids))
                .then(function () { return uuidsClone; });
        };
        /**
         * Recursively move nodes to a destination node.
         */
        DataService.prototype.moveNext = function (projectName, destinationUuid, uuids) {
            var _this = this;
            if (uuids === void 0) { uuids = []; }
            if (uuids.length === 0) {
                return;
            }
            else {
                var uuid = uuids.pop();
                return this.moveNode(projectName, uuid, destinationUuid)
                    .then(function () { return _this.moveNext(projectName, destinationUuid, uuids); });
            }
        };
        /**
         * Takes an array of either uuid strings or nodes, and returns an array of uuid strings.
         */
        DataService.prototype.toUuids = function (nodes) {
            var _this = this;
            return nodes.map(function (node) { return _this.toUuid(node); });
        };
        /**
         * Takes either a node or a uuid string, and returns a uuid string.
         */
        DataService.prototype.toUuid = function (node) {
            return (typeof node === 'string') ? node : node.uuid;
        };
        /**
         * Given a string, this method sends it to the Mesh Link Resolver
         * utility endpoint and returns the text with rendered links.
         */
        DataService.prototype.renderLinksInText = function (contents, resolveMode) {
            return this.meshPost("utilities/linkResolver?resolveLinks=" + resolveMode.toUpperCase(), contents);
        };
        /**
         */
        DataService.prototype.getTagFamilies = function (projectName, queryParams) {
            var url = projectName + '/tagFamilies';
            return this.meshGet(url, queryParams);
        };
        DataService.prototype.persistTagFamily = function (projectName, tagFamily) {
            var isNew = !tagFamily.hasOwnProperty('created');
            return isNew ? this.createTagFamily(projectName, tagFamily) : this.updateTagFamily(projectName, tagFamily);
        };
        DataService.prototype.createTagFamily = function (projectName, tagFamily) {
            return this.meshPost(projectName + '/tagFamilies', tagFamily);
        };
        DataService.prototype.updateTagFamily = function (projectName, tagFamily) {
            return this.meshPost(projectName + '/tagFamilies/' + tagFamily.uuid, tagFamily);
        };
        DataService.prototype.deleteTagFamily = function (projectName, tagFamily) {
            return this.meshDelete(projectName + '/tagFamilies/' + tagFamily.uuid);
        };
        /**
         * Get all tags in a given project.
         */
        DataService.prototype.getProjectTags = function (projectName, queryParams) {
            var _this = this;
            return this.getTagFamilies(projectName, queryParams)
                .then(function (response) {
                var promises = response.data.map(function (tagFamily) {
                    return _this.getTagFamilyTags(projectName, tagFamily.uuid);
                });
                return _this.$q.all(promises)
                    .then(function (responses) {
                    var flatten = function (arr) { return arr.reduce(function (arr, t) { return arr.concat(t); }, []); };
                    return flatten(responses.map(function (res) { return res.data; }));
                });
            });
        };
        DataService.prototype.getTagFamilyTags = function (projectName, tagFamilyUuid, queryParams) {
            var url;
            if (typeof tagFamilyUuid === 'undefined') {
                url = projectName + '/tags';
            }
            else {
                url = projectName + '/tagFamilies/' + tagFamilyUuid + '/tags';
            }
            return this.meshGet(url, queryParams);
            // TODO: use the search code below one the elasticsearch index allows us to query tag.project.name
            /*let query: ISearchQuery = {
             filter: {
             bool: {
             must: [
             { "term": { "project.name": projectName.toLowerCase() } }
             ]
             }
             },
             sort: [
             { "tagFamily.name": "asc" },
             { "fields.name": "asc" }
             ]
             };

             if (tagFamilyUuid) {
             query.filter.bool.must.push({ "term": { "tagFamily.uuid": tagFamilyUuid } });
             }
             return this.searchTags(query, queryParams);*/
        };
        DataService.prototype.searchTags = function (query, queryParams) {
            return this.meshPost('search/tags', query, queryParams);
        };
        DataService.prototype.persistTag = function (projectName, tag) {
            var isNew = !tag.hasOwnProperty('created');
            this.clearCache('tags');
            return isNew ? this.createTag(projectName, tag) : this.updateTag(projectName, tag);
        };
        DataService.prototype.createTag = function (projectName, tag) {
            return this.meshPost(projectName + "/tagFamilies/" + tag.tagFamily.uuid + "/tags", tag);
        };
        DataService.prototype.updateTag = function (projectName, tag) {
            return this.meshPost(projectName + "/tagFamilies/" + tag.tagFamily.uuid + "/tags/" + tag.uuid, tag);
        };
        DataService.prototype.deleteTag = function (projectName, tag) {
            this.clearCache('tags');
            return this.meshDelete(projectName + "/tagFamilies/" + tag.tagFamily.uuid + "/tags/" + tag.uuid);
        };
        DataService.prototype.updateNodeTags = function (projectName, node, tags) {
            this.clearCache('nodes');
            return this.meshPost(projectName + '/nodes/' + node.uuid + '/tags', { tags: tags });
        };
        /**
         * Get all the schemas available in Mesh.
         */
        DataService.prototype.getSchemas = function (queryParams) {
            return this.meshGet('schemas', queryParams);
        };
        DataService.prototype.getProjectSchemas = function (projectName) {
            return this.meshGet(projectName + '/schemas');
        };
        /**
         *
         */
        DataService.prototype.getSchema = function (uuid, queryParams) {
            return this.meshGet('schemas/' + uuid, queryParams);
        };
        DataService.prototype.addSchemaToProject = function (schemaUuid, projectName) {
            this.clearCache('schemas');
            return this.meshPost(projectName + '/schemas/' + schemaUuid, {});
        };
        DataService.prototype.removeSchemaFromProject = function (schemaUuid, projectName) {
            this.clearCache('schemas');
            return this.meshDelete(projectName + '/schemas/' + schemaUuid);
        };
        DataService.prototype.diffSchema = function (schema) {
            return this.meshPost("schemas/" + schema.uuid, schema);
        };
        DataService.prototype.applySchemaChangeset = function (schema, changeset) {
            return this.meshPost("schemas/" + schema.uuid + "/changes", changeset);
        };
        DataService.prototype.createSchema = function (schema) {
            this.clearCache('schemas');
            return this.meshPost('schemas', schema);
        };
        /**
         * Forces a schema update without doing a migration. Dangerous as data can be lost or corrupted.
         */
        DataService.prototype.forceSchemaUpdate = function (schema) {
            this.clearCache('schemas');
            return this.meshPost('schemas/' + schema.uuid, schema);
        };
        DataService.prototype.deleteSchema = function (schema) {
            this.clearCache('schemas');
            return this.meshDelete('schemas/' + schema.uuid);
        };
        /**
         * Microschema methods
         */
        DataService.prototype.getMicroschemas = function (queryParams) {
            return this.meshGet('microschemas', queryParams);
        };
        DataService.prototype.getProjectMicroschemas = function (projectName) {
            return this.meshGet(projectName + '/microschemas');
        };
        DataService.prototype.getMicroschema = function (uuid, queryParams) {
            return this.meshGet('microschemas/' + uuid, queryParams);
        };
        DataService.prototype.getMicroschemaByName = function (name, queryParams) {
            return this.getMicroschemas(queryParams)
                .then(function (response) { return response.data.filter(function (microschema) { return microschema.name === name; })[0]; });
        };
        DataService.prototype.addMicroschemaToProject = function (microschemaUuid, projectName) {
            this.clearCache('microschemas');
            return this.meshPost(projectName + '/microschemas/' + microschemaUuid, {});
        };
        DataService.prototype.removeMicroschemaFromProject = function (microschemaUuid, projectName) {
            this.clearCache('microschemas');
            return this.meshDelete(projectName + '/microschemas/' + microschemaUuid);
        };
        DataService.prototype.diffMicroschema = function (microschema) {
            return this.meshPost("microschemas/" + microschema.uuid + "/diff", microschema);
        };
        DataService.prototype.applyMicroschemaChangeset = function (microschema, changeset) {
            // TODO: API not yet implemented
            console.warn('Not yet implemented in API');
            return this.meshPost("microschemas/" + microschema.uuid + "/changes", changeset);
        };
        DataService.prototype.createMicroschema = function (microschema) {
            this.clearCache('microschemas');
            return this.meshPost('microschemas', microschema);
        };
        /**
         * Forces a microschema update without doing a migration. Dangerous as data can be lost or corrupted.
         */
        DataService.prototype.forceMicroschemaUpdate = function (microschema) {
            this.clearCache('microschemas');
            return this.meshPost('microschemas/' + microschema.uuid, microschema);
        };
        DataService.prototype.deleteMicroschema = function (microschema) {
            this.clearCache('microschemas');
            return this.meshDelete('microschemas/' + microschema.uuid);
        };
        /**
         *
         */
        DataService.prototype.getRoles = function (queryParams) {
            return this.meshGet('roles', queryParams);
        };
        DataService.prototype.getRole = function (uuid, queryParams) {
            return this.meshGet('roles/' + uuid, queryParams);
        };
        DataService.prototype.persistRole = function (role) {
            var isNew = !role.hasOwnProperty('created');
            this.clearCache('roles');
            return isNew ? this.createRole(role) : this.updateRole(role);
        };
        DataService.prototype.createRole = function (role) {
            return this.meshPost('roles', role);
        };
        DataService.prototype.updateRole = function (role) {
            return this.meshPost('roles/' + role.uuid, role);
        };
        DataService.prototype.deleteRole = function (role) {
            this.clearCache('roles');
            return this.meshDelete('roles/' + role.uuid);
        };
        /**
         * Permissions methods
         */
        DataService.prototype.getPermissions = function (roleUuid, path) {
            return this.meshGet('roles/' + roleUuid + '/permissions/' + path);
        };
        DataService.prototype.setNodePermissions = function (roleUuid, projectUuid, nodeUuid, permissions) {
            this.clearCache('nodes');
            return this.setPermissionsOnPath(roleUuid, 'projects/' + projectUuid + '/nodes/' + nodeUuid, permissions);
        };
        DataService.prototype.setProjectPermissions = function (roleUuid, permissions, projectUuid) {
            this.clearCache('projects');
            return this.setPermissionsOnRootOrNode('projects', roleUuid, permissions, projectUuid);
        };
        DataService.prototype.setSchemaPermissions = function (roleUuid, permissions, schemaUuid) {
            this.clearCache('schemas');
            return this.setPermissionsOnRootOrNode('schemas', roleUuid, permissions, schemaUuid);
        };
        DataService.prototype.setMicroschemaPermissions = function (roleUuid, permissions, microschemaUuid) {
            this.clearCache('microschemas');
            return this.setPermissionsOnRootOrNode('microschemas', roleUuid, permissions, microschemaUuid);
        };
        DataService.prototype.setUserPermissions = function (roleUuid, permissions, userUuid) {
            this.clearCache('users');
            return this.setPermissionsOnRootOrNode('users', roleUuid, permissions, userUuid);
        };
        DataService.prototype.setGroupPermissions = function (roleUuid, permissions, groupUuid) {
            this.clearCache('groups');
            return this.setPermissionsOnRootOrNode('groups', roleUuid, permissions, groupUuid);
        };
        DataService.prototype.setRolePermissions = function (roleUuid, permissions, targetRoleUuid) {
            this.clearCache('roles');
            return this.setPermissionsOnRootOrNode('roles', roleUuid, permissions, targetRoleUuid);
        };
        DataService.prototype.setTagPermissions = function (roleUuid, projectUuid, permissions, tag) {
            this.clearCache('tags');
            var path = 'projects/' + projectUuid + '/tagFamilies/' + tag.tagFamily.uuid + '/tags/' + tag.uuid;
            return this.setPermissionsOnPath(roleUuid, path, permissions);
        };
        DataService.prototype.setTagFamilyPermissions = function (roleUuid, projectUuid, permissions, tagFamilyUuid) {
            this.clearCache('tags');
            this.clearCache('roles');
            var pathBase = 'projects/' + projectUuid + '/tagFamilies';
            var path = tagFamilyUuid ? pathBase + "/" + tagFamilyUuid : pathBase;
            return this.setPermissionsOnPath(roleUuid, path, permissions);
        };
        /**
         * Sets the permissions on a particular node if specified by a uuid, else sets the permissions on the
         * root (aggregation) node for that type as given by pathBase.
         */
        DataService.prototype.setPermissionsOnRootOrNode = function (pathBase, roleUuid, permissions, nodeUuid) {
            var path = nodeUuid ? pathBase + "/" + nodeUuid : pathBase;
            if (!nodeUuid) {
                this.clearCache('roles');
            }
            return this.setPermissionsOnPath(roleUuid, path, permissions);
        };
        DataService.prototype.setPermissionsOnPath = function (roleUuid, path, permissions) {
            return this.meshPost('roles/' + roleUuid + '/permissions/' + path, permissions);
        };
        /**
         *
         */
        DataService.prototype.getBreadcrumb = function (project, currentNode) {
            //return this.meshGet(projectName + '/breadcrumb/' + uuid);
            var _this = this;
            /**
             * TODO: implement dedicated breadcrumbs endpoint when one becomes available.
             * This is a recursive promise-based solution which is inefficient in that it
             * needs to make a call for each level of the breadcrumb hierarchy.
             */
            var getBreadcrumbs = function (project, currentNode, breadcrumbs) {
                if (breadcrumbs === void 0) { breadcrumbs = []; }
                var complete = breadcrumbs;
                breadcrumbs.push(currentNode);
                if (currentNode.parentNode && currentNode.parentNode.uuid !== project.rootNode.uuid) {
                    complete = _this.getNode(project.name, currentNode.parentNode.uuid)
                        .then(function (node) { return getBreadcrumbs(project, node, breadcrumbs); });
                }
                return _this.$q.when(complete);
            };
            return getBreadcrumbs(project, currentNode);
        };
        /**
         * Sort function to be used with array.sort()
         */
        DataService.prototype.sortNodesBySchemaName = function (a, b) {
            if (a.schema.name < b.schema.name) {
                return -1;
            }
            if (a.schema.name > b.schema.name) {
                return 1;
            }
            return 0;
        };
        /**
         * Clear the $http cache of all keys matching groupName. Proper use of this method
         * depends on the use of the selectiveCache service, which allows selective removal
         * of only certain groups of cached keys at a time. The groupName parameter must
         * match a groupName registered with the `selectiveCacheProvider.setCacheableGroups()`
         * config method.
         */
        DataService.prototype.clearCache = function (groupName) {
            this.selectiveCache.remove(groupName);
        };
        return DataService;
    }());
    meshAdminUi.DataService = DataService;
    /**
     * Configure the dataService
     *
     * @param $httpProvider
     * @param selectiveCacheProvider
     */
    function dataServiceConfig($httpProvider, selectiveCacheProvider) {
        $httpProvider.interceptors.push(languageRequestInterceptor);
        $httpProvider.interceptors.push(function () {
            return {
                request: function (config) {
                    config.headers['Anonymous-Authentication'] = 'disable';
                    return config;
                }
            };
        });
        // define the urls we wish to cache
        var projectName = '^[a-zA-Z\\-]*', uuid = '[a-z0-9]{30,32}', _ = '\\/';
        var cacheable = {
            'projects': /^projects/,
            'nodes': new RegExp(projectName + _ + 'nodes\\/', 'gi'),
            'tags': new RegExp(projectName + _ + 'tagFamilies\\/?', 'gi'),
            'schemas': /\/?schemas\??/,
            'microschemas': /\/?microschemas\??/,
            'users': /^users\??/,
            'roles': /^roles\??/,
            'groups': /^groups\??/
        };
        selectiveCacheProvider.setCacheableGroups(cacheable);
    }
    dataServiceConfig.$inject = ["$httpProvider", "selectiveCacheProvider"];
    function languageRequestInterceptor(i18nService, mu) {
        return {
            request: function (config) {
                config = mu.safeClone(config);
                if (config.url.indexOf(meshUiConfig.apiUrl) > -1) {
                    config.params = config.params || {};
                    switch (config.params.lang) {
                        case false:
                            delete config.params.lang;
                            break;
                        case '*':
                            config.params.lang = i18nService.getAvailableLanguages().map(function (lang) { return lang.code; }).join(',');
                            break;
                        case undefined:
                            config.params.lang = i18nService.getCurrentLang().code;
                            break;
                        default:
                    }
                }
                return config;
            }
        };
    }
    languageRequestInterceptor.$inject = ["i18nService", "mu"];
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * The contextService allows app-wide communication of the current project & tag which is
     * being viewed by the user. The purpose is to facilitate context-aware searching and creation of objects.
     */
    var ContextService = (function () {
        function ContextService(dispatcher) {
            this.dispatcher = dispatcher;
            this.currentProject = {};
            this.currentNode = {};
            this.queueCallback = false;
        }
        ContextService.$inject = ["dispatcher"];
        /**
         * Set the current project and invoke any registered handlers
         */
        ContextService.prototype.setProject = function (project) {
            if (project === void 0) { project = {}; }
            this.currentProject = project;
            this.dispatchChangeEvent();
        };
        /**
         * Get the current project object.
         */
        ContextService.prototype.getProject = function () {
            return this.currentProject;
        };
        ContextService.prototype.setCurrentNode = function (node) {
            if (node === void 0) { node = {}; }
            this.currentNode = node;
            this.dispatchChangeEvent();
        };
        ContextService.prototype.getCurrentNode = function () {
            return this.currentNode;
        };
        /**
         * Invoke any registered context change handlers and pass each one the
         * current project and tag objects.
         *
         * A timeout is used to ensure that the callbacks are only invoked once per
         * event loop. Thus, even if this runContextChangeHandlers() public is called
         * several times during the event loop, all those calls will only result in the
         * registered callbacks being invoked exactly once.
         */
        ContextService.prototype.dispatchChangeEvent = function () {
            var _this = this;
            if (!this.queueCallback) {
                this.queueCallback = true;
                setTimeout(function () {
                    _this.dispatcher.publish(_this.dispatcher.events.contextChanged, _this.currentProject, _this.currentNode);
                    _this.queueCallback = false;
                }, 0);
            }
        };
        return ContextService;
    }());
    meshAdminUi.ContextService = ContextService;
    angular.module('meshAdminUi.common')
        .service('contextService', ContextService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Used to log a user in & out, and keep track of that fact.
     * Currently a mock service which will eventually use OAuth 2 once it has
     * been implemented on the server.
     */
    var AuthService = (function () {
        function AuthService($injector, $q, dispatcher, selectiveCache, notifyService, $cookies) {
            this.$injector = $injector;
            this.$q = $q;
            this.dispatcher = dispatcher;
            this.selectiveCache = selectiveCache;
            this.notifyService = notifyService;
            this.$cookies = $cookies;
            this.AUTH_HEADER_NAME = 'Authorization';
            this.CODE_RESPONSE_UNAUTHORIZED = 401;
            this._isLoggedIn = $cookies.get('isLoggedIn') === 'true';
            this.onLogInCallbacks = [];
            this.onLogOutCallbacks = [];
        }
        AuthService.$inject = ["$injector", "$q", "dispatcher", "selectiveCache", "notifyService", "$cookies"];
        // public API
        AuthService.prototype.isLoggedIn = function () {
            return this._isLoggedIn;
        };
        /**
         * Attempts to log the user in based on the supplied username and password.
         * Return true on success, else false.
         *
         * @param {string} userName
         * @param {string} password
         * @returns {boolean}
         */
        AuthService.prototype.logIn = function (userName, password) {
            var _this = this;
            var authHeaderValue, $http = this.$injector.get("$http"), config = { headers: {} }, deferred = this.$q.defer();
            authHeaderValue = "Basic " + btoa(userName + ":" + password);
            config.headers[this.AUTH_HEADER_NAME] = authHeaderValue;
            this.userRequestInFlight = $http.get(meshUiConfig.apiUrl + 'auth/login', config)
                .then(function (response) {
                return $http.get(meshUiConfig.apiUrl + 'auth/me');
            })
                .then(function (response) {
                if (response.status === 200) {
                    _this._currentUser = response.data;
                    _this.authString = authHeaderValue;
                    _this._isLoggedIn = true;
                    _this.$cookies.put('isLoggedIn', 'true');
                    _this.dispatcher.publish(_this.dispatcher.events.loginSuccess);
                    deferred.resolve(true);
                }
                else {
                    console.log('Error, status: ' + response.status);
                    deferred.reject();
                }
            }, function (response) {
                if (response.status === _this.CODE_RESPONSE_UNAUTHORIZED) {
                    //TODO: Display error message
                    console.log('not authorized: ' + response.status);
                    deferred.reject();
                }
            });
            return deferred.promise;
        };
        /**
         * If the browser is refreshed, then we will lose the "currentUser" data, so it needs
         * to be re-fetched from Mesh.
         */
        AuthService.prototype.reFetchCurrentUser = function () {
            var _this = this;
            if (this.userRequestInFlight) {
                return this.userRequestInFlight;
            }
            // need to re-fetch the current user data
            var config = {
                headers: (_a = {}, _a[this.AUTH_HEADER_NAME] = this.authString, _a)
            };
            var $http = this.$injector.get("$http");
            this.userRequestInFlight = $http.get(meshUiConfig.apiUrl + 'auth/me', config)
                .then(function (response) {
                _this._currentUser = response.data;
                _this.userRequestInFlight = undefined;
                return _this._currentUser;
            });
            return this.userRequestInFlight;
            var _a;
        };
        AuthService.prototype.getAuthValue = function () {
            return this.authString;
        };
        AuthService.prototype.getCurrentUser = function () {
            if (this._isLoggedIn && !this._currentUser) {
                this.reFetchCurrentUser();
            }
            return this._currentUser;
        };
        /**
         * Log the user out and persist that fact to the cookie.
         */
        AuthService.prototype.logOut = function () {
            var _this = this;
            var $http = this.$injector.get("$http"), deferred = this.$q.defer();
            $http.get(meshUiConfig.apiUrl + 'auth/logout').then(function () {
                _this.doLogoutHousekeeping();
                deferred.resolve(true);
            }).catch(function (error) {
                deferred.reject(error);
            });
            return deferred.promise;
        };
        /**
         * If the user is currently logged in, but interaction with the API indicated an unauthorized state (i.e. a 401 is
         * returned from a request), then we need to simply run the logout housekeeping which will get the UI state
         * back in sync with the API.
         */
        AuthService.prototype.setAsLoggedOut = function () {
            this.notifyService.toast('ERR_SESSION_EXPIRED');
            this.doLogoutHousekeeping();
        };
        AuthService.prototype.doLogoutHousekeeping = function () {
            // TODO: need to actually invalidate the basic auth on the browser, see
            // example solution here http://stackoverflow.com/a/492926/772859
            this._isLoggedIn = false;
            this._currentUser = null;
            this.$cookies.put('isLoggedIn', 'false');
            this.$cookies.put('authString', '');
            this.authString = '';
            this.dispatcher.publish(this.dispatcher.events.logoutSuccess);
            this.selectiveCache.removeAll();
        };
        return AuthService;
    }());
    meshAdminUi.AuthService = AuthService;
    angular.module('meshAdminUi.common')
        .service('authService', AuthService);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * Configures the languages available to the app and also the translation objects
     * to be used with those languages.
     *
     * For details on the $translateProvider see http://angular-translate.github.io/
     */
    function i18nConfig(i18nServiceProvider, $translateProvider, translationTable) {
        var availableLanguages = (meshUiConfig.availableLanguages || ['en']).map(function (s) { return s.toLowerCase(); });
        var defaultLanguage = (meshUiConfig.defaultLanguage || 'en').toLowerCase();
        i18nServiceProvider.setAvailableLanguages(availableLanguages);
        i18nServiceProvider.setDefaultLanguage(defaultLanguage);
        $translateProvider
            .useSanitizeValueStrategy('escapeParameters')
            .preferredLanguage(defaultLanguage)
            .fallbackLanguage(defaultLanguage);
        angular.forEach(availableLanguages, function (langCode) {
            $translateProvider.translations(langCode, translationTable.getLanguage(langCode));
        });
    }
    i18nConfig.$inject = ["i18nServiceProvider", "$translateProvider", "translationTable"];
    angular.module('meshAdminUi.common.i18n', [
        'pascalprecht.translate'
    ])
        .config(i18nConfig);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    var userAuth = {
        'LOG_IN': {
            en: 'Log In',
            de: 'Einloggen'
        },
        'LOG_OUT': {
            en: 'Log Out',
            de: 'Ausloggen'
        },
        'USER_NAME': {
            en: 'User Name',
            de: 'Benutzername'
        },
        'PASSWORD': {
            en: 'Password',
            de: 'Kennwort'
        },
        'PROFILE': {
            en: 'Profile',
            de: 'Profil'
        }
    };
    var projectsArea = {
        'WHOLE_PROJECT': {
            en: 'whole project',
            de: 'ganzes Projekt'
        },
        'PROJECTS': {
            en: 'Projects',
            de: 'Projekte'
        },
        'ALL_PROJECTS': {
            en: 'All Projects',
            de: 'Alle Projekte'
        },
        'CREATE_NODE': {
            en: 'Create node',
            de: 'Node erstellen'
        },
        'CLOSE_ALL_TABS': {
            en: 'Close all tabs',
            de: 'Alle Tabs schließen'
        },
        'MOVED_NODES': {
            en: 'Moved {{ count }} nodes',
            de: '{{ count }} Nodes verschoben'
        },
        'COPIED_NODE': {
            en: 'Copied node',
            de: 'Node kopiert'
        },
        'DELETED_NODES': {
            en: 'Deleted {{ count }} nodes',
            de: '{{ count }} Nodes gelöscht'
        },
        'AVAILABLE_IN': {
            en: 'Available in {{ lang }}',
            de: 'In {{ lang }} verfügbar'
        },
        'TRANSLATE_TO': {
            en: 'Translate to {{ lang }}',
            de: 'In {{ lang }} übersetzen'
        },
        'SWITCH_LANGUAGE_AND_OPEN_TITLE': {
            en: 'Switch Language to {{ lang }}?',
            de: 'Auf {{ lang }} ändern?'
        },
        'SWITCH_LANGUAGE_AND_OPEN_MESSAGE': {
            en: 'In order to open this node in {{ lang }}, the user interface language will be set to {{ lang }}',
            de: 'Um diesen Node zu öffnen, muss die Benutzeroberfläche zu {{ lang }} geändert werden'
        },
        'SELECT_DESTINATION': {
            en: 'Select destination',
            de: 'Zielort auswählen'
        }
    };
    var editorPane = {
        'EDIT_LOCALIZED_VERSION': {
            en: 'Edit localized version:',
            de: 'Lokalisierte Version bearbeiten:'
        },
        'CREATE_NEW_TAG': {
            en: 'Create new tag',
            de: 'Neuen Tag erstellen'
        },
        'NEW_TAG_CREATED': {
            en: 'Created new tag',
            de: 'Neuer Tag wurde erstellt'
        },
        'SELECT_A_TAG': {
            en: 'Select a tag',
            de: 'Tag auswählen'
        },
        'ADDED_TAG': {
            en: 'Added tag "{{ name }}"',
            de: 'Tag "{{ name }}" hinzugefügt'
        },
        'REMOVED_TAG': {
            en: 'Removed tag "{{ name }}"',
            de: 'Tag "{{ name }}" entfernt'
        },
        'TIMEPICKER_HOURS': {
            en: 'Hours',
            de: 'Stunden'
        },
        'TIMEPICKER_MINUTES': {
            en: 'Minutes',
            de: 'Minuten'
        },
        'TIMEPICKER_SECONDS': {
            en: 'Seconds',
            de: 'Sekunden'
        },
        'ADD_ITEM_TO': {
            en: 'Add item to',
            de: 'Hinzufügen zu'
        },
        'PUBLISHED': {
            en: 'Published',
            de: 'Veröffentlicht'
        },
        'DRAFT': {
            en: 'Draft',
            de: 'Entwurf'
        },
        'SAVE_DRAFT': {
            en: 'Save Draft',
            de: 'Entwurf Speichern'
        },
        'PUBLISH': {
            en: 'Publish',
            de: 'Veröffentlichen'
        },
        'SAVE_AND_PUBLISH': {
            en: 'Save & publish',
            de: 'Aktualisieren & Veröffentlichen'
        },
        'NODE_MUST_BE_SAVED_BEFORE_TRANSLATING': {
            en: 'Node must be saved before translating',
            de: 'Node muss vor der Übersetzung gespeichert werden'
        },
        'NO_NODE_SELECTED': {
            en: 'No node selected',
            de: 'Keinen Node ausgewählt'
        },
        'SELECT_NODE': {
            en: 'Select node',
            de: 'Node auswählen'
        },
        'REMOVE_NODE': {
            en: 'Remove node',
            de: 'Node löschen'
        }
    };
    var adminArea = {
        'FILTER_TAGS': {
            en: 'Filter tags',
            de: 'Tags filtern'
        },
        'CREATE_NEW_TAG_FAMILY': {
            en: 'Create new tag family',
            de: 'Neue Tag-Familie erstellen'
        },
        'EDIT_TAG_FAMILY': {
            en: 'Edit tag family',
            de: 'Tag-Familie bearbeiten'
        },
        'DELETE_TAG_FAMILY': {
            en: 'Delete tag family',
            de: 'Tag-Familie löschen'
        },
        'DELETE_TAG_FAMILY_AND_ALL_TAGS': {
            en: 'Delete this tag family and all tags?',
            de: 'Diese Tag-Familie und all ihre Tags löschen?'
        },
        'TAG_FAMILY_CREATED': {
            en: 'Tag family created',
            de: 'Tag-Familie erstellt'
        },
        'TAG_FAMILY_UPDATED': {
            en: 'Tag family updated',
            de: 'Tag-Familie aktualisiert'
        },
        'TAG_FAMILY_DELETED': {
            en: 'Tag family deleted',
            de: 'Tag-Familie gelöscht'
        },
        'ADD_TAG': {
            en: 'Add tag',
            de: 'Tag hinzufügen'
        },
        'EDIT_TAG': {
            en: 'Edit tag',
            de: 'Tag bearbeiten'
        },
        'UPDATED_TAG': {
            en: 'Updated tag',
            de: 'Tag aktualisieren'
        },
        'DELETED_TAG': {
            en: 'Deleted tag',
            de: 'Tag wurde gelöscht'
        },
        'CREATE_NEW_USER': {
            en: 'Create new user',
            de: 'Neuen Benutzer erstellen'
        },
        'NEW_USER_CREATED': {
            en: 'New user created',
            de: 'Neuer Benutzer erstellt'
        },
        'FILTER_USERS': {
            en: 'Filter users',
            de: 'Benutzer durchsuchen'
        },
        'USERNAME': {
            en: 'Username',
            de: 'Benutzername'
        },
        'PASSWORD': {
            en: 'Password',
            de: 'Kennwort'
        },
        'FIRST_NAME': {
            en: 'First name',
            de: 'Vorname'
        },
        'LAST_NAME': {
            en: 'Last name',
            de: 'Nachname'
        },
        'EMAIL_ADDRESS': {
            en: 'Email address',
            de: 'E-Mail-Adresse'
        },
        'USER_ALREADY_IN_GROUP': {
            en: 'User already in group "{{ name }}"',
            de: 'Der Benutzer befindet sich bereits in der Gruppe "{{ name }}"'
        },
        'USER_ADDED_TO_GROUP': {
            en: 'User {{ userName }} added to group "{{ groupName }}"',
            de: 'Der Benutzer {{ userName }} wurde zur Gruppe "{{ groupName }}" hinzugefügt'
        },
        'USER_REMOVED_FROM_GROUP': {
            en: 'User {{ userName }} removed from group "{{ groupName }}"',
            de: 'Der Benutzer {{ userName }} wurde von der Gruppe "{{ groupName }}" entfernt'
        },
        'FILTER_GROUPS': {
            en: 'Filter groups',
            de: 'Gruppen durchsuchen'
        },
        'CREATE_NEW_GROUP': {
            en: 'Create new group',
            de: 'Neue Gruppe erstellen'
        },
        'NEW_GROUP_CREATED': {
            en: 'New group created',
            de: 'Neue Gruppe erstellt'
        },
        'GROUP_NAME': {
            en: 'Group name',
            de: 'Gruppenname'
        },
        'GROUP_ALREADY_ASSIGNED_TO_ROLE': {
            en: 'Group already assigned to role {{ name }}"',
            de: 'Die Gruppe wurde bereits zur Rolle {{ name }}" hinzugefügt'
        },
        'GROUP_ADDED_TO_ROLE': {
            en: 'Group {{ groupName }} added to role "{{ roleName }}"',
            de: 'Die Gruppe {{ groupName }} wurde zur Rolle "{{ roleName }}" hinzugefügt'
        },
        'GROUP_REMOVED_FROM_ROLE': {
            en: 'Group {{ groupName }} removed from role "{{ roleName }}"',
            de: 'Die Gruppe {{ groupName }} wurde von der Rolle "{{ roleName }}" entfernt'
        },
        'FILTER_ROLES': {
            en: 'Filter roles',
            de: 'Rollen durchsuchen'
        },
        'CREATE_NEW_ROLE': {
            en: 'Create new role',
            de: 'Neue Rolle erstellen'
        },
        'NEW_ROLE_CREATED': {
            en: 'New role created',
            de: 'Neue Rolle erstellt'
        },
        'ROLE_NAME': {
            en: 'Role name',
            de: 'Rollenname'
        },
        'NODE_PERMISSIONS': {
            en: 'Node permissions',
            de: 'Node-Berechtigungen'
        },
        'PERMISSIONS_SET_ON_NODE': {
            en: 'Permissions set on node "{{ name }}"',
            de: 'Berechtigungen wurden "{{ name }}" zugewiesen'
        },
        'PERMISSIONS_SET_ON_TYPE': {
            en: 'Permissions set on {{ type }} "{{ name }}"',
            de: 'Berechtigungen wurden für {{ type }} "{{ name }}" zugewiesen'
        },
        'TAG_PERMISSIONS': {
            en: 'Tag permissions',
            de: 'Tag-Berechtigungen'
        },
        'FILTER_PROJECTS': {
            en: 'Filter projects',
            de: 'Projekte durchsuchen'
        },
        'CREATE_NEW_PROJECT': {
            en: 'Create new project',
            de: 'Neues Projekt erstellen'
        },
        'SCHEMA_ADDED_TO_PROJECT': {
            en: 'Schema "{{ name }}" was added to this project',
            de: 'Schema "{{ name }}" wurde zu diesem Projekt hinzugefügt'
        },
        'SCHEMA_REMOVED_FROM_PROJECT': {
            en: 'Schema "{{ name }}" was removed from this project',
            de: 'Das Schema "{{ name }}" wurde von diesem Projekt entfernt'
        },
        'MICROSCHEMA_ADDED_TO_PROJECT': {
            en: 'Microschema "{{ name }}" was added to this project',
            de: 'Microschema "{{ name }}" wurde zu diesem Projekt hinzugefügt'
        },
        'MICROSCHEMA_REMOVED_FROM_PROJECT': {
            en: 'Microschema "{{ name }}" was removed from this project',
            de: 'Das Microschema "{{ name }}" wurde von diesem Projekt entfernt'
        },
        'PROJECT_NAME': {
            en: 'Project name',
            de: 'Projektname'
        },
        'CREATE_NEW_SCHEMA': {
            en: 'Create new schema',
            de: 'Neues Schema erstellen'
        },
        'NEW_PROJECT_CREATED': {
            en: 'New project created',
            de: 'Neues Projekt erstellt'
        },
        'NEW_SCHEMA_CREATED': {
            en: 'New schema created',
            de: 'Neues Schema erstellt'
        },
        'FILTER_SCHEMAS': {
            en: 'Filter schemas',
            de: 'Schemas durchsuchen'
        },
        'FILTER_MICROSCHEMAS': {
            en: 'Filter microschemas',
            de: 'Microschemas durchsuchen'
        },
        'CREATE_NEW_MICROSCHEMA': {
            en: 'Create new microschema',
            de: 'Neues Microschema erstellen'
        },
        'NEW_MICROSCHEMA_CREATED': {
            en: 'New microschema created',
            de: 'Neues Microschema erstellt'
        },
        'IMPORTED_SCHEMAS': {
            en: 'Imported {{ count }} schemas',
            de: '{{ count }} Schemas importiert'
        },
        'ADD_NEW_FIELD': {
            en: 'Add new field',
            de: 'Neues Feld hinzufügen'
        },
        'EDIT_MIGRATION_SCRIPT': {
            en: 'Edit migration script',
            de: 'Migrationsskript bearbeiten',
        },
        'ADD_SCHEMA_FIELD_NAMED': {
            en: 'Add a field named "{{ fieldName }}"',
            de: 'Neues Feld "{{ fieldName }}" hinzufügen',
        },
        'REMOVE_SCHEMA_FIELD': {
            en: 'Remove field "{{ fieldName }}"',
            de: 'Das Feld "{{ fieldName }}" löschen',
        },
        'UPDATE_SCHEMA_FIELD': {
            en: 'Update field "{{ fieldName }}", setting keys [{{ keys }}] to values [{{ values }}]',
            de: 'Das Feld "{{ fieldName }}" aktualisieren, keys [{{ keys }}], values [{{ values }}]',
        },
        'CHANGE_SCHEMA_FIELD_TYPE': {
            en: 'Change field "{{ fieldName }}" to type "{{ type }}"',
            de: 'Change field "{{ fieldName }}" to type "{{ type }}"'
        },
        'UPDATE_SCHEMA_PROPERTY': {
            en: 'Update schema property "{{ key }}" to "{{ value }}"',
            de: 'Change field "{{ fieldName }}" to type "{{ type }}"'
        },
        'SCHEMA_OPERATION_NOT_RECOGNIZED': {
            en: 'Operation "{{ name }}" not recognized',
            de: 'Operation "{{ name }}" nicht erkannt',
        }
    };
    var common = {
        'SEARCH_IN': {
            en: 'Search in',
            de: 'Suche in'
        },
        'UPDATE': {
            en: 'Update',
            de: 'Aktualisieren'
        },
        'TITLE': {
            en: 'Title',
            de: 'Titel'
        },
        'SCHEMA': {
            en: 'Schema',
            de: 'Schema'
        },
        'AUTHOR': {
            en: 'Author',
            de: 'Autor'
        },
        'CREATED': {
            en: 'Created',
            de: 'Erstellt'
        },
        'ITEMS_PER_PAGE': {
            en: 'Items per page',
            de: 'Einträge pro Seite'
        },
        'SAVE_CHANGES': {
            en: 'Save Changes',
            de: 'Speichern'
        },
        'DISCARD_CHANGES': {
            en: 'Discard Changes',
            de: 'Verwerfen'
        },
        'EDIT': {
            en: 'Edit',
            de: 'Bearbeiten'
        },
        'COPY': {
            en: 'Copy',
            de: 'Kopieren'
        },
        'MOVE': {
            en: 'Move',
            de: 'Verschieben'
        },
        'OKAY': {
            en: 'Okay',
            de: 'Okay'
        },
        'CANCEL': {
            en: 'Cancel',
            de: 'Abbrechen'
        },
        'DELETE': {
            en: 'Delete',
            de: 'Löschen'
        },
        'DELETED': {
            en: 'Deleted',
            de: 'Gelöscht'
        },
        'CREATE': {
            en: 'Create',
            de: 'Erstellen'
        },
        'PREVIEW': {
            en: 'Preview',
            de: 'Vorschau'
        },
        'ADD': {
            en: 'Add',
            de: 'Hinzufügen'
        },
        'IMPORT': {
            en: 'Import',
            de: 'Importieren'
        },
        'EXPORT': {
            en: 'Export',
            de: 'Exportieren'
        },
        'FILE_NAME': {
            en: 'File name',
            de: 'Dateiname'
        },
    };
    var dialogs = {
        'CONTENT_MODIFIED': {
            en: 'Content Modified',
            de: 'Inhalt geändert'
        },
        'CONTENT_MODIFIED_HOW_TO_PROCEED': {
            en: 'This content has been modified. How would you like to proceed?',
            de: 'Dieser Inhalt wurde geändert. Wie möchten Sie vorgehen?'
        },
        'SAVED_CHANGES': {
            en: 'Saved changes',
            de: 'Gespeichert'
        },
        'NEW_CONTENT_CREATED': {
            en: 'New content created',
            de: 'Neuen Inhalt erstellt'
        },
        'EDIT_IMAGE': {
            en: 'Edit Image',
            de: 'Bild Bearbeiten'
        },
        'USE_SELECTED_NODE': {
            en: 'Use selected node',
            de: 'Benutze ausgewählten Node'
        },
        'CONFIRM_DELETE_TITLE': {
            en: 'Delete?',
            de: 'Löschen?'
        },
        'CONFIRM_DELETE_NODE_MESSAGE': {
            en: 'This node exists in the following languages. Check the ones you wish to delete:',
            de: 'Dieser Node existiert in folgenden Sprachen. Wählen Sie die zu löschenden Versionen:'
        },
        'CONFIRM_DELETE_NODE_MULTI_MESSAGE': {
            en: 'The selected nodes may exist in more than one language.',
            de: 'Die ausgewählten Nodes könnten in mehreren Sprachen existieren.'
        },
        'CONFIRM_DELETE_NODE_MULTI_CURRENT_MESSAGE': {
            en: 'Delete current language version only.',
            de: 'Nur derzeitige Sprachversion löschen.'
        },
        'CONFIRM_DELETE_NODE_MULTI_ALL_MESSAGE': {
            en: 'Delete all language versions.',
            de: 'Alle Sprachversionen löschen.'
        },
        'CONFIRM_DELETE_MESSAGE': {
            en: 'Are you sure you want to delete this object?',
            de: 'Sind Sie sicher, dass Sie dieses Objekt löschen wollen?'
        },
        'CONFIRM_DELETE_GROUP_TITLE': {
            en: 'Delete Group?',
            de: 'Gruppe löschen?'
        },
        'CONFIRM_DELETE_GROUP_MESSAGE': {
            en: 'Are you sure you want to delete this group?',
            de: 'Sind Sie sicher, dass Sie diese Gruppe löschen wollen?'
        },
        'CONFIRM_DELETE_MICROSCHEMA_TITLE': {
            en: 'Delete Microschema?',
            de: 'Microschema löschen?'
        },
        'CONFIRM_DELETE_MICROSCHEMA_MESSAGE': {
            en: 'Are you sure you want to delete this microschema?',
            de: 'Sind Sie sicher, dass Sie dieses Microschema löschen wollen?'
        },
        'CONFIRM_DELETE_PROJECT_TITLE': {
            en: 'Delete Project?',
            de: 'Projekt löschen?'
        },
        'CONFIRM_DELETE_PROJECT_MESSAGE': {
            en: 'Are you sure you want to delete this project?',
            de: 'Sind Sie sicher, dass Sie dieses Projekt löschen wollen?'
        },
        'CONFIRM_DELETE_ROLE_TITLE': {
            en: 'Delete Role?',
            de: 'Rolle löschen?'
        },
        'CONFIRM_DELETE_ROLE_MESSAGE': {
            en: 'Are you sure you want to delete this role?',
            de: 'Sind Sie sicher, dass Sie diese Rolle löschen wollen?'
        },
        'CONFIRM_DELETE_SCHEMA_TITLE': {
            en: 'Delete Schema?',
            de: 'Schema löschen?'
        },
        'CONFIRM_DELETE_SCHEMA_MESSAGE': {
            en: 'Are you sure you want to delete this schema?',
            de: 'Sind Sie sicher, dass Sie dieses Schema löschen wollen?'
        }
    };
    var errors = {
        'ERROR': {
            en: 'Error',
            de: 'Fehler'
        },
        'ERR_CHECK_LOGIN_DETAILS': {
            en: 'Please check your login details and try again.',
            de: 'Überprüfen Sie bitte Ihre Daten und versuchen Sie es erneut.'
        },
        'ERR_SESSION_EXPIRED': {
            en: 'Session has expired. Please log in again.',
            de: 'Die Sitzung ist abgelaufen. Bitte melden Sie sich erneut an.'
        },
        'JSON_IS_INVALID': {
            en: 'JSON is invalid',
            de: 'JSON ist ungültig'
        },
        'ERR_SCHEMA_NAME_INVALID': {
            en: 'Schema name may contain only letters, numbers and underscores, and may not start with a number.',
            de: 'Der Schema Name darf nur Burchstaben, Nummern und Unterstriche enthalten und darf nicht mit einer Nummer beginnen.'
        },
        'ERR_SCHEMA_NAME_EMPTY': {
            en: 'Please specify a schema name',
            de: 'Bitte geben Sie einen Schema Name an'
        },
        'ERR_SCHEMA_PLEASE_SPECIFY_DISPLAY_FIELD': {
            en: 'Please specify a displayField',
            de: 'Bitte geben Sie ein Display-Field an'
        },
        'ERR_SCHEMA_PLEASE_SPECIFY_SEGMENT_FIELD': {
            en: 'Please specify a segmentField',
            de: 'Bitte geben Sie ein Segment-Field an'
        },
        'ERR_SCHEMA_MUST_HAVE_AT_LEAST_ONE_FIELD': {
            en: 'Schema must have at least one field defined',
            de: 'Das Schema muss mindestens ein Feld besitzen'
        },
        'ERR_SCHEMA_DISPLAY_FIELD_DOES_NOT_MATCH': {
            en: 'displayField value "{{ value }}" does not match any fields',
            de: 'Der Display-Field-Wert "{{ value }}" entspricht keinem Feld'
        },
        'ERR_SCHEMA_SEGMENT_FIELD_DOES_NOT_MATCH': {
            en: 'segmentField value "{{ value }}" does not match any fields',
            de: 'Der Segment-Field-Wert "{{ value }}" entspricht keinem Feld'
        },
        'ERR_SCHEMA_REQUIRE_NAME_AND_TYPE': {
            en: 'All fields must have a "name" and "type" property',
            de: 'Alle Felder müssen mindestens ein "name"- und "type"-Attribut besitzen.'
        },
        'ERR_SCHEMA_INVALID_TYPES': {
            en: 'The following fields have invalid types: {{ names }}',
            de: 'Die folgenden Felder haben einen ungültigen Typ: {{ names }}'
        },
        'ERR_SCHEMA_DUPLICATE_FIELD_NAMES': {
            en: 'Fields must have unique names - duplicate field detected: {{ names }}',
            de: 'Felder müssen einen eindeutigen Namen besitzen. Doppelt vergebene Felder wurden erkannt: {{ names }}'
        },
        'ERR_SCHEMA_INVALID_LIST_TYPE': {
            en: 'The following list fields have an invalid listType: {{ names }}',
            de: 'Die folgenden List Felder haben einen ungültigen List-Typ: {{ names }}'
        },
        'ERR_SCHEMA_ALLOW_PROPERTY_MISSING': {
            en: 'The following micronode fields must have an "allow" property defined: {{ names }}',
            de: 'Die folgenden Micronode-Felder müssen ein "allow" Attribut angeben: {{ names }}'
        }
    };
    var translations = angular.merge({}, userAuth, projectsArea, editorPane, adminArea, common, dialogs, errors);
    meshAdminUi.TranslationTable = {
        getLanguage: function (code) {
            var table = {};
            for (var key in translations) {
                if (translations[key][code]) {
                    table[key] = translations[key][code];
                }
                else {
                    // specified language not found for this key,
                    // so fall back to the first one.
                    var options = Object.keys(translations[key]);
                    table[key] = translations[key][options[0]];
                }
            }
            return table;
        }
    };
    angular.module('meshAdminUi.common.i18n')
        .constant('translationTable', meshAdminUi.TranslationTable);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * This provider allows language settings to be set up at config time for the app.
     */
    function i18nServiceProvider() {
        var defaultLang = 'en';
        var availableLangs = ['en'];
        function getFn($translate, localStorageService, dispatcher) {
            return new I18nService($translate, localStorageService, dispatcher, availableLangs, defaultLang);
        }
        getFn.$inject = ["$translate", "localStorageService", "dispatcher"];
        this.$get = getFn;
        /**
         * Set the app's default language at config time.
         */
        this.setDefaultLanguage = function (value) {
            if (!isValidIsoLanguage(value)) {
                throw new Error('i18nServiceProvider#setDefaultLanguage: "' + value + '" is not a valid ISO-639-1 code.');
            }
            if (-1 < availableLangs.indexOf(value)) {
                defaultLang = value;
            }
            else {
                throw new Error('i18nServiceProvider#setDefaultLanguage: "' + value + '" is not an available language.');
            }
        };
        /**
         * Specify available languages in the form of an array of the 2-character ISO-639-1 codes,
         * e.g. ['en', 'de', 'fr']
         */
        this.setAvailableLanguages = function (value) {
            if (Array.isArray(value)) {
                value.forEach(function (code) {
                    if (!isValidIsoLanguage(code)) {
                        throw new Error('i18nServiceProvider#setAvailableLanguages: "' + code + '" is not a valid ISO-639-1 code.');
                    }
                });
                availableLangs = value;
            }
            else {
                throw new Error('i18nServiceProvider#setAvailableLanguages: argument must be an array');
            }
        };
    }
    /**
     * Service for setting and retrieving app-wide language settings.
     */
    var I18nService = (function () {
        function I18nService($translate, localStorageService, dispatcher, availableLangs, defaultLang) {
            this.$translate = $translate;
            this.localStorageService = localStorageService;
            this.dispatcher = dispatcher;
            this.availableLangs = availableLangs;
            this.defaultLang = defaultLang;
            this.languageInfoCache = {};
            this.isoLangs = this.getIsoLangs();
            var currentLang = localStorageService.get('lang') || defaultLang;
            this.setCurrentLang(currentLang);
        }
        Object.defineProperty(I18nService.prototype, "languages", {
            get: function () {
                return this.getAvailableLanguages();
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Getter function to get an array of available languages.
         */
        I18nService.prototype.getAvailableLanguages = function () {
            var _this = this;
            return this.availableLangs.map(function (code) { return _this.codeToFullInfo(code, _this.isoLangs); });
        };
        /**
         * Set the 2-character (ISO-639-1) language code, which must be part of the
         * availableLanguages array.
         */
        I18nService.prototype.setCurrentLang = function (value) {
            if (this.validateLanguageCode(value)) {
                this.localStorageService.set('lang', value);
                this.currentLang = value;
                this.$translate.use(value);
                this.dispatcher.publish(this.dispatcher.events.languageChanged, this.getCurrentLang());
            }
        };
        /**
         * Get the language info for the current language.
         */
        I18nService.prototype.getCurrentLang = function () {
            return this.codeToFullInfo(this.currentLang, this.isoLangs);
        };
        /**
         * Returns the default language.
         */
        I18nService.prototype.getDefaultLang = function () {
            return this.codeToFullInfo(this.defaultLang, this.isoLangs);
        };
        I18nService.prototype.getLanguageInfo = function (code) {
            if (this.validateLanguageCode(code)) {
                return this.codeToFullInfo(code, this.isoLangs);
            }
        };
        I18nService.prototype.validateLanguageCode = function (code) {
            if (-1 < this.availableLangs.indexOf(code)) {
                return true;
            }
            else {
                throw new Error('I18nService#setCurrentLang: ' + code + ' is not an available language.');
            }
        };
        /**
         * Given an ISO language code, returns a ILanguageInfo object.
         */
        I18nService.prototype.codeToFullInfo = function (code, isoLangs) {
            if (!this.languageInfoCache.hasOwnProperty(code)) {
                this.languageInfoCache[code] = {
                    code: code,
                    name: isoLangs[code].name,
                    nativeName: isoLangs[code].nativeName
                };
            }
            return this.languageInfoCache[code];
        };
        /**
         * Returns an object has of ISO-639-1 codes and the language's name and
         * name in the language itself:
         *
         * {
         *   'de': {
         *     name: 'German',
         *     nativeName: 'Deutsch'
         *     }
         * }
         *
         * @author Phil Teare using wikipedia data
         */
        I18nService.prototype.getIsoLangs = function () {
            // disable jsHint check for unsafe characters
            /* jshint -W100 */
            return ISO_LANGUAGES;
        };
        return I18nService;
    }());
    meshAdminUi.I18nService = I18nService;
    var ISO_LANGUAGES = {
        "ab": {
            "name": "Abkhaz",
            "nativeName": "аҧсуа"
        },
        "aa": {
            "name": "Afar",
            "nativeName": "Afaraf"
        },
        "af": {
            "name": "Afrikaans",
            "nativeName": "Afrikaans"
        },
        "ak": {
            "name": "Akan",
            "nativeName": "Akan"
        },
        "sq": {
            "name": "Albanian",
            "nativeName": "Shqip"
        },
        "am": {
            "name": "Amharic",
            "nativeName": "አማርኛ"
        },
        "ar": {
            "name": "Arabic",
            "nativeName": "العربية"
        },
        "an": {
            "name": "Aragonese",
            "nativeName": "Aragonés"
        },
        "hy": {
            "name": "Armenian",
            "nativeName": "Հայերեն"
        },
        "as": {
            "name": "Assamese",
            "nativeName": "অসমীয়া"
        },
        "av": {
            "name": "Avaric",
            "nativeName": "авар мацӀ, магӀарул мацӀ"
        },
        "ae": {
            "name": "Avestan",
            "nativeName": "avesta"
        },
        "ay": {
            "name": "Aymara",
            "nativeName": "aymar aru"
        },
        "az": {
            "name": "Azerbaijani",
            "nativeName": "azərbaycan dili"
        },
        "bm": {
            "name": "Bambara",
            "nativeName": "bamanankan"
        },
        "ba": {
            "name": "Bashkir",
            "nativeName": "башҡорт теле"
        },
        "eu": {
            "name": "Basque",
            "nativeName": "euskara, euskera"
        },
        "be": {
            "name": "Belarusian",
            "nativeName": "Беларуская"
        },
        "bn": {
            "name": "Bengali",
            "nativeName": "বাংলা"
        },
        "bh": {
            "name": "Bihari",
            "nativeName": "भोजपुरी"
        },
        "bi": {
            "name": "Bislama",
            "nativeName": "Bislama"
        },
        "bs": {
            "name": "Bosnian",
            "nativeName": "bosanski jezik"
        },
        "br": {
            "name": "Breton",
            "nativeName": "brezhoneg"
        },
        "bg": {
            "name": "Bulgarian",
            "nativeName": "български език"
        },
        "my": {
            "name": "Burmese",
            "nativeName": "ဗမာစာ"
        },
        "ca": {
            "name": "Catalan; Valencian",
            "nativeName": "Català"
        },
        "ch": {
            "name": "Chamorro",
            "nativeName": "Chamoru"
        },
        "ce": {
            "name": "Chechen",
            "nativeName": "нохчийн мотт"
        },
        "ny": {
            "name": "Chichewa; Chewa; Nyanja",
            "nativeName": "chiCheŵa, chinyanja"
        },
        "zh": {
            "name": "Chinese",
            "nativeName": "中文 (Zhōngwén), 汉语, 漢語"
        },
        "cv": {
            "name": "Chuvash",
            "nativeName": "чӑваш чӗлхи"
        },
        "kw": {
            "name": "Cornish",
            "nativeName": "Kernewek"
        },
        "co": {
            "name": "Corsican",
            "nativeName": "corsu, lingua corsa"
        },
        "cr": {
            "name": "Cree",
            "nativeName": "ᓀᐦᐃᔭᐍᐏᐣ"
        },
        "hr": {
            "name": "Croatian",
            "nativeName": "hrvatski"
        },
        "cs": {
            "name": "Czech",
            "nativeName": "česky, čeština"
        },
        "da": {
            "name": "Danish",
            "nativeName": "dansk"
        },
        "dv": {
            "name": "Divehi; Dhivehi; Maldivian;",
            "nativeName": "ދިވެހި"
        },
        "nl": {
            "name": "Dutch",
            "nativeName": "Nederlands, Vlaams"
        },
        "en": {
            "name": "English",
            "nativeName": "English"
        },
        "eo": {
            "name": "Esperanto",
            "nativeName": "Esperanto"
        },
        "et": {
            "name": "Estonian",
            "nativeName": "eesti, eesti keel"
        },
        "ee": {
            "name": "Ewe",
            "nativeName": "Eʋegbe"
        },
        "fo": {
            "name": "Faroese",
            "nativeName": "føroyskt"
        },
        "fj": {
            "name": "Fijian",
            "nativeName": "vosa Vakaviti"
        },
        "fi": {
            "name": "Finnish",
            "nativeName": "suomi, suomen kieli"
        },
        "fr": {
            "name": "French",
            "nativeName": "français, langue française"
        },
        "ff": {
            "name": "Fula; Fulah; Pulaar; Pular",
            "nativeName": "Fulfulde, Pulaar, Pular"
        },
        "gl": {
            "name": "Galician",
            "nativeName": "Galego"
        },
        "ka": {
            "name": "Georgian",
            "nativeName": "ქართული"
        },
        "de": {
            "name": "German",
            "nativeName": "Deutsch"
        },
        "el": {
            "name": "Greek, Modern",
            "nativeName": "Ελληνικά"
        },
        "gn": {
            "name": "Guaraní",
            "nativeName": "Avañeẽ"
        },
        "gu": {
            "name": "Gujarati",
            "nativeName": "ગુજરાતી"
        },
        "ht": {
            "name": "Haitian; Haitian Creole",
            "nativeName": "Kreyòl ayisyen"
        },
        "ha": {
            "name": "Hausa",
            "nativeName": "Hausa, هَوُسَ"
        },
        "he": {
            "name": "Hebrew (modern)",
            "nativeName": "עברית"
        },
        "hz": {
            "name": "Herero",
            "nativeName": "Otjiherero"
        },
        "hi": {
            "name": "Hindi",
            "nativeName": "हिन्दी, हिंदी"
        },
        "ho": {
            "name": "Hiri Motu",
            "nativeName": "Hiri Motu"
        },
        "hu": {
            "name": "Hungarian",
            "nativeName": "Magyar"
        },
        "ia": {
            "name": "Interlingua",
            "nativeName": "Interlingua"
        },
        "id": {
            "name": "Indonesian",
            "nativeName": "Bahasa Indonesia"
        },
        "ie": {
            "name": "Interlingue",
            "nativeName": "Originally called Occidental; then Interlingue after WWII"
        },
        "ga": {
            "name": "Irish",
            "nativeName": "Gaeilge"
        },
        "ig": {
            "name": "Igbo",
            "nativeName": "Asụsụ Igbo"
        },
        "ik": {
            "name": "Inupiaq",
            "nativeName": "Iñupiaq, Iñupiatun"
        },
        "io": {
            "name": "Ido",
            "nativeName": "Ido"
        },
        "is": {
            "name": "Icelandic",
            "nativeName": "Íslenska"
        },
        "it": {
            "name": "Italian",
            "nativeName": "Italiano"
        },
        "iu": {
            "name": "Inuktitut",
            "nativeName": "ᐃᓄᒃᑎᑐᑦ"
        },
        "ja": {
            "name": "Japanese",
            "nativeName": "日本語 (にほんご／にっぽんご)"
        },
        "jv": {
            "name": "Javanese",
            "nativeName": "basa Jawa"
        },
        "kl": {
            "name": "Kalaallisut, Greenlandic",
            "nativeName": "kalaallisut, kalaallit oqaasii"
        },
        "kn": {
            "name": "Kannada",
            "nativeName": "ಕನ್ನಡ"
        },
        "kr": {
            "name": "Kanuri",
            "nativeName": "Kanuri"
        },
        "ks": {
            "name": "Kashmiri",
            "nativeName": "कश्मीरी, كشميري‎"
        },
        "kk": {
            "name": "Kazakh",
            "nativeName": "Қазақ тілі"
        },
        "km": {
            "name": "Khmer",
            "nativeName": "ភាសាខ្មែរ"
        },
        "ki": {
            "name": "Kikuyu, Gikuyu",
            "nativeName": "Gĩkũyũ"
        },
        "rw": {
            "name": "Kinyarwanda",
            "nativeName": "Ikinyarwanda"
        },
        "ky": {
            "name": "Kirghiz, Kyrgyz",
            "nativeName": "кыргыз тили"
        },
        "kv": {
            "name": "Komi",
            "nativeName": "коми кыв"
        },
        "kg": {
            "name": "Kongo",
            "nativeName": "KiKongo"
        },
        "ko": {
            "name": "Korean",
            "nativeName": "한국어 (韓國語), 조선말 (朝鮮語)"
        },
        "ku": {
            "name": "Kurdish",
            "nativeName": "Kurdî, كوردی‎"
        },
        "kj": {
            "name": "Kwanyama, Kuanyama",
            "nativeName": "Kuanyama"
        },
        "la": {
            "name": "Latin",
            "nativeName": "latine, lingua latina"
        },
        "lb": {
            "name": "Luxembourgish, Letzeburgesch",
            "nativeName": "Lëtzebuergesch"
        },
        "lg": {
            "name": "Luganda",
            "nativeName": "Luganda"
        },
        "li": {
            "name": "Limburgish, Limburgan, Limburger",
            "nativeName": "Limburgs"
        },
        "ln": {
            "name": "Lingala",
            "nativeName": "Lingála"
        },
        "lo": {
            "name": "Lao",
            "nativeName": "ພາສາລາວ"
        },
        "lt": {
            "name": "Lithuanian",
            "nativeName": "lietuvių kalba"
        },
        "lu": {
            "name": "Luba-Katanga",
            "nativeName": ""
        },
        "lv": {
            "name": "Latvian",
            "nativeName": "latviešu valoda"
        },
        "gv": {
            "name": "Manx",
            "nativeName": "Gaelg, Gailck"
        },
        "mk": {
            "name": "Macedonian",
            "nativeName": "македонски јазик"
        },
        "mg": {
            "name": "Malagasy",
            "nativeName": "Malagasy fiteny"
        },
        "ms": {
            "name": "Malay",
            "nativeName": "bahasa Melayu, بهاس ملايو‎"
        },
        "ml": {
            "name": "Malayalam",
            "nativeName": "മലയാളം"
        },
        "mt": {
            "name": "Maltese",
            "nativeName": "Malti"
        },
        "mi": {
            "name": "Māori",
            "nativeName": "te reo Māori"
        },
        "mr": {
            "name": "Marathi (Marāṭhī)",
            "nativeName": "मराठी"
        },
        "mh": {
            "name": "Marshallese",
            "nativeName": "Kajin M̧ajeļ"
        },
        "mn": {
            "name": "Mongolian",
            "nativeName": "монгол"
        },
        "na": {
            "name": "Nauru",
            "nativeName": "Ekakairũ Naoero"
        },
        "nv": {
            "name": "Navajo, Navaho",
            "nativeName": "Diné bizaad, Dinékʼehǰí"
        },
        "nb": {
            "name": "Norwegian Bokmål",
            "nativeName": "Norsk bokmål"
        },
        "nd": {
            "name": "North Ndebele",
            "nativeName": "isiNdebele"
        },
        "ne": {
            "name": "Nepali",
            "nativeName": "नेपाली"
        },
        "ng": {
            "name": "Ndonga",
            "nativeName": "Owambo"
        },
        "nn": {
            "name": "Norwegian Nynorsk",
            "nativeName": "Norsk nynorsk"
        },
        "no": {
            "name": "Norwegian",
            "nativeName": "Norsk"
        },
        "ii": {
            "name": "Nuosu",
            "nativeName": "ꆈꌠ꒿ Nuosuhxop"
        },
        "nr": {
            "name": "South Ndebele",
            "nativeName": "isiNdebele"
        },
        "oc": {
            "name": "Occitan",
            "nativeName": "Occitan"
        },
        "oj": {
            "name": "Ojibwe, Ojibwa",
            "nativeName": "ᐊᓂᔑᓈᐯᒧᐎᓐ"
        },
        "cu": {
            "name": "Old Church Slavonic, Church Slavic, Church Slavonic, Old Bulgarian, Old Slavonic",
            "nativeName": "ѩзыкъ словѣньскъ"
        },
        "om": {
            "name": "Oromo",
            "nativeName": "Afaan Oromoo"
        },
        "or": {
            "name": "Oriya",
            "nativeName": "ଓଡ଼ିଆ"
        },
        "os": {
            "name": "Ossetian, Ossetic",
            "nativeName": "ирон æвзаг"
        },
        "pa": {
            "name": "Panjabi, Punjabi",
            "nativeName": "ਪੰਜਾਬੀ, پنجابی‎"
        },
        "pi": {
            "name": "Pāli",
            "nativeName": "पाऴि"
        },
        "fa": {
            "name": "Persian",
            "nativeName": "فارسی"
        },
        "pl": {
            "name": "Polish",
            "nativeName": "polski"
        },
        "ps": {
            "name": "Pashto, Pushto",
            "nativeName": "پښتو"
        },
        "pt": {
            "name": "Portuguese",
            "nativeName": "Português"
        },
        "qu": {
            "name": "Quechua",
            "nativeName": "Runa Simi, Kichwa"
        },
        "rm": {
            "name": "Romansh",
            "nativeName": "rumantsch grischun"
        },
        "rn": {
            "name": "Kirundi",
            "nativeName": "kiRundi"
        },
        "ro": {
            "name": "Romanian, Moldavian, Moldovan",
            "nativeName": "română"
        },
        "ru": {
            "name": "Russian",
            "nativeName": "русский язык"
        },
        "sa": {
            "name": "Sanskrit (Saṁskṛta)",
            "nativeName": "संस्कृतम्"
        },
        "sc": {
            "name": "Sardinian",
            "nativeName": "sardu"
        },
        "sd": {
            "name": "Sindhi",
            "nativeName": "सिन्धी, سنڌي، سندھی‎"
        },
        "se": {
            "name": "Northern Sami",
            "nativeName": "Davvisámegiella"
        },
        "sm": {
            "name": "Samoan",
            "nativeName": "gagana faa Samoa"
        },
        "sg": {
            "name": "Sango",
            "nativeName": "yângâ tî sängö"
        },
        "sr": {
            "name": "Serbian",
            "nativeName": "српски језик"
        },
        "gd": {
            "name": "Scottish Gaelic; Gaelic",
            "nativeName": "Gàidhlig"
        },
        "sn": {
            "name": "Shona",
            "nativeName": "chiShona"
        },
        "si": {
            "name": "Sinhala, Sinhalese",
            "nativeName": "සිංහල"
        },
        "sk": {
            "name": "Slovak",
            "nativeName": "slovenčina"
        },
        "sl": {
            "name": "Slovene",
            "nativeName": "slovenščina"
        },
        "so": {
            "name": "Somali",
            "nativeName": "Soomaaliga, af Soomaali"
        },
        "st": {
            "name": "Southern Sotho",
            "nativeName": "Sesotho"
        },
        "es": {
            "name": "Spanish; Castilian",
            "nativeName": "español, castellano"
        },
        "su": {
            "name": "Sundanese",
            "nativeName": "Basa Sunda"
        },
        "sw": {
            "name": "Swahili",
            "nativeName": "Kiswahili"
        },
        "ss": {
            "name": "Swati",
            "nativeName": "SiSwati"
        },
        "sv": {
            "name": "Swedish",
            "nativeName": "svenska"
        },
        "ta": {
            "name": "Tamil",
            "nativeName": "தமிழ்"
        },
        "te": {
            "name": "Telugu",
            "nativeName": "తెలుగు"
        },
        "tg": {
            "name": "Tajik",
            "nativeName": "тоҷикӣ, toğikī, تاجیکی‎"
        },
        "th": {
            "name": "Thai",
            "nativeName": "ไทย"
        },
        "ti": {
            "name": "Tigrinya",
            "nativeName": "ትግርኛ"
        },
        "bo": {
            "name": "Tibetan Standard, Tibetan, Central",
            "nativeName": "བོད་ཡིག"
        },
        "tk": {
            "name": "Turkmen",
            "nativeName": "Türkmen, Түркмен"
        },
        "tl": {
            "name": "Tagalog",
            "nativeName": "Wikang Tagalog, ᜏᜒᜃᜅ᜔ ᜆᜄᜎᜓᜄ᜔"
        },
        "tn": {
            "name": "Tswana",
            "nativeName": "Setswana"
        },
        "to": {
            "name": "Tonga (Tonga Islands)",
            "nativeName": "faka Tonga"
        },
        "tr": {
            "name": "Turkish",
            "nativeName": "Türkçe"
        },
        "ts": {
            "name": "Tsonga",
            "nativeName": "Xitsonga"
        },
        "tt": {
            "name": "Tatar",
            "nativeName": "татарча, tatarça, تاتارچا‎"
        },
        "tw": {
            "name": "Twi",
            "nativeName": "Twi"
        },
        "ty": {
            "name": "Tahitian",
            "nativeName": "Reo Tahiti"
        },
        "ug": {
            "name": "Uighur, Uyghur",
            "nativeName": "Uyƣurqə, ئۇيغۇرچە‎"
        },
        "uk": {
            "name": "Ukrainian",
            "nativeName": "українська"
        },
        "ur": {
            "name": "Urdu",
            "nativeName": "اردو"
        },
        "uz": {
            "name": "Uzbek",
            "nativeName": "zbek, Ўзбек, أۇزبېك‎"
        },
        "ve": {
            "name": "Venda",
            "nativeName": "Tshivenḓa"
        },
        "vi": {
            "name": "Vietnamese",
            "nativeName": "Tiếng Việt"
        },
        "vo": {
            "name": "Volapük",
            "nativeName": "Volapük"
        },
        "wa": {
            "name": "Walloon",
            "nativeName": "Walon"
        },
        "cy": {
            "name": "Welsh",
            "nativeName": "Cymraeg"
        },
        "wo": {
            "name": "Wolof",
            "nativeName": "Wollof"
        },
        "fy": {
            "name": "Western Frisian",
            "nativeName": "Frysk"
        },
        "xh": {
            "name": "Xhosa",
            "nativeName": "isiXhosa"
        },
        "yi": {
            "name": "Yiddish",
            "nativeName": "ייִדיש"
        },
        "yo": {
            "name": "Yoruba",
            "nativeName": "Yorùbá"
        },
        "za": {
            "name": "Zhuang, Chuang",
            "nativeName": "Saɯ cueŋƅ, Saw cuengh"
        }
    };
    /**
     * Returns true if the code is a valid ISO-639-1 code.
     */
    function isValidIsoLanguage(code) {
        return -1 < Object.keys(ISO_LANGUAGES).indexOf(code);
    }
    angular.module('meshAdminUi.common.i18n')
        .provider('i18nService', i18nServiceProvider);
})(meshAdminUi || (meshAdminUi = {}));

var meshAdminUi;
(function (meshAdminUi) {
    /**
     * A convenience service for whenever translated strings need to be used in
     * code rather than in a view.
     *
     * Rather than injecting $filter and then doing
     * $filter('translate')('TRANSLATED_STRING');
     *
     * we can inject i18n and do
     * i18n('TRANSLATED_STRING');
     */
    function i18nFilter($filter) {
        return $filter('translate');
    }
    i18nFilter.$inject = ["$filter"];
    meshAdminUi.i18nFilter = i18nFilter;
    angular.module('meshAdminUi.common.i18n')
        .factory('i18n', i18nFilter);
})(meshAdminUi || (meshAdminUi = {}));

//# sourceMappingURL=app.js.map
